const SFA_masterModel = require("./../../models/app/SFA_masterModel");
const SFA_masterModelWeb = require("./../../models/web/SFA_masterModel");
const User = require("./../../models/app/userModel");
const errorModel = require("./../../models/app/errorModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
var NodeGeocoder = require('node-geocoder');

var options = {
    provider: 'google',
    // provider: 'openstreetmap',
    httpAdapter: 'https', // Default
    apiKey: process.env.GOOGLE_MAP_APIKEY, // for Mapquest, OpenCage, Google Premier
    formatter: 'json' // 'gpx', 'string', ...
};


exports.getAttendanceTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getAttendanceTypeData = await SFA_masterModel.getAttendanceTypeData(req.body);
        // console.log('getAttendanceTypeData',getAttendanceTypeData);

        res.status(200).json({
            message: "Attendance type data",
            status: "success",
            result: {
                data: getAttendanceTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getAttendanceTodayData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        req.body.attend_date = moment(new Date()).format('YYYY-MM-DD');

        var getTodayAttendanceData = await SFA_masterModel.getTodayAttendanceData(req.body);
        // console.log('getTodayAttendanceData',getTodayAttendanceData);

        if (getTodayAttendanceData.id) {
            return res.status(200).json({
                message: getTodayAttendanceData.message,
                status: "error"
            });
        } else {
            return res.status(200).json({
                message: 'Attendance not added',
                status: "success"
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.addAttendanceData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        req.body.start_time = moment(new Date()).format('YYYY-MM-DD HH:mm:ss');
        req.body.attend_date = moment(new Date()).format('YYYY-MM-DD');

        var getTodayAttendanceData = await SFA_masterModel.getTodayAttendanceData(req.body);
        // console.log('getTodayAttendanceData',getTodayAttendanceData);

        if (getTodayAttendanceData.id) {
            return res.status(200).json({
                message: getTodayAttendanceData.message,
                status: "error"
            });
        }


        // var geocoder = NodeGeocoder(options);

        // // Target latitude and longitude
        // const targetLatitude = 28.6846846; // Replace with your target latitude
        // const targetLongitude = 77.0404084; // Replace with your target longitude

        // // Function to calculate the distance between two points using Haversine formula
        // function calculateDistance(lat1, lon1, lat2, lon2) {
        //     const R = 6371; // Radius of the earth in kilometers
        //     const dLat = (lat2 - lat1) * (Math.PI / 180);
        //     const dLon = (lon2 - lon1) * (Math.PI / 180);
        //     const a =
        //     Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        //     Math.cos(lat1 * (Math.PI / 180)) *
        //     Math.cos(lat2 * (Math.PI / 180)) *
        //     Math.sin(dLon / 2) *
        //     Math.sin(dLon / 2);
        //     const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        //     const distance = R * c; // Distance in kilometers
        //     return distance;
        // }

        // // Generate a list of nearby coordinates (you can adjust the distance as needed)
        // const numberOfNearbyPoints = 5;
        // const nearbyCoordinates = [];
        // const distanceStep = 0.0001; // Adjust this value to increase or decrease distance between points

        // for (let i = 0; i < numberOfNearbyPoints; i++) {
        //     const lat = targetLatitude + i * distanceStep;
        //     const lon = targetLongitude + i * distanceStep;
        //     nearbyCoordinates.push({ lat, lon });
        // }

        // // Find the most nearby address among the generated coordinates
        // let closestAddress = null;
        // let closestDistance = 0.01;

        // async function findClosestAddress() {
        //     console.log('nearbyCoordinates',nearbyCoordinates);
        //     for (const coord of nearbyCoordinates) {
        //         const { lat, lon } = coord;
        //         console.log('latitude', lat, 'longitude', lon);
        //         const res = await geocoder.reverse({ lat: lat, lon: lon });
        //         console.log('res ---->',res);
        //         if (res.length > 0) {
        //             const address = res[0].formattedAddress;
        //             const distance = calculateDistance(
        //                 targetLatitude,
        //                 targetLongitude,
        //                 lat,
        //                 lon
        //                 );
        //                 console.log('closestDistance',closestDistance);
        //                 console.log('distance',distance);
        //                 if (distance < closestDistance) {
        //                     closestDistance = distance;
        //                     closestAddress = address;
        //                 }
        //             }
        //         }
        //         console.log('Most Nearby Address:', closestAddress);
        //     }

        //     var testData  = await findClosestAddress().catch((error) => {
        //         console.log('test ==>', JSON.stringify(error));
        //         console.error('Error:', error.message || error);
        //     });

        //     console.log('testData', testData);


        var geocoder = NodeGeocoder(options);
        await geocoder.reverse({ lat: req.body.start_lat, lon: req.body.start_lng }, function (err, res) {
            // console.log('res ---->', res);
            if (res.length) {
                req.body.start_address = res[0].formattedAddress
            }
        });

        var addAttendanceData = await SFA_masterModel.addAttendanceData(req.body);
        console.log('addAttendanceData', addAttendanceData);

        if (addAttendanceData.insertId) {

            var trackUserLocationData = {
                sfa_app_user_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                lat: req.body.start_lat ? req.body.start_lat : '',
                lng: req.body.start_lng ? req.body.start_lng : '',
                address: req.body.start_address ? req.body.start_address : '',
                attendance_id: addAttendanceData.insertId ? addAttendanceData.insertId : 0,
                track_type: 'Attendance Start'
            }

            var trackUserLocation = await SFA_masterModel.trackUserLocation(trackUserLocationData);
            // console.log('trackUserLocation',trackUserLocation);

            await db.commit();
            res.status(200).json({
                message: "Attendance data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Attendance data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };

        var saveErrorLog = await errorModel.saveErrorLogData({ 'fn_name': 'addAttendanceData', 'error_from': 'app', 'error_log': JSON.stringify(e) });
        console.log('saveErrorLog', saveErrorLog);

        // res.status(200).json({
        //     message: "Error Occured",
        //     status: "error",
        //     err :err
        // });
        res.status(200).json({
            message: "Attendance data added",
            status: "success"
        });
        return false;
    }
};

function getDistanceFromLatLonInKm(lat1, lon1, lat2, lon2) {
    var p = 0.017453292519943295;    // Math.PI / 180
    var c = Math.cos;
    var a = 0.5 - c((lat2 - lat1) * p) / 2 +
        c(lat1 * p) * c(lat2 * p) *
        (1 - c((lon2 - lon1) * p)) / 2;
    var km = 12742 * Math.asin(Math.sqrt(a)); // 2 * R; R = 6371 km
    return km ? km.toFixed(2) : 0;
}


exports.updateAttendanceData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        var getPendingCheckinList = await SFA_masterModel.getPendingCheckinList(req.body);
        // console.log('getPendingCheckinList',getPendingCheckinList);

        if (getPendingCheckinList.length) {
            return res.status(200).json({
                message: "Stop checkin first",
                status: "error"
            });
        }


        var getDetailAttendanceData = await SFA_masterModel.getDetailAttendanceData(req.body);
        console.log('getDetailAttendanceData', getDetailAttendanceData);

        if (getDetailAttendanceData.id) {
            req.body.start_time = getDetailAttendanceData.start_time;
            req.body.start_lat = getDetailAttendanceData.start_lat;
            req.body.start_lng = getDetailAttendanceData.start_lng;
        }

        req.body.stop_time = moment(new Date()).format('YYYY-MM-DD HH:mm:ss');
        req.body.start_time = moment(req.body.start_time).format('YYYY-MM-DD HH:mm:ss');
        // start time and end time
        var a = moment(req.body.stop_time);
        var b = moment(req.body.start_time);
        // req.body.total_time = parseInt(a.diff(b, 'hours')) + ' hour and ' + parseInt(a.diff(b, 'minutes')) % 60 + ' minutes.';
        req.body.total_time = parseInt(a.diff(b, 'hours')) + ' hour ' + parseInt(a.diff(b, 'minutes')) % 60 + ' min.';


        var geocoder = NodeGeocoder(options);
        await geocoder.reverse({ lat: req.body.stop_lat, lon: req.body.stop_lng }, function (err, res) {
            // console.log(res);
            if (res.length) {
                req.body.stop_address = res[0].formattedAddress
            }
        });

        var getGoogleKmCheckinSum = await SFA_masterModel.getGoogleKmCheckinSum(req.body);
        console.log('getGoogleKmCheckinSum', getGoogleKmCheckinSum);

        req.body.checkin_date = moment(new Date()).format('YYYY-MM-DD');
        var getTodayLastCheckinData = await SFA_masterModel.getTodayLastCheckinData({ filter: { 'checkin_date': req.body.checkin_date, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds }, 'sfa_app_user_id': req.body.sfa_app_user_id });
        console.log('getTodayLastCheckinData', getTodayLastCheckinData);

        // here we take last checkin as start lat to get google km between last stop address and attendance stop        
        if (getTodayLastCheckinData.length > 0) {
            req.body.start_lat = getTodayLastCheckinData[0].stop_lat ? getTodayLastCheckinData[0].stop_lat : '';
            req.body.start_lng = getTodayLastCheckinData[0].stop_lng ? getTodayLastCheckinData[0].stop_lng : '';
        }

        // console.log('body ==>', req.body);
        req.body.google_km = getDistanceFromLatLonInKm(req.body.start_lat, req.body.start_lng, req.body.stop_lat, req.body.stop_lng)
        // console.log('req.body.google_km  ==>',req.body.google_km );
        req.body.google_km = parseFloat(req.body.google_km) + parseFloat(getGoogleKmCheckinSum);

        var updateAttendanceData = await SFA_masterModel.updateAttendanceData(req.body);
        console.log('updateAttendanceData', updateAttendanceData);

        if (updateAttendanceData.affectedRows) {

            var trackUserLocationData = {
                sfa_app_user_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                lat: req.body.stop_lat ? req.body.stop_lat : '',
                lng: req.body.stop_lng ? req.body.stop_lng : '',
                address: req.body.stop_address ? req.body.stop_address : '',
                attendance_id: req.body.attendance_id ? req.body.attendance_id : 0,
                track_type: 'Attendance Stop'
            }

            var trackUserLocation = await SFA_masterModel.trackUserLocation(trackUserLocationData);
            // console.log('trackUserLocation',trackUserLocation);

            await db.commit();
            res.status(200).json({
                message: "Attendance data updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Attendance data not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getAttendanceData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        // console.log('body ==>', req.body);
        var startAttendDateMonth = '1';
        var getAttendanceListData = [];
        var attendYearMonthArray = []
        var attendYearMonthDisplayArray = []
        for (let index = startAttendDateMonth; index <= moment(new Date()).format('M'); index++) {
            index = (index < 10) ? '0' + index.toString() : index.toString();
            // console.log('startAttendDateMonth', index);  
            attendYearMonthArray.push(moment(new Date()).format('YYYY') + '-' + index);
        }

        req.body.attend_month = req.body.attend_month ? req.body.attend_month : moment(new Date()).format('YYYY') + '-' + moment(new Date()).format('MM')
        var getAttendanceListData = [];
        getAttendanceListData = await SFA_masterModel.getAttendanceListData(req.body);
        // console.log('getAttendanceListData',getAttendanceListData);

        if (attendYearMonthArray.length) {
            attendYearMonthArray.forEach(element => {
                attendYearMonthDisplayArray.push(moment(element).format('MMM-Y'));
                console.log(moment(element).format('MMM-Y'));
            });
        }

        res.status(200).json({
            message: "Attendance List Data",
            status: "success",
            result: {
                data: getAttendanceListData,
                attendYearMonthData: attendYearMonthArray,
                attendYearMonthDisplayArray: attendYearMonthDisplayArray
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getBeatPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        // console.log('body ==>', req.body);
        var startAttendDateMonth = '1';
        var getBeatPlanListData = [];
        var beatPlanYearMonthArray = []
        var beatPlanYearMonthDisplayArray = []
        for (let index = startAttendDateMonth; index <= moment(new Date()).format('M'); index++) {
            index = (index < 10) ? '0' + index.toString() : index.toString();
            // console.log('startAttendDateMonth', index);  
            beatPlanYearMonthArray.push(moment(new Date()).format('YYYY') + '-' + index);
        }

        req.body.beat_month = req.body.beat_month ? req.body.beat_month : moment(new Date()).format('YYYY') + '-' + moment(new Date()).format('MM')
        var getBeatPlanListData = [];
        getBeatPlanListData = await SFA_masterModel.getBeatPlanListData(req.body);
        // console.log('getBeatPlanListData',getBeatPlanListData);

        if (beatPlanYearMonthArray.length) {
            beatPlanYearMonthArray.forEach(element => {
                beatPlanYearMonthDisplayArray.push(moment(element).format('MMM-Y'));
                console.log(moment(element).format('MMM-Y'));
            });
        }

        res.status(200).json({
            message: "Beat Plan List Data",
            status: "success",
            result: {
                data: getBeatPlanListData,
                beatPlanYearMonthData: beatPlanYearMonthArray,
                beatPlanYearMonthDisplayArray: beatPlanYearMonthDisplayArray
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getLeaveDataCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Pending' },
            { 'name': 'Approved' },
            { 'name': 'Rejected' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getLeaveListCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds, 'filter': { 'status': element.name, 'team': req.body.filter?.team } });
                // console.log('element.count',element.count);
                if (i == 0) {
                    await db.commit();
                    res.status(200).json({
                        message: "Leave List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            await db.commit();
            res.status(200).json({
                message: "Leave List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getLeaveData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var getLeaveListCount = await SFA_masterModel.getLeaveListCount(req.body);
        console.log('getLeaveListCount', getLeaveListCount);
        var getLeaveListData = [];
        if (getLeaveListCount > 0) {
            getLeaveListData = await SFA_masterModel.getLeaveListData(req.body);
            // console.log('getLeaveListData',getLeaveListData);
        }

        res.status(200).json({
            message: "Leave List Data",
            status: "success",
            isHead: req.body.sfa_app_user_isBranchHead,
            result: {
                data: getLeaveListData,
                count: getLeaveListCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getLeaveTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getLeaveTypeData = await SFA_masterModel.getLeaveTypeData(req.body);
        // console.log('getLeaveTypeData',getLeaveTypeData);

        res.status(200).json({
            message: "Leave type data",
            status: "success",
            result: {
                data: getLeaveTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addLeaveData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        console.log('body ==>', req.body);

        var addLeaveData = await SFA_masterModel.addLeaveData(req.body);
        console.log('addLeaveData', addLeaveData);

        if (addLeaveData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Leave data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Leave data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.updateStatusLeaveData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateStatusLeaveData = await SFA_masterModel.updateStatusLeaveData(req.body);
        console.log('updateStatusLeaveData', updateStatusLeaveData);

        if (updateStatusLeaveData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Leave Status Updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Leave status not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteLeaveData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteLeaveData = await SFA_masterModel.deleteLeaveData(req.body);
        console.log('deleteLeaveData', deleteLeaveData);

        if (deleteLeaveData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Leave deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Leave not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.getCheckinData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var getCheckinListCount = await SFA_masterModel.getCheckinListCount(req.body);
        console.log('getCheckinListCount', getCheckinListCount);
        var getCheckinListData = [];
        if (getCheckinListCount > 0) {
            getCheckinListData = await SFA_masterModel.getCheckinListData(req.body);
            // console.log('getCheckinListData',getCheckinListData);
        }

        var getPendingCheckinList = await SFA_masterModel.getPendingCheckinList(req.body);
        // console.log('getPendingCheckinList',getPendingCheckinList);

        res.status(200).json({
            message: "Checkin List Data",
            status: "success",
            isHead: req.body.sfa_app_user_isBranchHead,
            result: {
                data: getCheckinListData,
                count: getCheckinListCount,
                pendingCheckIn: getPendingCheckinList
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getcheckinTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getcheckinTypeData = await SFA_masterModel.getcheckinTypeData(req.body);
        // console.log('getcheckinTypeData',getcheckinTypeData);

        res.status(200).json({
            message: "Checkin type data",
            status: "success",
            result: {
                data: getcheckinTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getLeadListCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Pending' },
            { 'name': 'Verified' },
            // {'name' : 'Win'},
            { 'name': 'Lost' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {

                element.count = await SFA_masterModel.getDistributorNetworkCount({ 'filter': { 'status': element.name, 'cust_type_id': req.body.filter.cust_type_id, 'is_lead': req.body.filter.is_lead }, 'sfa_app_user_id': req.body.sfa_app_user_id, 'sfa_app_user_assignedDrIds': req.body.sfa_app_user_assignedDrIds });
                // console.log('element.count',element.count);
                if (i == 0) {
                    res.status(200).json({
                        message: "Lead List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            res.status(200).json({
                message: "Followup List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }

};


exports.getDistributorNetworkList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        var getDistributorNetworkCount = await SFA_masterModel.getDistributorNetworkCount(req.body)

        var getDistributorNetworkList = [];
        if (getDistributorNetworkCount > 0) {
            getDistributorNetworkList = await SFA_masterModel.getDistributorNetworkList(req.body);
            // console.log('getDistributorNetworkList',getDistributorNetworkList);
        }

        res.status(200).json({
            message: "Network List",
            status: "success",
            result: {
                data: getDistributorNetworkList,
                count: getDistributorNetworkCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getAssignedRetailerNetworkListCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Pending' },
            { 'name': 'Verified' },
            // {'name' : 'Win'},
            { 'name': 'Lost' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {

                element.count = await SFA_masterModel.getAssignedRetailerNetworkListCount({ 'filter': { 'status': element.name},'appType': req.body.appType, 'sfa_app_user_id': req.body.sfa_app_user_id});
                // console.log('element.count',element.count);
                if (i == 0) {
                    res.status(200).json({
                        message: "Assigned Retailer List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            res.status(200).json({
                message: "Followup List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }

};

exports.getAssignedRetailerNetworkList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {

        var getAssignedRetailerNetworkList = await SFA_masterModel.getAssignedRetailerNetworkList(req.body);
        // console.log('getAssignedRetailerNetworkList',getAssignedRetailerNetworkList);


        res.status(200).json({
            message: "Assigned Retailers List",
            status: "success",
            result: {
                data: getAssignedRetailerNetworkList,
           
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getDistributionNetworkDetail = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // req.body.customer_id = decrypt(req.body.customer_id);
        req.body.image_app_path = res.locals.image_app_path;

        var getBasicDetail = await SFA_masterModel.getDistributionNetworkBasicDetail(req.body);
        // console.log('getBasicDetail',getBasicDetail);

        var counterData = {
            orderCount: await SFA_masterModel.getOrderDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'filter': { 'order_status': 'All', 'customer_id': req.body.customer_id } }),
            followupCount: await SFA_masterModel.getFollowupListCount({ 'filter': { 'status': 'All', 'customer_id': req.body.customer_id }, 'sfa_app_user_id': req.body.sfa_app_user_id }),
            checkinCount: await SFA_masterModel.getCheckinListCount({ 'filter': { 'status': 'All', 'customer_id': req.body.customer_id }, 'sfa_app_user_id': req.body.sfa_app_user_id }),
            remarkCount: await SFA_masterModel.getDistributionNetworkLogCount({ 'filter': { 'log_type': 'Remark Added', 'customer_id': req.body.customer_id }, 'sfa_app_user_id': req.body.sfa_app_user_id, 'customer_id': req.body.customer_id })
        }

        var getContactDetail = await SFA_masterModel.getDistributionNetworkContactDetail(req.body);
        // console.log('getContactDetail',getContactDetail);

        var getDealerDetail = await SFA_masterModel.getDistributionNetworkDealerDetail(req.body);
        // console.log('getDistributionNetworkDealerDetail',getDistributionNetworkDealerDetail);

        var getDocumentDetail = await SFA_masterModel.getDistributionNetworkDocumentDetail(req.body);
        // console.log('getDocumentDetail',getDocumentDetail);


        var getAssignedUserDetail = await SFA_masterModel.getAssignedUserDetail(req.body);
        // console.log('getAssignedUserDetail',getAssignedUserDetail);

        var getAssignedSalesUserDetail = await SFA_masterModel.getAssignedSalesUserDetail(req.body);
        // console.log('getAssignedSalesUserDetail',getAssignedSalesUserDetail);


        var getDealerPartnerDetail = await SFA_masterModel.getDealerPartnerDetail(req.body);
        // console.log('getDealerPartnerDetail',getDealerPartnerDetail);


        req.body.limit = 1000;
        req.body.start = 0;
        req.body.filter = {
            log_type: 'Remark Added',
            customer_id: req.body.customer_id
        }

        var getLogDetail = await SFA_masterModel.getDistributionNetworkLogDetail(req.body);
        // console.log('getLogDetail',getLogDetail);

        var getDrCheckinListData = await SFA_masterModel.getCheckinListData(req.body);
        // console.log('getCheckinListData',getCheckinListData);

        var getDrFollowupListData = await SFA_masterModel.getFollowupListData(req.body);
        // console.log('getFollowupListData',getFollowupListData);


        var getDrOrderDataList = await SFA_masterModel.getOrderDataList(req.body);
        // console.log('getDrOrderDataList',getDrOrderDataList);


        res.status(200).json({
            message: "Network detail",
            status: "success",
            data: {
                basicDetail: getBasicDetail,
                contactDetail: getContactDetail,
                assignedUserDetail: getAssignedUserDetail,
                documentDetail: getDocumentDetail,
                dealerDetail: getDealerDetail,
                salesUserDetail: getAssignedSalesUserDetail,
                partnerDetail: getDealerPartnerDetail,
                logDetail: getLogDetail,
                counterData: counterData,
                orderList: getDrOrderDataList,
                followupList: getDrFollowupListData,
                checkinList: getDrCheckinListData
            }
        });


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addDistributionNetworkLogs = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // req.body.customer_id = decrypt(req.body.customer_id);

        var logData = {
            customer_id: req.body.customer_id,
            type: 'Remark Added',
            description: req.body.remark,
            created_by: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
            created_by_type: req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '',
            created_by_name: req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '',
        }

        var distributionNetworkLogData = await SFA_masterModel.distributionNetworkLogData(logData);
        // console.log('distributionNetworkLogData',distributionNetworkLogData);

        if (distributionNetworkLogData.insertId) {

            await db.commit();
            res.status(200).json({
                message: "Logs added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Logs not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if (e.code == 'ER_DUP_ENTRY') {
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>', myArray[1])
            var key = myArray[1];
            console.log('key', key);
            key = key.replace(/'/g, "");
            console.log('key', key);
            if (key == 'tg_customer.tg_customer_mobile_UN') {
                errMessage = 'Duplicate entry for mobile number - ' + req.body.basicDetail.mobile;
            } else if (key == 'tg_customer.tg_customer_UN') {
                errMessage = 'Duplicate entry for email - ' + req.body.basicDetail.email;
            } else {
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage', errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getMasterExpenseTypeList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var getMasterExpenseTypeList = await SFA_masterModel.getMasterExpenseTypeList(req.body);
        // console.log('getMasterExpenseTypeList',getMasterExpenseTypeList);

        var data = {
            type: [
                { label: 'Local Conveyance', value: 'Local Conveyance' },
                { label: 'Outstation Travel', value: 'Outstation Travel' },
                { label: 'Other', value: 'Other' }
                // {label:'Sales Promotion Activity',value:'Sales Promotion Activity'},
                // {label:'Outstation Travel',value:'Outstation Travel'},
                // {label:'Misc Exp.',value:'Misc Exp.'},
                // {label:'Local Conveyance',value:'Local Conveyance'},
            ],
            travel_mode: [
                { label: 'Car', value: 'Car' },
                { label: 'Bike', value: 'Bike' },
            ],
            conveyance_travel_mode: [
                { label: 'Car', value: 'Car' },
                { label: 'Bike', value: 'Bike' },
            ],
            nature_of_expense: [
                { label: 'Car', value: 'Car' },
                { label: 'Bike', value: 'Bike' },
            ]
        };

        res.status(200).json({
            message: "Master Expense Type List",
            status: "success",
            result: {
                data: data,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addExpenseData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var addExpenseData = await SFA_masterModel.addExpenseData(req.body);
        console.log('addExpenseData', addExpenseData);

        if (addExpenseData.insertId) {

            if (req.body.misc_array?.length) {

                for (let index = 0; index < req.body.misc_array.length; index++) {
                    const element = req.body.misc_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'MISC';
                    element.sfa_app_user_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type = req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name = req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';

                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData', addExpenseDetailData);
                }
            }

            if (req.body.local_conveyance_array?.length) {
                for (let index = 0; index < req.body.local_conveyance_array.length; index++) {
                    const element = req.body.local_conveyance_array[index];
                    element.expense_type = 'Local Conveyance';
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.sfa_app_user_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type = req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name = req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';

                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData', addExpenseDetailData);
                }
            }

            if (req.body.food_array?.length) {
                for (let index = 0; index < req.body.food_array.length; index++) {
                    const element = req.body.food_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Food';
                    element.sfa_app_user_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type = req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name = req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';

                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData', addExpenseDetailData);
                }
            }

            if (req.body.hotel_array?.length) {
                for (let index = 0; index < req.body.hotel_array.length; index++) {
                    const element = req.body.hotel_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Hotel';
                    element.sfa_app_user_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type = req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name = req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';

                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData', addExpenseDetailData);
                }
            }


            if (req.body.travel_entitlement_array?.length) {
                for (let index = 0; index < req.body.travel_entitlement_array.length; index++) {
                    const element = req.body.travel_entitlement_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Travel Entitlement';
                    element.sfa_app_user_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type = req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name = req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';

                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData', addExpenseDetailData);
                }
            }

            await db.commit();
            res.status(200).json({
                message: "Expense data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Expense data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.updateStatusExpenseData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateStatusExpenseData = await SFA_masterModel.updateStatusExpenseData(req.body);
        console.log('updateStatusExpenseData', updateStatusExpenseData);

        if (updateStatusExpenseData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Expense Status Updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Expense status not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getExpenseDataCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Pending' },
            { 'name': 'Approved' },
            { 'name': 'Rejected' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {

                element.count = await SFA_masterModel.getExpenseDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds, 'filter': { 'status': element.name } });
                // console.log('element.count',element.count);
                if (i == 0) {
                    await db.commit();
                    res.status(200).json({
                        message: "Expense List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            await db.commit();
            res.status(200).json({
                message: "Order List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getExpenseDataList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        var getExpenseDataCount = await SFA_masterModel.getExpenseDataCount(req.body)

        var getExpenseDataList = [];
        if (getExpenseDataCount > 0) {
            getExpenseDataList = await SFA_masterModel.getExpenseDataList(req.body);
            // console.log('getExpenseDataList',getExpenseDataList);
        }

        res.status(200).json({
            message: "Expense List",
            status: "success",
            isHead: req.body.sfa_app_user_isBranchHead,
            result: {
                data: getExpenseDataList,
                count: getExpenseDataCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getExpenseDataDetail = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        var getExpenseData = await SFA_masterModel.getExpenseData(req.body)

        var getExpenseDataDetail = await SFA_masterModel.getExpenseDataDetail(req.body);
        // console.log('getExpenseDataList',getExpenseDataList);

        res.status(200).json({
            message: "Expense Detail",
            status: "success",
            result: {
                expenseData: getExpenseData,
                expenseDataDetail: getExpenseDataDetail
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.addCheckinData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getPendingCheckinList = await SFA_masterModel.getPendingCheckinList(req.body);
        // console.log('getPendingCheckinList',getPendingCheckinList);

        if (getPendingCheckinList.length) {
            return res.status(200).json({
                message: "Stop checkin first",
                status: "error"
            });
        }

        req.body.start_time = moment(new Date()).format('YYYY-MM-DD HH:mm:ss');
        req.body.checkin_date = moment(new Date()).format('YYYY-MM-DD');

        var geocoder = NodeGeocoder(options);
        await geocoder.reverse({ lat: req.body.start_lat, lon: req.body.start_lng }, function (err, res) {
            // console.log(res);
            if (res.length) {
                req.body.start_address = res[0].formattedAddress
            }
        });

        var getTodayLastCheckinData = await SFA_masterModel.getTodayLastCheckinData({ filter: { 'checkin_date': req.body.checkin_date, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds }, 'sfa_app_user_id': req.body.sfa_app_user_id });
        console.log('getTodayLastCheckinData', getTodayLastCheckinData);

        // return false;
        if (getTodayLastCheckinData.length > 0) {
            req.body.last_lat = getTodayLastCheckinData[0].stop_lat ? getTodayLastCheckinData[0].stop_lat : '';
            req.body.last_lng = getTodayLastCheckinData[0].stop_lng ? getTodayLastCheckinData[0].stop_lng : '';
            req.body.last_address = getTodayLastCheckinData[0].stop_address ? getTodayLastCheckinData[0].stop_address : '';
        } else {
            req.body.attend_date = moment(new Date()).format('YYYY-MM-DD');
            var getTodayAttendanceData = await SFA_masterModel.getTodayAttendanceData(req.body);
            console.log('getTodayAttendanceData', getTodayAttendanceData);
            if (getTodayAttendanceData.id) {
                req.body.last_lat = getTodayAttendanceData.start_lat ? getTodayAttendanceData.start_lat : '';
                req.body.last_lng = getTodayAttendanceData.start_lng ? getTodayAttendanceData.start_lng : '';
                req.body.last_address = getTodayAttendanceData.start_address ? getTodayAttendanceData.start_address : '';
            }
        }

        req.body.start_google_km = getDistanceFromLatLonInKm(req.body.last_lat, req.body.last_lng, req.body.start_lat, req.body.start_lng)

        console.log('req.body.start_google_km ==>', req.body.start_google_km);


        var addCheckinData = await SFA_masterModel.addCheckinData(req.body);
        console.log('addCheckinData', addCheckinData);

        if (addCheckinData.insertId) {

            var trackUserLocationData = {
                sfa_app_user_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                lat: req.body.start_lat ? req.body.start_lat : '',
                lng: req.body.start_lng ? req.body.start_lng : '',
                address: req.body.start_address ? req.body.start_address : '',
                checkin_id: addCheckinData.insertId ? addCheckinData.insertId : 0,
                track_type: 'Checkin Start'
            }

            var trackUserLocation = await SFA_masterModel.trackUserLocation(trackUserLocationData);
            // console.log('trackUserLocation',trackUserLocation);

            await db.commit();
            res.status(200).json({
                message: "Checkin data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Checkin data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };

        var saveErrorLog = await errorModel.saveErrorLogData({ 'fn_name': 'addCheckinData', 'error_from': 'app', 'error_log': JSON.stringify(e) });
        console.log('saveErrorLog', saveErrorLog);

        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateCheckinData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        if (req.body.checkinInfo.id) {
            req.body.start_time = req.body.checkinInfo.start_time;
            req.body.checkin_date = req.body.checkinInfo.checkin_date;
        }

        // console.log('body ==>', req.body);
        req.body.stop_time = moment(new Date()).format('YYYY-MM-DD HH:mm:ss');
        req.body.start_time = moment(req.body.start_time).format('YYYY-MM-DD HH:mm:ss');
        // start time and end time

        var a = moment(req.body.stop_time);
        var b = moment(req.body.start_time);
        // req.body.total_time = parseInt(a.diff(b, 'hours')) + ' hour and ' + parseInt(a.diff(b, 'minutes')) % 60 + ' minutes.';
        req.body.total_time = parseInt(a.diff(b, 'hours')) + ' hour ' + parseInt(a.diff(b, 'minutes')) % 60 + ' min.';

        var geocoder = NodeGeocoder(options);
        await geocoder.reverse({ lat: req.body.stop_lat, lon: req.body.stop_lng }, function (err, res) {
            console.log(res);
            if (res.length) {
                req.body.stop_address = res[0].formattedAddress
            }
        });


        var getTodayLastCheckinData = await SFA_masterModel.getTodayLastCheckinData({ filter: { 'checkin_date': req.body.checkin_date, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds }, 'sfa_app_user_id': req.body.sfa_app_user_id });
        console.log('getTodayLastCheckinData', getTodayLastCheckinData);

        // return false;
        if (getTodayLastCheckinData.length > 0) {
            req.body.last_lat = getTodayLastCheckinData[0].stop_lat ? getTodayLastCheckinData[0].stop_lat : '';
            req.body.last_lng = getTodayLastCheckinData[0].stop_lng ? getTodayLastCheckinData[0].stop_lng : '';
            req.body.last_address = getTodayLastCheckinData[0].stop_address ? getTodayLastCheckinData[0].stop_address : '';
        } else {
            req.body.attend_date = moment(new Date()).format('YYYY-MM-DD');
            var getTodayAttendanceData = await SFA_masterModel.getTodayAttendanceData(req.body);
            console.log('getTodayAttendanceData', getTodayAttendanceData);
            if (getTodayAttendanceData.id) {
                req.body.last_lat = getTodayAttendanceData.start_lat ? getTodayAttendanceData.start_lat : '';
                req.body.last_lng = getTodayAttendanceData.start_lng ? getTodayAttendanceData.start_lng : '';
                req.body.last_address = getTodayAttendanceData.start_address ? getTodayAttendanceData.start_address : '';
            }
        }

        req.body.google_km = getDistanceFromLatLonInKm(req.body.last_lat, req.body.last_lng, req.body.stop_lat, req.body.stop_lng)
        req.body.stop_google_km = req.body.google_km;

        console.log('req.body.google_km ==>', req.body.google_km);

        var updateCheckinData = await SFA_masterModel.updateCheckinData(req.body);
        console.log('updateCheckinData', updateCheckinData);


        if (req.body.document.length) {
            req.body.checkin_id = req.body.checkinInfo.id;
            var insertCheckinDocument = await SFA_masterModel.insertCheckinDocument(req.body);
            // console.log('insertCheckinDocument',insertCheckinDocument);
        }


        if (updateCheckinData.affectedRows) {

            var trackUserLocationData = {
                sfa_app_user_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                lat: req.body.stop_lat ? req.body.stop_lat : '',
                lng: req.body.stop_lng ? req.body.stop_lng : '',
                address: req.body.stop_address ? req.body.stop_address : '',
                checkin_id: req.body.checkin_id ? req.body.checkin_id : 0,
                track_type: 'Checkin Stop'
            }

            var trackUserLocation = await SFA_masterModel.trackUserLocation(trackUserLocationData);
            // console.log('trackUserLocation',trackUserLocation);


            await db.commit();
            res.status(200).json({
                message: "Checkin data updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Checkin data not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };


        var saveErrorLog = await errorModel.saveErrorLogData({ 'fn_name': 'updateCheckinData', 'error_from': 'app', 'error_log': JSON.stringify(e) });
        console.log('saveErrorLog', saveErrorLog);

        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.detailCheckinData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var detailCheckinData = await SFA_masterModel.detailCheckinData(req.body);
        console.log('detailCheckinData', detailCheckinData);

        await db.commit();
        res.status(200).json({
            message: "Checkin data detail",
            status: "success",
            data: {
                detail: detailCheckinData
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getFollowupListCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var statusArrayData = [
            { 'name': 'Pending' },
            { 'name': 'Upcoming' },
            { 'name': 'Completed' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getFollowupListCount({ 'filter': { 'status': element.name, 'followup_date': req.body.filter.followup_date, 'team': req.body.filter.team, 'date_from': req.body.filter.date_from, 'date_to': req.body.filter.date_to }, 'sfa_app_user_id': req.body.sfa_app_user_id, 'sfa_app_user_assignedUserIds': req.body.sfa_app_user_assignedUserIds });
                // console.log('element.count',element.count);
                if (i == 0) {
                    res.status(200).json({
                        message: "Followup List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            res.status(200).json({
                message: "Followup List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }

};

exports.getFollowupData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var getFollowupListCount = await SFA_masterModel.getFollowupListCount(req.body);
        // console.log('getFollowupListCount',getFollowupListCount);
        var getFollowupListData = [];
        if (getFollowupListCount > 0) {
            getFollowupListData = await SFA_masterModel.getFollowupListData(req.body);
            // console.log('getFollowupListData',getFollowupListData);
        }

        res.status(200).json({
            message: "Followup List Data",
            status: "success",
            isHead: req.body.sfa_app_user_isBranchHead,
            result: {
                data: getFollowupListData,
                count: getFollowupListCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getFollowupStatusList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getFollowupStatusList = await SFA_masterModel.getFollowupStatusList(req.body);
        // console.log('getFollowupStatusList',getFollowupStatusList);

        res.status(200).json({
            message: "Followup Status List",
            status: "success",
            result: {
                data: getFollowupStatusList,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.addFollowUpData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var addFollowUpData = await SFA_masterModel.addFollowUpData(req.body);
        console.log('addFollowUpData', addFollowUpData);

        if (addFollowUpData.insertId) {

            if (req.body.checkin_id) {
                req.body.followup_id = addFollowUpData.insertId ? addFollowUpData.insertId : 0;
                req.body.followup_added = 1
                req.body.update_type = 'followup';
                var updateCheckinData = await SFA_masterModel.updateCheckinDataLinkedDetail(req.body);
                console.log('updateCheckinData', updateCheckinData);
            }

            await db.commit();
            res.status(200).json({
                message: "Followup data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Followup data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.detailFollowupData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var detailFollowupData = await SFA_masterModel.detailFollowupData(req.body);
        console.log('detailFollowupData', detailFollowupData);

        await db.commit();
        res.status(200).json({
            message: "Followup data detail",
            status: "success",
            data: {
                detail: detailFollowupData
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateFollowupStatus = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateFollowupStatus = await SFA_masterModel.updateFollowupStatus(req.body);
        console.log('updateFollowupStatus', updateFollowupStatus);

        if (updateFollowupStatus.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Followup data updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Followup data not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.deleteFollowupData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteFollowupData = await SFA_masterModel.deleteFollowupData(req.body);
        console.log('deleteFollowupData', deleteFollowupData);

        if (deleteFollowupData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Followup data deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Followup data not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.getTravelPlanDataCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Pending' },
            { 'name': 'Approved' },
            { 'name': 'Rejected' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getTravelPlanListCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'filter': { 'status': element.name } });
                // console.log('element.count',element.count);
                if (i == 0) {
                    await db.commit();
                    res.status(200).json({
                        message: "Travel Plan List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            await db.commit();
            res.status(200).json({
                message: "Travel Plan List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getTravelPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var getTravelPlanListCount = await SFA_masterModel.getTravelPlanListCount(req.body);
        console.log('getTravelPlanListCount', getTravelPlanListCount);
        var getTravelPlanListData = [];
        if (getTravelPlanListCount > 0) {
            getTravelPlanListData = await SFA_masterModel.getTravelPlanListData(req.body);
            // console.log('getTravelPlanListData',getTravelPlanListData);
        }

        res.status(200).json({
            message: "Travel Plan List Data",
            status: "success",
            result: {
                data: getTravelPlanListData,
                count: getTravelPlanListCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getTravelPlanTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getTravelPlanTypeData = await SFA_masterModel.getTravelPlanTypeData(req.body);
        // console.log('getTravelPlanTypeData',getTravelPlanTypeData);

        res.status(200).json({
            message: "Travel type data",
            status: "success",
            result: {
                data: getTravelPlanTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addTravelPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var addTravelPlanData = await SFA_masterModel.addTravelPlanData(req.body);
        console.log('addTravelPlanData', addTravelPlanData);

        if (addTravelPlanData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Travel Plan data added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Travel Plan data not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateStatusTravelPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateStatusTravelPlanData = await SFA_masterModel.updateStatusTravelPlanData(req.body);
        console.log('updateStatusTravelPlanData', updateStatusTravelPlanData);

        if (updateStatusTravelPlanData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Travel Plan Status Updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Travel Plan status not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.detailTravelPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var detailTravelPlanData = await SFA_masterModel.detailTravelPlanData(req.body);
        console.log('detailTravelPlanData', detailTravelPlanData);

        var getDistributorNetworkList = [];
        var getLeadNetworkList = [];
        if (detailTravelPlanData.id) {

            var drDataFilter = {
                limit: 100,
                start: 0,
                filter: {
                    is_lead: "0",
                    state_name: detailTravelPlanData.state_name ? detailTravelPlanData.state_name : '',
                    district_name: detailTravelPlanData.district_name ? detailTravelPlanData.district_name : '',
                    city: detailTravelPlanData.city ? detailTravelPlanData.city : '',
                    area: detailTravelPlanData.area ? detailTravelPlanData.area : ''
                }
            }
            var getDistributorNetworkList = await SFA_masterModel.getDistributorNetworkList(drDataFilter);
            // console.log('getDistributorNetworkList',getDistributorNetworkList);


            var leadDataFilter = {
                limit: 100,
                start: 0,
                filter: {
                    is_lead: "1",
                    state_name: detailTravelPlanData.state_name ? detailTravelPlanData.state_name : '',
                    district_name: detailTravelPlanData.district_name ? detailTravelPlanData.district_name : '',
                    city: detailTravelPlanData.city ? detailTravelPlanData.city : '',
                    area: detailTravelPlanData.area ? detailTravelPlanData.area : ''
                }
            }
            var getLeadNetworkList = await SFA_masterModel.getDistributorNetworkList(leadDataFilter);
            // console.log('getLeadNetworkList',getLeadNetworkList);

        }

        await db.commit();
        res.status(200).json({
            message: "Travel Plan data detail",
            status: "success",
            data: {
                detail: detailTravelPlanData,
                drNetwork: getDistributorNetworkList,
                leadNetwork: getLeadNetworkList
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.deleteTravelPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteTravelPlanData = await SFA_masterModel.deleteTravelPlanData(req.body);
        console.log('deleteTravelPlanData', deleteTravelPlanData);

        if (deleteTravelPlanData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Travel Plan data deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Travel Plan data not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getLeadTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getLeadTypeData = await SFA_masterModel.getLeadTypeData(req.body);
        // console.log('getLeadTypeData',getLeadTypeData);

        res.status(200).json({
            message: "Lead type data",
            status: "success",
            result: {
                data: getLeadTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getLeadStageListData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getLeadStageListData = await SFA_masterModel.getLeadStageListData(req.body);
        // console.log('getLeadStageListData',getLeadStageListData);

        res.status(200).json({
            message: "Lead type data",
            status: "success",
            result: {
                data: getLeadStageListData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getLeadSourceListData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getLeadSourceListData = await SFA_masterModel.getLeadSourceListData(req.body);
        // console.log('getLeadSourceListData',getLeadSourceListData);

        res.status(200).json({
            message: "Lead source data",
            status: "success",
            result: {
                data: getLeadSourceListData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getLeadStatusListData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        // console.log('body ==>', req.body);

        var getLeadStatusListData = await SFA_masterModel.getLeadStatusListData(req.body);
        // console.log('getLeadStatusListData',getLeadStatusListData);

        res.status(200).json({
            message: "Lead status list",
            status: "success",
            result: {
                data: getLeadStatusListData,
            }

        });

    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.addLeadData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var cust_type_name = await SFA_masterModel.getCustTypeNameById(req.body);
        req.body.cust_type = cust_type_name ? cust_type_name : '';

        if (req.body.cust_type_id == 3) {
            req.body.company_name = req.body.contact_person;
        }


        req.body.mobile_no = req.body.mobile;
        var findByMobileNumber = await User.findByMobileNumber_salesUser(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id) {
            return res.json({
                message: "Sales User Exist with this Mobile Number",
                status: "notexist",
            });
        }


        console.log('body -->', req.body);
        var addLeadData = await SFA_masterModel.addLeadData(req.body);
        // console.log('addLeadData',addLeadData);

        if (addLeadData.insertId) {

            req.body.customer_id = addLeadData.insertId ? addLeadData.insertId : 0;

            if (req.body.lead_image) {
                var documentData = {
                    customer_id: req.body.customer_id ? req.body.customer_id : 0,
                    doc_type: 'Lead Data',
                    doc_type_value: req.body.doc_no ? req.body.doc_no : '',
                    doc_filename: req.body.lead_image ? req.body.lead_image : '',
                    doc_filename_1: req.body.lead_image_2 ? req.body.lead_image_2 : '',
                };

                var addCustomerDocument = await User.addCustomerDocument(documentData);
                console.log('addCustomerDocument', addCustomerDocument);
            }


            if (req.body.carpenter?.id || req.body.architect?.id) {
                var addCustomerDealer = await User.addCustomerDealer(req.body);
                console.log('addCustomerDealer', addCustomerDealer);
            }

            var addCustomerAssignedUser = await User.addCustomerAssignedSalesUser(req.body);
            console.log(addCustomerAssignedUser);

            await db.commit();
            res.status(200).json({
                message: "Lead data added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Lead data not added",
                status: "error",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if (e.code == 'ER_DUP_ENTRY') {
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>', myArray[1])
            var key = myArray[1];
            console.log('key', key);
            key = key.replace(/'/g, "");
            console.log('key', key);
            if (key == 'tg_customer.tg_customer_mobile_UN') {
                errMessage = 'Duplicate entry for mobile number - ' + req.body.mobile;
            } else if (key == 'tg_customer.tg_customer_UN') {
                errMessage = 'Duplicate entry for email - ' + req.body.email;
            } else {
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage', errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateStageStatus = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateStageStatus = await SFA_masterModel.updateStageStatus(req.body);
        console.log('updateStageStatus', updateStageStatus);

        if (updateStageStatus.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Stage status updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Stage status not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// order module code start


exports.getOrderDrTypeData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var getOrderDrTypeData = await SFA_masterModel.getOrderDrTypeData(req.body);
        // console.log('getOrderDrTypeData',getOrderDrTypeData);

        res.status(200).json({
            message: "Order Dr type data",
            status: "success",
            result: {
                data: getOrderDrTypeData,
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getOrderDataCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        var statusArrayData = [
            { 'name': 'All' },
            { 'name': 'Draft' },
            { 'name': 'Pending' },
            { 'name': 'Approved' },
            { 'name': 'Rejected' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getOrderDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'filter': { 'order_status': element.name, 'order_type': req.body.order_type } });
                // console.log('element.count',element.count);
                if (i == 0) {
                    await db.commit();
                    res.status(200).json({
                        message: "Order List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            await db.commit();
            res.status(200).json({
                message: "Order List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getOrderDataList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        var getOrderDataCount = await SFA_masterModel.getOrderDataCount(req.body)

        var getOrderDataList = [];
        if (getOrderDataCount > 0) {
            getOrderDataList = await SFA_masterModel.getOrderDataList(req.body);
            // console.log('getOrderDataList',getOrderDataList);
        }

        res.status(200).json({
            message: "Order List",
            status: "success",
            result: {
                data: getOrderDataList,
                count: getOrderDataCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addOrderData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // if(req.body.dr_type.id == 2){
        //     req.body.order_type = 'Secondary';
        //     req.body.customer_id = req.body.distributorData.id;
        //     req.body.customer_name = req.body.distributorData.company_name;
        //     req.body.customer_type = req.body.distributorData.cust_type;
        //     req.body.dealer_id = req.body.drData.id;
        //     req.body.dealer_name = req.body.drData.company_name;
        //     req.body.dealer_type = req.body.drData.cust_type;
        // }else{
        //     req.body.order_type = 'Primary';
        //     req.body.customer_id = req.body.drData.id;
        //     req.body.customer_name = req.body.drData.company_name;
        //     req.body.customer_type = req.body.drData.cust_type;
        // }



        req.body.order_type = 'Primary';
        req.body.customer_id = req.body.drData.id;
        req.body.customer_name = req.body.drData.company_name;
        req.body.customer_type = req.body.drData.cust_type;

        // console.log('body ==>', req.body);

        var addOrderData = await SFA_masterModel.addOrderData(req.body);
        console.log('addOrderData', addOrderData);

        req.body.orderItem = req.body.orderItemData;

        if (addOrderData.insertId) {
            var i = req.body.orderItem.length - 1;
            if (req.body.orderItem.length) {
                req.body.orderItem.forEach(async element => {
                    element.order_id = addOrderData.insertId;
                    element.order_qty = element.qty ? element.qty : 0;
                    element.product_name = element.product_name ? element.product_name : '';
                    element.product_code = element.product_code ? element.product_code : '';
                    element.product_id = element.product_id ? element.product_id : 0;
                    element.brand = element.brand ? element.brand : '';
                    element.sub_brand = element.sub_brand ? element.sub_brand : '';
                    element.category = element.category ? element.category : '';
                    element.product_group = element.product_group ? element.product_group : '';
                    element.unit = element.unit ? element.unit : '';
                    element.thickness = element.thickness ? element.thickness : '';
                    element.size = element.size ? element.size : '';

                    element.customer_id = req.body.customer_id ? req.body.customer_id : 0;
                    element.product_type = req.body.type ? req.body.type : '';

                    var addOrderItemData = await SFA_masterModel.addOrderItemData(element);
                    // console.log('addOrderItemData',addOrderItemData);
                    if (i == 0) {

                        if (req.body.checkin_id) {
                            req.body.order_id = addOrderData.insertId ? addOrderData.insertId : 0;
                            req.body.order_added = 1
                            req.body.update_type = 'order';
                            var updateCheckinData = await SFA_masterModel.updateCheckinDataLinkedDetail(req.body);
                            console.log('updateCheckinData', updateCheckinData);
                        }

                        await db.commit();
                        res.status(200).json({
                            message: "Order Data added",
                            status: "success",
                        });
                    }
                    i--;
                });
            } else {
                await db.commit();
                res.status(200).json({
                    message: "Order Data added",
                    status: "success"
                });
            }
        }



    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getOrderDetail = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // req.body.order_id = decrypt(req.body.order_id);

        var getOrderBasicDetail = await SFA_masterModel.getOrderBasicDetail(req.body);
        // console.log('getOrderBasicDetail',getOrderBasicDetail);
        var getOrderItemDetail = await SFA_masterModel.getOrderItemDetail(req.body);

        console.log('getOrderBasicDetail', getOrderBasicDetail.id);

        await db.commit();
        res.status(200).json({
            message: "Order Detail",
            status: "success",
            result: {
                data: getOrderBasicDetail,
                orderItem: getOrderItemDetail,
            }
        });



    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateOrderItemData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateOrderItemData = await SFA_masterModel.updateOrderItemData(req.body);
        console.log('updateOrderItemData', updateOrderItemData);

        if (updateOrderItemData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Order item data updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Order item data not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteOrderItemData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteOrderItemData = await SFA_masterModel.deleteOrderItemData(req.body);
        console.log('deleteOrderItemData', deleteOrderItemData);

        if (deleteOrderItemData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Order item data deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Order item data not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// order module code end


exports.getEnquiryType = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        req.body.image_app_path = res.locals.image_app_path;

        var getEnquiryType = await SFA_masterModel.getEnquiryType(req.body);
        // console.log('getEnquiryType',getEnquiryType);

        await db.commit();
        res.status(200).json({
            data: getEnquiryType,
            message: "Enquiry data",
            status: "success"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.submitEnquiry = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var submitEnquiry = await SFA_masterModel.submitEnquiry(req.body);

        var responseData = {
            type: 'submit_enquiry'
        }

        var getResponseMessage = await SFA_masterModel.getResponseMessage(responseData);

        if (submitEnquiry.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Enquiry submitted",
                status: "success",
                responseMessage: getResponseMessage
            });
        } else {
            await db.rollback()
            res.status(200).json({
                message: "Enquiry submitted",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getPopAndGiftList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;

        var getPopAndGiftDataCount = await SFA_masterModel.getPopAndGiftDataCount(req.body)

        var getPopAndGiftDataList = [];
        if (getPopAndGiftDataCount > 0) {
            getPopAndGiftDataList = await SFA_masterModel.getPopAndGiftDataList(req.body);
            // console.log('getPopAndGiftDataList',getPopAndGiftDataList);
        }

        res.status(200).json({
            message: "Pop and Gift List",
            status: "success",
            isHead: req.body.sfa_app_user_isBranchHead,
            result: {
                data: getPopAndGiftDataList,
                count: getPopAndGiftDataCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.detailPopAndGiftData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var detailPopAndGiftData = await SFA_masterModel.detailPopAndGiftData(req.body);
        // console.log('detailPopAndGiftData',detailPopAndGiftData);

        var getGiftAssignDataDetail = await SFA_masterModel.getGiftAssignDataDetail(req.body);
        console.log('getGiftAssignDataDetail', getGiftAssignDataDetail);


        var getGiftRequestDataDetail = await SFA_masterModel.getGiftRequestDataDetail(req.body);
        console.log('getGiftRequestDataDetail', getGiftRequestDataDetail);

        await db.commit();
        res.status(200).json({
            message: "Pop and gift data detail",
            status: "success",
            data: {
                detail: detailPopAndGiftData,
                assignedGiftDetail: getGiftAssignDataDetail,
                requestGiftDetail: getGiftRequestDataDetail
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.requestPopAndGiftData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var requestPopAndGiftData = await SFA_masterModel.requestPopAndGiftData(req.body);
        console.log('requestPopAndGiftData', requestPopAndGiftData);

        if (requestPopAndGiftData.insertId) {

            await db.commit();
            res.status(200).json({
                message: "Pop and Gift requested",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Pop and Gift not requested",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.assignPopAndGiftData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var assignPopAndGiftData = await SFA_masterModel.assignPopAndGiftData(req.body);
        console.log('assignPopAndGiftData', assignPopAndGiftData);

        if (assignPopAndGiftData.insertId) {

            var stockData = {
                pop_gift_id: req.body.pop_gift_id ? req.body.pop_gift_id : 0,
                stock: req.body.qty ? req.body.qty : 0,
                gift_assigned_id: assignPopAndGiftData.insertId ? assignPopAndGiftData.insertId : 0,
                type: 'stockout',
                remark: 'Stock Assigned'
            };

            var popAndGiftStockLedgerMaintaindata = await SFA_masterModelWeb.popAndGiftStockLedgerMaintaindata(stockData);
            console.log('popAndGiftStockLedgerMaintaindata', popAndGiftStockLedgerMaintaindata);


            await db.commit();
            res.status(200).json({
                message: "Pop and Gift assigned",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Pop and Gift not assigned",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// event plan code start


exports.addEventPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var addEventPlanData = await SFA_masterModel.addEventPlanData(req.body);
        console.log('addEventPlanData', addEventPlanData);

        if (addEventPlanData.insertId) {

            await db.commit();
            res.status(200).json({
                message: "Event Plan added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Event Plan not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getEventPlanDataCount = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var statusArrayData = [
            { 'name': 'Pending' },
            { 'name': 'Approved' },
            { 'name': 'Completed' },
            { 'name': 'Cancelled' },
            { 'name': 'Rejected' }
        ]

        if (statusArrayData.length) {
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getEventPlanDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, 'filter': { 'status': element.name } });
                // console.log('element.count',element.count);
                if (i == 0) {
                    await db.commit();
                    res.status(200).json({
                        message: "Event Plan List Count",
                        status: "success",
                        result: {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        } else {
            await db.commit();
            res.status(200).json({
                message: "Event Plan List Count",
                status: "success",
                result: {
                    data: statusArrayData,
                }
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getEventPlanDataList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;

        var getEventPlanDataCount = await SFA_masterModel.getEventPlanDataCount(req.body)

        var getEventPlanDataList = [];
        if (getEventPlanDataCount > 0) {
            getEventPlanDataList = await SFA_masterModel.getEventPlanDataList(req.body);
            // console.log('getEventPlanDataList',getEventPlanDataList);
        }

        res.status(200).json({
            message: "Event and Plan List",
            status: "success",
            result: {
                data: getEventPlanDataList,
                count: getEventPlanDataCount
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteEventPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteEventPlanData = await SFA_masterModel.deleteEventPlanData(req.body);
        console.log('deleteEventPlanData', deleteEventPlanData);

        if (deleteEventPlanData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Event Plan deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Event Plan not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.detailEventPlanData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var detailEventPlanData = await SFA_masterModel.detailEventPlanData(req.body);
        console.log('detailEventPlanData', detailEventPlanData);

        req.body.type = 'participant';
        var detailEventPlanParticipantData = await SFA_masterModel.detailEventPlanParticipantData(req.body);
        console.log('detailEventPlanParticipantData', detailEventPlanParticipantData);

        req.body.type = 'gallery';
        var detailEventPlanGalleryData = await SFA_masterModel.detailEventPlanParticipantData(req.body);
        console.log('detailEventPlanGalleryData', detailEventPlanGalleryData);

        await db.commit();
        res.status(200).json({
            message: "Event plan data detail",
            status: "success",
            data: {
                detail: detailEventPlanData,
                participantData: detailEventPlanParticipantData,
                galleryData: detailEventPlanGalleryData,
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateEventPlanStatus = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var updateEventPlanStatus = await SFA_masterModel.updateEventPlanStatus(req.body);
        console.log('updateEventPlanStatus', updateEventPlanStatus);

        if (updateEventPlanStatus.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Event Plan status updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Event Plan status not updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addEventPlanParticipantData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var addEventPlanParticipantData = await SFA_masterModel.addEventPlanParticipantData(req.body);
        console.log('addEventPlanParticipantData', addEventPlanParticipantData);

        if (addEventPlanParticipantData.insertId) {

            await db.commit();
            res.status(200).json({
                message: "Participant added",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Participant not added",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteEventPlanParticipantData = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        var deleteEventPlanParticipantData = await SFA_masterModel.deleteEventPlanParticipantData(req.body);
        console.log('deleteEventPlanParticipantData', deleteEventPlanParticipantData);

        if (deleteEventPlanParticipantData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Participant deleted",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Participant not deleted",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// event plan code end


// live track code start


exports.trackUserLocation = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        // var geocoder = NodeGeocoder(options);
        // await geocoder.reverse({lat:req.body.latitude, lon:req.body.longitude}, function(err, res) {
        //     // console.log('res ---->', res);
        //     if(res.length){
        //         req.body.address = res[0].formattedAddress
        //     }
        // });
        req.body.lat = req.body.latitude;
        req.body.lng = req.body.longitude;

        var trackUserLocation = await SFA_masterModel.trackUserLocation(req.body);
        // console.log('trackUserLocation',trackUserLocation);

        await db.commit();
        return res.status(200).json({
            message: "Location added",
            status: "success"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// live track code end

// report DWR

exports.getDwrReport = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var dwrReportData = [];

        req.body.attend_date = req.body.report_date;
        if (req.body.user_id) {
            req.body.sfa_app_user_id = req.body.user_id;
        }


        var getTodayAttendanceData = await SFA_masterModel.getTodayAttendanceData(req.body);
        // console.log('getTodayAttendanceData',getTodayAttendanceData);

        if (getTodayAttendanceData.id) {
            var attendanceStartData = {
                start_address: getTodayAttendanceData.start_address,
                start_lat: getTodayAttendanceData.start_lat,
                start_lng: getTodayAttendanceData.start_lng,
                start_time: getTodayAttendanceData.start_time
            }
            dwrReportData.push(attendanceStartData);
        }

        req.body.filter = {
            checkin_date: req.body.report_date,
            order_by: 'ASC'
        }

        var getCheckinListData = await SFA_masterModel.getCheckinListData(req.body);
        // console.log('getCheckinListData',getCheckinListData);

        if (getCheckinListData.length) {
            getCheckinListData.map(x => {
                dwrReportData.push(x);
            });
        }

        if (getTodayAttendanceData.id && getTodayAttendanceData.stop_lat) {
            console.log('here');
            var attendanceStartData = {
                stop_address: getTodayAttendanceData.stop_address,
                stop_lat: getTodayAttendanceData.stop_lat,
                stop_lng: getTodayAttendanceData.stop_lng,
                stop_time: getTodayAttendanceData.stop_time
            }
            dwrReportData.push(attendanceStartData);
        }

        // var getOrderDataCount = await SFA_masterModel.getOrderDataCount(req.body)

        var getCountData = {
            total_checkin_visit: await SFA_masterModel.getCheckinListCount({ "sfa_app_user_id": req.body.sfa_app_user_id, "filter": { "order_type": "Secondary", "checkin_date": req.body.report_date } }),
            total_new_lead: await SFA_masterModel.getDistributorNetworkCount({ "sfa_app_user_id": req.body.sfa_app_user_id, "filter": { "is_lead": "1", "created_on": req.body.report_date, 'created_by': req.body.sfa_app_user_id } }),
            travel_in_km: getTodayAttendanceData.google_km,
            primary_order: await SFA_masterModel.getOrderDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, "filter": { "order_type": "Primary", "created_on": req.body.report_date } }),
            secondary_order: await SFA_masterModel.getOrderDataCount({ "sfa_app_user_id": req.body.sfa_app_user_id, "filter": { "order_type": "Secondary", "created_on": req.body.report_date } }),
        };

        var getAssignedUserList = [];
        if (req.body.user_id) {
            getAssignedUserList = await SFA_masterModel.getAssignedUserList(req.body);
            console.log('getAssignedUserList', getAssignedUserList);
        }

        res.status(200).json({
            message: "DWR Report Data",
            status: "success",
            result: {
                data: dwrReportData,
                attendanceData: getTodayAttendanceData,
                countData: getCountData,
                assignedUserData: getAssignedUserList
            },
            isHead: req.body.sfa_app_user_isBranchHead
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getAssignedUserList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        // await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        if (req.body.user_id) {
            req.body.sfa_app_user_id = req.body.user_id
        }
        // return false;
        var getAssignedUserList = await SFA_masterModel.getAssignedUserList(req.body);
        console.log('getAssignedUserList', getAssignedUserList);

        res.status(200).json({
            message: "Get Assigned User Data",
            status: "success",
            result: {
                data: getAssignedUserList
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.getBlockedThirdPartyApps = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        // await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var getBlockedThirdPartyApps = await SFA_masterModel.getBlockedThirdPartyApps(req.body);
        // console.log('getBlockedThirdPartyApps',getBlockedThirdPartyApps);

        res.status(200).json({
            message: "Third Party Apps List",
            status: "success",
            result: {
                data: getBlockedThirdPartyApps
            }
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.thirdPartyAppUsed = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        // await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var thirdPartyAppUsed = await SFA_masterModel.thirdPartyAppUsed(req.body);
        // console.log('thirdPartyAppUsed',thirdPartyAppUsed);

        res.status(200).json({
            message: "Data Updated",
            status: "success"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};
