const Home = require("./../../models/app/homeModel");
const User = require("./../../models/app/userModel");
const SFA_masterModel = require("./../../models/app/SFA_masterModel");
const masterModel = require("./../../models/app/masterModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');


exports.getAppHomeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.image_app_path = res.locals.image_app_path;
        
        var getBannerData = await Home.getBannerData(req.body);
        // console.log('getBannerData',getBannerData);
        
        var getTodayPointData = await Home.getTodayPointData(req.body);
        // console.log('getTodayPointData',getTodayPointData);
        
        var getLast7DaysPointData = await Home.getLast7DaysPointData(req.body);
        // console.log('getLast7DaysPointData',getLast7DaysPointData);
        
        var getMonthPointData = await Home.getMonthPointData(req.body);
        // console.log('getMonthPointData',getMonthPointData);
        
        var getWalletPointData = await Home.getWalletPointData(req.body);
        // console.log('getWalletPointData',getWalletPointData);
        
        var getSiteImageDetailData = await Home.getSiteImageDetailData(req.body);
        console.log('getSiteImageDetailData',getSiteImageDetailData);
        var getSiteImageDetailData =  getSiteImageDetailData.length >= 10 ? getSiteImageDetailData : [];
        var getProjectData = await Home.getProjectData(req.body);
        // console.log('getProjectData',getProjectData);
        
        var getProductCategoryData = await Home.getProductCategoryData(req.body);
        console.log('getProductData',getProductCategoryData);
        
        var getCustTypeList = await Home.getCustTypeList(req.body);
        // console.log('getCustTypeList',getCustTypeList);

        // var getLeadTypeList = await Home.getLeadTypeList(req.body);
        // console.log('getLeadTypeList',getLeadTypeList);
        
        await db.commit();
        res.status(200).json({
            message: "App Home",
            status: "success",
            result : {
                bannerData: getBannerData,
                getCustTypeListCount : getCustTypeList,
                getLeadTypeList : getCustTypeList,
                pointData : {
                    "today" : getTodayPointData,
                    "last7Days" : getLast7DaysPointData,
                    "month" : getMonthPointData,
                    "walletPoint" : getWalletPointData
                },
                // order_data:{
                //     "ytd":"3,455",
                //     "mtd":"24,554",
                //     "pending":"30,000",
                //     "pending_count":"4"
                // },
                siteImageData : getSiteImageDetailData,
                projectData : getProjectData,
                productCategoryData : getProductCategoryData,
                status : req.body.sfa_app_user_status ? req.body.sfa_app_user_status : 'Pending'
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSfaAppHomeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.image_app_path = res.locals.image_app_path;
        
        var getBannerData = await Home.getBannerData(req.body);
        // console.log('getBannerData',getBannerData);
        
        var getTodayAttendanceData = await Home.getTodayAttendanceData(req.body);
        // console.log('getTodayAttendanceData',getTodayAttendanceData);
        
        var getCustTypeList = await Home.getCustTypeList(req.body);
        // console.log('getCustTypeList',getCustTypeList);
        
        var getLeadTypeList = await Home.getLeadTypeList(req.body);
        console.log('getLeadTypeList',getLeadTypeList);
        
        var getSfaCountData = {};
        
        getSfaCountData.primaryOrderCount = await SFA_masterModel.getOrderDataCount({"sfa_app_user_id" : req.body.sfa_app_user_id , 'filter' : {'order_status' : 'All', 'order_type' : 'Primary', 'created_on' : moment(new Date()).format('YYYY-MM-DD')}});
        getSfaCountData.secondaryOrderCount = await SFA_masterModel.getOrderDataCount({"sfa_app_user_id" : req.body.sfa_app_user_id , 'filter' : {'order_status' : 'All', 'order_type' : 'Secondary', 'created_on' : moment(new Date()).format('YYYY-MM-DD')}});
        getSfaCountData.followUpCount =  await SFA_masterModel.getFollowupListCount({"sfa_app_user_id" : req.body.sfa_app_user_id ,'filter' : {'followup_date' : moment(new Date()).format('YYYY-MM-DD')}});
        getSfaCountData.checkInCount =  await SFA_masterModel.getCheckinListCount({"sfa_app_user_id" : req.body.sfa_app_user_id , 'filter' : {'checkin_date' : moment(new Date()).format('YYYY-MM-DD')}});
        
        
        var getPendingCheckinList = await Home.getPendingCheckinList(req.body);
        // console.log('getPendingCheckinList',getPendingCheckinList);
        
        var getPdfData = await masterModel.getContentData({"type" : 'sfaHomePdf'});
        
        var getTodayBeatPlan = await User.getTodayAssignedBeatPlan(req.body);
        
        await db.commit();
        res.status(200).json({
            message: "App Home",
            status: "success",
            result : {
                bannerData: getBannerData,
                todayAttendanceData : getTodayAttendanceData,
                getCustTypeListCount : getCustTypeList,
                getLeadTypeList : getLeadTypeList,
                countData : getSfaCountData,
                pendingCheckinList : getPendingCheckinList,
                pdfData : getPdfData,
                beatPlanData : getTodayBeatPlan
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};
