const User = require("./../../models/app/userModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var base64ToFilePath = require('../../config/genericFn/base64ToFilePath');
var moment = require('moment');
const SFA_masterModel = require("./../../models/app/SFA_masterModel");
const Notification = require("../../models/web/notificationModel");



exports.getAppVersion = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var getAppVersion = await User.getAppVersion(req.body);
        // console.log('getAppVersion',getAppVersion);

        await db.commit();
        res.status(200).json({
            data: getAppVersion,
            message: "App version data",
            status: "success"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.checkMobileNumber = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var mobileNumberRegistered = false;
        var checkMobileNumber = await User.checkMobileNumber(req.body);
        // console.log('checkMobileNumber',checkMobileNumber);

        if (checkMobileNumber.id) {
            mobileNumberRegistered = true;
        }

        await db.commit();
        res.status(200).json({
            data: {
                mobileNumberRegistered: mobileNumberRegistered,
                id: checkMobileNumber.id ? checkMobileNumber.id : 0,
            },
            message: 'Check Mobile Number',
            status: "success"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.leadCheckMobileNumber = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var mobileNumberRegistered = false;
        var checkMobileNumber = await User.checkMobileNumber(req.body);
        // console.log('checkMobileNumber',checkMobileNumber);

        if (checkMobileNumber.id) {
            mobileNumberRegistered = true;
            var message = req.body.mobile_no + ' this mobile number already registered with us as ' + checkMobileNumber.cust_type;

        }

        var checkMobileNumber = await User.findByMobileNumber_salesUser(req.body);
        console.log('checkMobileNumber', checkMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (checkMobileNumber.id) {
            mobileNumberRegistered = true;
            var message = req.body.mobile_no + ' this mobile number already registered with us as Sales Executive';
        }


        await db.commit();
        res.status(200).json({
            data: {
                mobileNumberRegistered: mobileNumberRegistered,
                id: checkMobileNumber.id ? checkMobileNumber.id : 0,
            },
            message: message,
            status: "Record Found"
        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.loginGetOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var findByMobileNumber = await User.findByMobileNumber(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id == undefined) {
            res.json({
                message: "User Not Exist with this Mobile Number",
                status: "notexist",
            });
            return false;
        }

        var findByMobileNumber_salesUser = await User.findByMobileNumber_salesUser(req.body);
        console.log('findByMobileNumber_salesUser', findByMobileNumber_salesUser);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber_salesUser.id) {
            res.json({
                message: "Sales User Exist with this Mobile Number",
                status: "exist",
            });
            return false;
        }


        // var checkLastLoginOtpRequest = await User.checkLastLoginOtpRequest(req.body);
        // // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest);
        // // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest.id);

        // if(checkLastLoginOtpRequest.id != undefined){
        //     res.status(200).json({
        //         message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest.mobile,
        //         status: "failed",
        //         statusType: "already_sent",
        //     });
        //     return false;
        // }

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }

        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code
        };
        var sendOtp = await User.sendOtp(sendOtpData);

        var loginUpdateOtp = await User.loginUpdateOtp(req.body);
        // console.log('loginUpdateOtp',loginUpdateOtp);
        // console.log('data',loginUpdateOtp.id);

        if (loginUpdateOtp.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "OTP Sent",
                status: "success",
            });

        } else {
            await db.rollback();
            res.status(200).json({
                message: "OTP not sent",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.loginGetOtp_salesUser = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var findByMobileNumber = await User.findByMobileNumber_salesUser(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id == undefined) {
            res.json({
                message: "User Not Exist with this Mobile Number",
                status: "notexist",
            });
            return false;
        }


        var findByMobileNumber_DR = await User.findByMobileNumber(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber_DR.id) {
            res.json({
                message: "DR User Exist with this Mobile Number",
                status: "exist",
            });
            return false;
        }


        var checkLastLoginOtpRequest = await User.checkLastLoginOtpRequest_salesUser(req.body);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest.id);

        if (checkLastLoginOtpRequest.id != undefined) {
            res.status(200).json({
                message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest.mobile_no,
                status: "failed",
                statusType: "already_sent",
            });
            return false;
        }

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }
        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code
        };
        var sendOtp = await User.sendOtp(sendOtpData);

        var loginUpdateOtp = await User.loginUpdateOtp_salesUser(req.body);
        // console.log('loginUpdateOtp',loginUpdateOtp);
        // console.log('data',loginUpdateOtp.id);

        if (loginUpdateOtp.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "OTP Sent",
                status: "success",
            });

        } else {
            await db.rollback();
            res.status(200).json({
                message: "OTP not sent",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.loginWithOtp_salesUser = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        var loginCheckOtp = await User.loginCheckOtpRequest_salesUser(req.body);
        console.log('loginCheckOtp', loginCheckOtp);
        console.log('loginCheckOtp', loginCheckOtp.id);
        if (loginCheckOtp.id == undefined) {
            res.json({
                message: "Invalid OTP",
                status: "failed",
            });
            return false;
        } else {

            var updateSessionToken = await User.updateSessionToken_salesUser(loginCheckOtp);
            console.log('updateSessionToken', updateSessionToken);
            if (updateSessionToken.affectedRows) {
                loginCheckOtp.session_token = updateSessionToken.session_token;
                await db.commit();
                res.json({
                    message: "Login Successfully",
                    status: "success",
                    data: loginCheckOtp
                });
            } else {
                await db.rollback();
                res.json({
                    message: "Error occured",
                    status: "failed",
                });
            }

        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.resendOtp_salesUser = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var findByMobileNumber = await User.findByMobileNumber_salesUser(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id == undefined) {
            res.json({
                message: "User Not Exist with this Mobile Number",
                status: "notexist",
            });
            return false;
        }


        var checkLastLoginOtpRequest = await User.checkLastLoginOtpRequest_salesUser(req.body);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest.id);

        if (checkLastLoginOtpRequest.id != undefined) {
            res.status(200).json({
                message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest.mobile_no,
                status: "failed",
            });
            return false;
        }

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }

        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code
        };
        var sendOtp = await User.sendOtp(sendOtpData);

        var loginUpdateOtp = await User.loginUpdateOtp_salesUser(req.body);
        // console.log('loginUpdateOtp',loginUpdateOtp);
        // console.log('data',loginUpdateOtp.id);

        if (loginUpdateOtp.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "OTP Sent",
                status: "success",
            });

        } else {
            await db.rollback();
            res.status(200).json({
                message: "OTP not sent",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.redeemRequestGetOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        var getGiftDetail = await User.getGiftDetail(req.body);
        var findByMobileNumber = await User.findByMobileNumber(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id == undefined) {
            res.json({
                message: "User Not Exist with this Mobile Number",
                status: "notexist",
            });
            return false;
        }


        // var checkLastLoginOtpRequest = await User.checkLastLoginOtpRequest(req.body);
        // // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest);
        // // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest.id);

        // if(checkLastLoginOtpRequest.id != undefined){
        //     res.status(200).json({
        //         message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest.mobile,
        //         status: "failed",
        //         statusType: "already_sent",
        //     });
        //     return false;
        // }

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }

        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code,
            message_type: 'redeem_request',
            first_name: req.body.sfa_app_user_name,
            gift_name: getGiftDetail.title ? getGiftDetail.title : '',
            point: getGiftDetail.points ? getGiftDetail.points : 0,
        };

        var sendOtp = await User.sendOtp(sendOtpData);

        var walletCoinsSum = await User.getWalletPoints(req.body);

        var loginUpdateOtp = await User.loginUpdateOtp(req.body);
        // console.log('loginUpdateOtp',loginUpdateOtp);
        // console.log('data',loginUpdateOtp.id);

        if (loginUpdateOtp.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "OTP Sent",
                status: "success",
                walletPoints: walletCoinsSum
            });

        } else {
            await db.rollback();
            res.status(200).json({
                message: "OTP not sent",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.loginWithOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        var loginCheckOtp = await User.loginCheckOtpRequest(req.body);
        console.log('loginCheckOtp', loginCheckOtp);
        console.log('loginCheckOtp', loginCheckOtp.id);
        if (loginCheckOtp.id == undefined) {
            res.json({
                message: "Invalid OTP",
                status: "failed",
            });
            return false;
        } else {

            var updateSessionToken = await User.updateSessionToken(loginCheckOtp);
            console.log('updateSessionToken', updateSessionToken);
            if (updateSessionToken.affectedRows) {
                loginCheckOtp.session_token = updateSessionToken.session_token;
                await db.commit();
                res.json({
                    message: "Login Successfully",
                    status: "success",
                    data: loginCheckOtp
                });
            } else {
                await db.rollback();
                res.json({
                    message: "Error occured",
                    status: "failed",
                });
            }

        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.registerGetOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }


        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code
        };;
        var sendOtp = await User.sendOtp(sendOtpData);

        var findByMobileNumber_Temp = await User.findByMobileNumber_Temp(req.body);
        // console.log('findByMobileNumber_Temp',findByMobileNumber_Temp);
        // console.log('data',findByMobileNumber_Temp.id);

        if (findByMobileNumber_Temp.id == undefined) {
            var registerInsertOtp_Temp = await User.registerInsertOtp_Temp(req.body);
            console.log('registerInsertOtp_Temp', registerInsertOtp_Temp);
            console.log('data', registerInsertOtp_Temp.id);

            if (registerInsertOtp_Temp.insertId) {
                await db.commit();
                res.status(200).json({
                    message: "OTP Sent",
                    status: "success",
                    data: {
                        mobile_no: req.body.mobile_no
                    }
                });

            } else {
                await db.rollback();
                res.status(200).json({
                    message: "OTP not sent",
                    status: "failed",
                });
            }

        } else {
            var checkLastLoginOtpRequest_Temp = await User.checkLastLoginOtpRequest_Temp(req.body);
            // console.log('checkLastLoginOtpRequest_Temp',checkLastLoginOtpRequest_Temp);
            // console.log('checkLastLoginOtpRequest_Temp',checkLastLoginOtpRequest_Temp.id);

            if (checkLastLoginOtpRequest_Temp.id != undefined) {
                await db.commit();
                res.status(200).json({
                    message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest_Temp.mobile,
                    status: "failed",
                });
                return false;
            }

            var registerUpdateOtp_Temp = await User.registerUpdateOtp_Temp(req.body);
            // console.log('registerUpdateOtp_Temp',registerUpdateOtp_Temp);
            // console.log('data',registerUpdateOtp_Temp.id);

            if (registerUpdateOtp_Temp.affectedRows) {
                await db.commit();
                res.status(200).json({
                    message: "OTP Sent",
                    status: "success",
                    data: {
                        mobile_no: req.body.mobile_no
                    }
                });

            } else {
                await db.rollback();
                res.status(200).json({
                    message: "OTP not sent",
                    status: "failed",
                });
            }

        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.registerVerifyOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        var registerCheckOtpRequest_Temp = await User.registerCheckOtpRequest_Temp(req.body);
        console.log('registerCheckOtpRequest_Temp', registerCheckOtpRequest_Temp);
        console.log('registerCheckOtpRequest_Temp', registerCheckOtpRequest_Temp.id);
        if (registerCheckOtpRequest_Temp.id == undefined) {
            res.json({
                message: "Invalid OTP",
                status: "failed",
            });
            return false;
        } else {

            var registerCheckOtpVerify_Temp = await User.registerCheckOtpVerify_Temp(registerCheckOtpRequest_Temp);
            console.log('registerCheckOtpVerify_Temp', registerCheckOtpVerify_Temp);
            console.log('registerCheckOtpVerify_Temp', registerCheckOtpVerify_Temp.id);

            await db.commit();
            res.json({
                message: "OTP Verified",
                status: "success",
                data: {
                    mobile_no: req.body.mobile_no
                }
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.resendOtp = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();



        var findByMobileNumber = await User.findByMobileNumber(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id == undefined) {
            res.json({
                message: "User Not Exist with this Mobile Number",
                status: "notexist",
            });
            return false;
        }


        var checkLastLoginOtpRequest = await User.checkLastLoginOtpRequest(req.body);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest);
        // console.log('checkLastLoginOtpRequest',checkLastLoginOtpRequest.id);

        if (checkLastLoginOtpRequest.id != undefined) {
            res.status(200).json({
                message: "OTP already sent on your mobile number - " + checkLastLoginOtpRequest.mobile,
                status: "failed",
            });
            return false;
        }

        if (process.env.OTP_SERVICE == "development" || process.env.OTP_SERVICE_MOBILE.includes(req.body.mobile_no)) {
            req.body.otp_code = 1234;
        } else {
            req.body.otp_code = Math.floor(1000 + Math.random() * 9000);
        }

        var sendOtpData = {
            isd_code: req.body.isd_code,
            mobile_no: req.body.mobile_no,
            otp_code: req.body.otp_code
        };
        var sendOtp = await User.sendOtp(sendOtpData);

        var loginUpdateOtp = await User.loginUpdateOtp(req.body);
        // console.log('loginUpdateOtp',loginUpdateOtp);
        // console.log('data',loginUpdateOtp.id);

        if (loginUpdateOtp.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "OTP Sent",
                status: "success",
            });

        } else {
            await db.rollback();
            res.status(200).json({
                message: "OTP not sent",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.registration = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        req.body.user_referral_code = generateRandomString(8);

        var cust_type_name = await SFA_masterModel.getCustTypeNameById(req.body);
        req.body.cust_type = cust_type_name ? cust_type_name : '';


        req.body.mobile_no = req.body.mobile;
        var findByMobileNumber = await User.findByMobileNumber_salesUser(req.body);
        // console.log('findByMobileNumber',findByMobileNumber);
        // console.log('data',findByMobileNumber.id);

        if (findByMobileNumber.id) {
            return res.json({
                message: "Sales User Exist with this Mobile Number",
                status: "notexist",
            });
        }

        var registerAppUser = await User.registerAppUser(req.body);
        console.log('registerAppUser', registerAppUser);

        if (registerAppUser.insertId) {

            var documentData = {
                customer_id: registerAppUser.insertId ? registerAppUser.insertId : 0,
                doc_type: req.body.doc_type ? req.body.doc_type : '',
                doc_type_value: req.body.doc_no ? req.body.doc_no : '',
                doc_filename: req.body.doc_image ? req.body.doc_image : '',
                doc_filename_1: req.body.doc_image2 ? req.body.doc_image2 : '',
            };

            var addCustomerDocument = await User.addCustomerDocument(documentData);
            console.log('addCustomerDocument', addCustomerDocument);

            if (req.body.pan_no) {
                var documentData = {
                    customer_id: registerAppUser.insertId ? registerAppUser.insertId : 0,
                    doc_type: 'PAN Card',
                    doc_type_value: req.body.pan_no ? req.body.pan_no : '',
                    doc_filename: req.body.pan_image ? req.body.pan_image : '',
                };

                var addCustomerDocument = await User.addCustomerDocument(documentData);
                console.log('addCustomerDocument', addCustomerDocument);

            }

            if (req.body.company_doc) {
                var documentData = {
                    customer_id: registerAppUser.insertId ? registerAppUser.insertId : 0,
                    doc_type: 'Company Document',
                    doc_type_value: '',
                    doc_filename: req.body.company_doc ? req.body.company_doc : '',
                };

                var addCustomerDocument = await User.addCustomerDocument(documentData);
                console.log('addCustomerDocument', addCustomerDocument);

            }
            req.body.customer_id = registerAppUser.insertId ? registerAppUser.insertId : 0;
            if (req.body.dealer?.id) {
                var addCustomerDealer = await User.addCustomerDealer(req.body);
                console.log('addCustomerDealer', addCustomerDealer);
            }


            // console.log('test ==>',req.body.sales_employee.id);

            if (req.body.sales_employee?.id) {
                var addCustomerAssignedSalesUser = await User.addCustomerAssignedSalesUser(req.body);
                console.log('addCustomerAssignedSalesUser', addCustomerAssignedSalesUser);
            }


            if (req.body.partner_detail_array?.length) {
                var addCustomerPartnerData = await User.addCustomerPartnerData(req.body);
                console.log('addCustomerPartnerData', addCustomerPartnerData);
            }

            var userData = {
                id: registerAppUser.insertId,
                session_token: registerAppUser.session_token ? registerAppUser.session_token : null,
                name: req.body.first_name ? req.body.first_name : '',
                mobile: req.body.mobile ? req.body.mobile : '',
                profile_image: req.body.profile_image ? req.body.image_app_path + req.body.profile_image : '',
                cust_type: req.body.cust_type ? req.body.cust_type : '',
                cust_type_id: req.body.cust_type_id ? req.body.cust_type_id : 0,
            }



            // wallet point code start
            var getReferralBonusPoint = await User.getReferralBonusPoint(req.body);
            console.log('getReferralBonusPoint', getReferralBonusPoint);

            var referral_user_point = getReferralBonusPoint.referral_user_point ? getReferralBonusPoint.referral_user_point : 0;
            console.log('referral_user_point', referral_user_point);
            var referral_owner_point = getReferralBonusPoint.referral_owner_point ? getReferralBonusPoint.referral_owner_point : 0;
            console.log('referral_user_point', referral_owner_point);
            var welcome_point = getReferralBonusPoint.welcome_point ? getReferralBonusPoint.welcome_point : 0;
            console.log('welcome_point', welcome_point);


            var first_name = req.body.first_name ? req.body.first_name : ''

            var findReferredCode = {};
            if (req.body.referred_by_code) {
                findReferredCode = await User.findReferredCode(req.body);
                console.log('findReferredCode', findReferredCode);
                console.log('findReferredCode', findReferredCode.id);
                

                if (findReferredCode.id) {

                    var sendReferralOwnerPointData = {
                        customer_id: findReferredCode.id ? findReferredCode.id : 0,
                        against_customer_id: registerAppUser.insertId,
                        type: 'referral_owner',
                        point: referral_owner_point
                    }

                    var sendReferralOwnerPoint = await User.sendWalletPoints(sendReferralOwnerPointData);
                    console.log('sendReferralOwnerPoint', sendReferralOwnerPoint);


                    var title = 'Hey, your referral code used by ' + first_name;
                    var description = 'Enjoy your wallet points';
                    var sendNotificationDataBody = {
                        customer_id: findReferredCode.id ? findReferredCode.id : 0,
                        title: title ? title : '',
                        description: description ? description : '',
                        remark: description ? description : '',
                        notification_type: 'referral_user',
                    }

                    var sendNotificationData = await Notification.sendNotificationData(sendNotificationDataBody);
                    console.log('sendNotificationData', sendNotificationData);

                    var sendReferralUserPointData = {
                        customer_id: registerAppUser.insertId,
                        against_customer_id: findReferredCode.id ? findReferredCode.id : 0,
                        type: 'referral_user',
                        point: referral_user_point
                    };

                    var sendReferralUserPoint = await User.sendWalletPoints(sendReferralUserPointData);
                    console.log('sendReferralUserPoint', sendReferralUserPoint);

                    var title = 'Hey ' + first_name + ', you get bonus in your wallet using signup referral code';
                    var description = 'Enjoy your wallet points';
                    var sendNotificationDataBody = {
                        customer_id: registerAppUser.insertId ? registerAppUser.insertId : 0,
                        title: title ? title : '',
                        description: description ? description : '',
                        remark: description ? description : '',
                        notification_type: 'referral_user',
                    }

                    var sendNotificationData = await Notification.sendNotificationData(sendNotificationDataBody);
                    console.log('sendNotificationData', sendNotificationData);

                }
            } else {

                var sendWelcomeUserPointData = {
                    customer_id: registerAppUser.insertId,
                    type: 'welcome_bonus',
                    point: welcome_point
                };

                var sendWelcomeUserPoint = await User.sendWalletPoints(sendWelcomeUserPointData);
                console.log('sendWelcomeUserPoint', sendWelcomeUserPoint);

                var title = 'Hey ' + first_name + ', your profile status is verified from the admin';
                var description = 'Enjoy your welcome bonus wallet points';
                var sendNotificationDataBody = {
                    customer_id: registerAppUser.insertId ? registerAppUser.insertId : 0,
                    title: title ? title : '',
                    description: description ? description : '',
                    remark: description ? description : '',
                    notification_type: 'welcome_bonus',
                }

                var sendNotificationData = await Notification.sendNotificationData(sendNotificationDataBody);
                console.log('sendNotificationData', sendNotificationData);

               
            }

            // wallet point code end



            var referredData = {
                id: registerAppUser.insertId,
                referred_by: findReferredCode.id ? findReferredCode.id : 0,
                referred_by_name: findReferredCode.first_name ? findReferredCode.first_name : '',
                referred_by_code: req.body.referred_by_code ? req.body.referred_by_code : '',
            };


            var updateReferredDetails = await User.updateReferredDetails(referredData);
            console.log('updateReferredDetails', updateReferredDetails);



            await db.commit();
            res.status(200).json({
                message: "Registered Successfully",
                status: "success",
                data: userData
            });
        } else {
            res.status(200).json({
                message: "Not Registered",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if (e.code == 'ER_DUP_ENTRY') {
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>', myArray[1])
            var key = myArray[1];
            console.log('key', key);
            key = key.replace(/'/g, "");
            console.log('key', key);
            if (key == 'tg_customer_mobile_UN') {
                errMessage = 'Duplicate entry for mobile number - ' + req.body.mobile;
            } else if (key == 'tg_customer_UN') {
                errMessage = 'Duplicate entry for email - ' + req.body.email;
            } else {
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage', errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err: err
        });
        return false;
    }
};




exports.getProfileData = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;

        if (req.body.appType == 'sfa') {
            var getProfileData = await User.getProfileData_salesUser(req.body);
            // console.log('getProfileData',getProfileData);

            var getAssignedUser = await User.getAssignedUser_salesUser(req.body);
            // console.log('getAssignedUser',getAssignedUser);

            var getAssignedState = await User.getAssignedState_salesUser(req.body);
            // console.log('getAssignedUser',getAssignedUser);


            var getAssignedProductDivision = await User.getAssignedProductDivision_salesUser(req.body);
            // console.log('getAssignedProductDivision',getAssignedProductDivision);

            await db.commit();
            res.status(200).json({
                message: "Profile Data",
                status: "success",
                data: {
                    basicData: getProfileData,
                    assignedBeatPlan: req.body.sfa_app_user_assignedBeatPlan,
                    assignedSalesUser: getAssignedUser,
                    assignedState: getAssignedState,
                    assignedProductDivision: getAssignedProductDivision
                }
            });

        } else {
            var getProfileData = await User.getProfileData(req.body);
            // console.log('getProfileData',getProfileData);

            var getUserDocumentData = await User.getUserDocumentData(req.body);
            // console.log('getUserDocumentData',getUserDocumentData);

            var getAssignedSEDataInfo = await User.getAssignedSEDataInfo(req.body);
            // console.log('getAssignedSEDataInfo',getAssignedSEDataInfo);

            var getDealerPartnerData = await User.getDealerPartnerData(req.body);
            // console.log('getDealerPartnerData',getDealerPartnerData);

            var getDealerDataInfo = await User.getDealerDataInfo(req.body);
            // console.log('getDealerDataInfo',getDealerDataInfo);

            var getWalletPoints = await User.getWalletPoints(req.body);
            // console.log('getWalletPoints',getWalletPoints);

            await db.commit();
            res.status(200).json({
                message: "Profile Data",
                status: "success",
                data: {
                    basicData: getProfileData,
                    docData: getUserDocumentData,
                    walletPoints: getWalletPoints,
                    dealerData: getDealerDataInfo,
                    salesExecutiveData: getAssignedSEDataInfo,
                    partnerData: getDealerPartnerData
                }
            });
        }



    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getReferFriendText = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {

        var message = 'Hey, join me (' + req.body.sfa_app_user_name + ' - ' + req.body.sfa_app_user_mobile + ') on Saya Stationeries. Enter my code *' + req.body.sfa_app_user_referral_code + '* to earn points back in your wallet!';

        res.status(200).json({
            message: "Refer a friend data",
            status: "success",
            data: {
                message: message
            }
        });

    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getShareFriendText = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {

        var message = 'Share app content Here!';

        res.status(200).json({
            message: "Share a friend",
            status: "success",
            data: {
                message: message
            }
        });

    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateFcmToken = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        if (req.body.appType == 'sfa') {
            var updateFcmToken = await User.updateFcmToken_salesUser(req.body);
        } else {
            var updateFcmToken = await User.updateFcmToken(req.body);
        }

        // console.log('updateFcmToken',updateFcmToken);

        if (updateFcmToken.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "FCM token updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "FCM token updated",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.updateProfileDetail = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        console.log('body ==>', req.body);


        if (req.body.appType == 'sfa') {
            var updateProfileDetail = await User.updateProfileDetail_salesUser(req.body);
            console.log('update ==>', updateProfileDetail);

        } else {
            var updateProfileDetail = await User.updateProfileDetail(req.body);
            console.log('update ==>', updateProfileDetail);

            var documentData = {
                customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                doc_type: req.body.doc_type ? req.body.doc_type : '',
                doc_type_value: req.body.doc_no ? req.body.doc_no : '',
                doc_filename: req.body.doc_image ? req.body.doc_image : '',
                doc_filename_1: req.body.doc_image2 ? req.body.doc_image2 : '',
            };


            var deleteCustomerDocument = await User.deleteCustomerDocument(req.body);
            console.log('deleteCustomerDocument', deleteCustomerDocument);

            var addCustomerDocument = await User.addCustomerDocument(documentData);
            console.log('addCustomerDocument', addCustomerDocument);

            if (req.body.pan_no) {
                var documentData = {
                    customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                    doc_type: 'PAN Card',
                    doc_type_value: req.body.pan_no ? req.body.pan_no : '',
                    doc_filename: req.body.pan_image ? req.body.pan_image : '',
                };
                req.body.doc_type = 'PAN Card';
                var deleteCustomerDocument = await User.deleteCustomerDocument(req.body);
                console.log('deleteCustomerDocument', deleteCustomerDocument);

                var addCustomerDocument = await User.addCustomerDocument(documentData);
                console.log('addCustomerDocument', addCustomerDocument);

            }

            if (req.body.company_doc) {
                var documentData = {
                    customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                    doc_type: 'Company Document',
                    doc_type_value: '',
                    doc_filename: req.body.company_doc ? req.body.company_doc : '',
                };

                req.body.doc_type = 'Company Document';
                var deleteCustomerDocument = await User.deleteCustomerDocument(req.body);
                console.log('deleteCustomerDocument', deleteCustomerDocument);

                var addCustomerDocument = await User.addCustomerDocument(documentData);
                console.log('addCustomerDocument', addCustomerDocument);

            }
            req.body.customer_id = req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
            if (req.body.dealer?.id) {

                var deleteCustomerDealer = await User.deleteCustomerDealer(req.body);
                console.log('deleteCustomerDealer', deleteCustomerDealer);

                var addCustomerDealer = await User.addCustomerDealer(req.body);
                console.log('addCustomerDealer', addCustomerDealer);
            }


            // console.log('test ==>',req.body.sales_employee.id);

            if (req.body.sales_employee?.id) {

                var deleteCustomerAssignedSalesUser = await User.deleteCustomerAssignedSalesUser(req.body);
                console.log('deleteCustomerAssignedSalesUser', deleteCustomerAssignedSalesUser);

                var addCustomerAssignedSalesUser = await User.addCustomerAssignedSalesUser(req.body);
                console.log('addCustomerAssignedSalesUser', addCustomerAssignedSalesUser);
            }


            if (req.body.partner_detail_array?.length) {
                var deleteCustomerPartnerData = await User.deleteCustomerPartnerData(req.body);
                console.log('deleteCustomerPartnerData', deleteCustomerPartnerData);

                var addCustomerPartnerData = await User.addCustomerPartnerData(req.body);
                console.log('addCustomerPartnerData', addCustomerPartnerData);
            }

        }


        if (updateProfileDetail.affectedRows) {
            await db.commit();
            res.json({
                message: "Details updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.json({
                message: "Error occured",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProfileImage = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // console.log('body ==>', req.body);
        var base64ToImage = await base64ToFilePath.base64ToImage(req.body, 'system_profile_image');
        console.log('base64ToImage', base64ToImage);

        if (base64ToImage.fileName) {
            req.body.profile_image = base64ToImage.uploadPath ? base64ToImage.uploadPath : '';
        }

        if (req.body.appType == 'sfa') {
            var updateProfileImage = await User.updateProfileImage_salesUser(req.body);
        } else {
            var updateProfileImage = await User.updateProfileImage(req.body);
        }

        if (updateProfileImage.affectedRows) {
            await db.commit();
            res.json({
                message: "Details updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.json({
                message: "Error occured",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateSeInformation = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        console.log('body ==>', req.body);

        var updateSeInformation = await User.updateSeInformation(req.body);
        console.log('updateSeInformation ==>', updateSeInformation);
        if (updateSeInformation.affectedRows) {
            await db.commit();
            res.json({
                message: "Details updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.json({
                message: "Error occured",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.updateProfileDocument = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // console.log('body ==>', req.body);

        var updateProfileDocument = await User.updateProfileDocument(req.body);
        console.log('update ==>', updateProfileDocument);
        if (updateProfileDocument.affectedRows) {
            await db.commit();
            res.json({
                message: "Details updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.json({
                message: "Error occured",
                status: "failed",
            });
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateDealerInformation = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        // console.log('body ==>', req.body);

        if (req.body.dealer_id) {

            var updateDealerInformation = await User.updateDealerInformation(req.body);
            console.log('update ==>', updateDealerInformation);
            if (updateDealerInformation.affectedRows) {
                await db.commit();
                res.json({
                    message: "Details updated",
                    status: "success",
                });
            } else {
                await db.rollback();
                res.json({
                    message: "Error occured",
                    status: "failed",
                });
            }

        } else {
            var addDealerInformation = await User.addDealerInformation(req.body);
            console.log('update ==>', addDealerInformation);
            if (addDealerInformation.insertId) {
                await db.commit();
                res.json({
                    message: "Details added",
                    status: "success",
                });
            } else {
                await db.rollback();
                res.json({
                    message: "Error occured",
                    status: "failed",
                });
            }
        }



    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.logout = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        if (req.body.appType == 'sfa') {
            var logout = await User.logout_salesUser(req.body);
            // console.log('logout',logout);    
        } else {
            var logout = await User.logout(req.body);
            // console.log('logout',logout);    
        }

        if (logout.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Logout",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Logout",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteAccount = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();

        if (req.body.appType == 'sfa') {
            var deleteAccount = await User.deleteAccount_salesUser(req.body);
            // console.log('deleteAccount',deleteAccount);
        } else {
            var deleteAccount = await User.deleteAccount(req.body);
            // console.log('deleteAccount',deleteAccount);
        }


        if (deleteAccount.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Delete Account",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Delete Account",
                status: "error"
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getUserPointList = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var walletCoinsSum = await User.getWalletPoints(req.body);

        var getUserPointListCount = await User.getUserPointListCount(req.body);
        console.log('getUserPointListCount', getUserPointListCount);
        var getUserPointListData = [];
        if (getUserPointListCount > 0) {
            getUserPointListData = await User.getUserPointListData(req.body);
            // console.log('getUserPointListData',getUserPointListData);
        }

        res.status(200).json({
            message: "Point List Data",
            status: "success",
            result: {
                data: getUserPointListData,
                count: getUserPointListCount,
                walletPoints: walletCoinsSum
            }

        });

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getUserWalletDetail = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;

        var walletCoinsSum = await User.getWalletPoints(req.body);


        var walletDetailArray = [
            { "type": "General", "listCount": 0 },
            { "type": "Coupon Scan", "listCount": 0 }
        ]

        if (walletDetailArray.length) {
            for (let index = 0; index < walletDetailArray.length; index++) {
                const element = walletDetailArray[index];
                req.body.walletDetailType = element.type;
                var walletCoinsSum = await User.getWalletPoints(req.body);
                element.walletCoinsSum = walletCoinsSum ? walletCoinsSum : 0;
                var getUserPointListCount = await User.getUserPointListCount(req.body);
                console.log('getUserPointListCount', getUserPointListCount);
                element.listCount = getUserPointListCount;
                console.log('element', element);
                var getUserPointListData = [];
                if (getUserPointListCount > 0) {
                    getUserPointListData = await User.getUserPointListData(req.body);
                    // console.log('getUserPointListData',getUserPointListData);
                    element.listData = getUserPointListData;
                }
                if (index == walletDetailArray.length - 1) {
                    res.status(200).json({
                        message: "Point List Data",
                        status: "success",
                        result: {
                            walletDetail: walletDetailArray,
                            walletPoints: walletCoinsSum
                        }

                    });
                }
            }
        }

    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addManualCoupon = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);

        // var checkCouponCodeValid = await User.checkCouponCodeValid(req.body);
        // // console.log('checkCouponCodeValid',checkCouponCodeValid);

        // if(checkCouponCodeValid.checkCouponCodeAvailable == false){
        //     res.status(200).json({
        //         message: "Coupon code expired",
        //         status: "error", 
        //     });
        //     return false;
        // }

        // var checkCouponOfferActive = await User.checkCouponOfferActive(checkCouponCodeValid);
        // console.log('checkCouponOfferActive',checkCouponOfferActive);

        // if(checkCouponOfferActive == false){
        //     res.status(200).json({
        //         message: "Offer is not active",
        //         status: "error", 
        //     });
        //     return false;
        // }

        var checkCouponCodeAvailable = await User.checkCouponCodeAvailable(req.body);
        console.log('checkCouponCodeAvailable', checkCouponCodeAvailable);

        // if (checkCouponCodeAvailable.id == undefined) {
        //     res.status(200).json({
        //         message: "Coupon code already used",
        //         status: "error",
        //     });
        //     return false;
        // }
        if (checkCouponCodeAvailable.used_by_retailer !== 0) {
            res.status(200).json({
                message: `Coupon code already used by retailer -> ${checkCouponCodeAvailable.used_by_retailer_name}`,
                status: "error",
            });
            return false;
        }
        

        req.body.coupon_id = checkCouponCodeAvailable.id ? checkCouponCodeAvailable.id : 0;
        req.body.product_id = checkCouponCodeAvailable.product_id ? checkCouponCodeAvailable.product_id : 0;
        req.body.offer_id = checkCouponCodeAvailable.offer_id ? checkCouponCodeAvailable.offer_id : 0;

        req.body.retailer_point = checkCouponCodeAvailable.retailer_point ? checkCouponCodeAvailable.retailer_point : 0;
        req.body.distributor_point = checkCouponCodeAvailable.distributor_point ? checkCouponCodeAvailable.distributor_point : 0;
        req.body.sales_user_point = checkCouponCodeAvailable.sales_user_point ? checkCouponCodeAvailable.sales_user_point : 0;

        var getDealerDataInfo = await User.getDealerDataInfo(req.body);
        var getAssignedSEDataInfo = await User.getAssignedSEDataInfo(req.body);

        req.body.used_by_distributor = getDealerDataInfo.length ? getDealerDataInfo[0].dealer_id : 0;
        req.body.used_by_distributor_name = getDealerDataInfo.length ? getDealerDataInfo[0].name : '';
        req.body.used_by_distributor_mobile = getDealerDataInfo.length ? getDealerDataInfo[0].mobile : '';

        req.body.used_by_sales_user = getAssignedSEDataInfo.length ? getAssignedSEDataInfo[0].user_id : 0;
        req.body.used_by_sales_user_name = getAssignedSEDataInfo.length ? getAssignedSEDataInfo[0].user_name : '';
        req.body.used_by_sales_user_mobile = getAssignedSEDataInfo.length ? getAssignedSEDataInfo[0].user_mobile : '';

        var redeemCouponCode = await User.redeemCouponCode(req.body);
        // console.log('redeemCouponCode', redeemCouponCode);

        // retailer code start
        if (req.body.sfa_app_user_id) {
            var sendWalletPointsData = {
                customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                type: 'coupon_code',
                point: req.body.retailer_point ? req.body.retailer_point : 0,
                coupon_id: req.body.coupon_id ? req.body.coupon_id : 0,
                offer_id: req.body.offer_id ? req.body.offer_id : 0,
                product_id: req.body.product_id ? req.body.product_id : 0
            }

            console.log('sendWalletPointsData user_id -->', sendWalletPointsData);
            var sendWalletPoints = await User.sendWalletPoints(sendWalletPointsData);
            console.log('sendWalletPoints', sendWalletPoints);
        }

        // retailer code end


        // distributor code start
        if (req.body.used_by_distributor || 1 == 1) {
            var sendWalletPointsData = {
                customer_id: req.body.used_by_distributor ? req.body.used_by_distributor : 0,
                against_customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                type: 'coupon_code',
                point: req.body.distributor_point ? req.body.distributor_point : 0,
                coupon_id: req.body.coupon_id ? req.body.coupon_id : 0,
                offer_id: req.body.offer_id ? req.body.offer_id : 0,
                product_id: req.body.product_id ? req.body.product_id : 0
            }

            console.log('sendWalletPointsData distributor -->', sendWalletPointsData);
            var sendWalletPoints = await User.sendWalletPoints(sendWalletPointsData);
            console.log('sendWalletPoints', sendWalletPoints);
        }
        // distributor code end

        // sales user code start
        if (req.body.used_by_sales_user || 1 == 1) {
            var sendWalletPointsData = {
                user_id: req.body.used_by_sales_user ? req.body.used_by_sales_user : 0,
                // customer_id : req.body.used_by_distributor ? req.body.used_by_distributor : 0,
                against_customer_id: req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0,
                type: 'coupon_code',
                point: req.body.sales_user_point ? req.body.sales_user_point : 0,
                coupon_id: req.body.coupon_id ? req.body.coupon_id : 0,
                offer_id: req.body.offer_id ? req.body.offer_id : 0,
                product_id: req.body.product_id ? req.body.product_id : 0
            }
            console.log('sendWalletPointsData sales user -->', sendWalletPointsData);
            var sendWalletPoints = await User.sendWalletPoints(sendWalletPointsData);
            console.log('sendWalletPoints', sendWalletPoints);
        }

        // sales user code end


        if (redeemCouponCode.affectedRows && sendWalletPoints.insertId) {
            await db.commit();
            var message = req.body.retailer_point + " points has been added into your wallet.";
            res.status(200).json({
                message: message,
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Coupon Code not redeemed",
                status: "error",
            });
        }


    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.base64ToFile = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    // console.log('here noww');

    // console.log('herererer');
    // var base64ToFile = await User.base64ToFile(req.body);
    // console.log('base64ToFile', base64ToFile);

    var base64ToImage = await base64ToFilePath.base64ToImage(req.body);
    console.log('base64ToImage', base64ToImage);

    res.status(200).json({
        message: 'Uploaded',
        status: "success",
        response: base64ToImage
    });

};

exports.unlinkFile = async function (req, res) {

    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }

    // var unlinkFile = await User.unlinkFile(req.body);
    // console.log('unlinkFile', unlinkFile);

    var unlinkFile = await base64ToFilePath.removebase64image(req.body);
    console.log('unlinkFile', unlinkFile);

    res.status(200).json({
        message: 'Unlinked',
        status: "success",
        response: unlinkFile
    });
};

function generateRandomString(n) {
    let randomString = '';
    let characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';

    for (let i = 0; i < n; i++) {
        randomString += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return randomString;
}
