const SFA_masterModel = require("./../../models/web/SFA_masterModel");
const Network = require("./../../models/web/networkModel");
const Order = require("./../../models/web/orderModel");
const User = require("./../../models/web/userModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');
const Notification = require("../../models/web/notificationModel");


function getDaysInMonth(year, month) {
    console.log('year',year);
    console.log('month',month);
    
    return new Date(year, month, 0).getDate();
}

exports.getAttendanceDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        req.body.image_app_path = res.locals.image_app_path;
        
        var monthsArray = [];
        var monthName = new Array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");
        var d = new Date();
        d.setDate(1);
        for (let index = moment(new Date()).format('M'); index >= 1; index--) {
            // console.log(monthName[d.getMonth()] + ' ' + d.getFullYear());
            var monthNumber = moment().month(monthName[d.getMonth()]).format("MM");
            monthsArray.push({'display_value' : monthName[d.getMonth()] + ' ' + d.getFullYear(), 'value' : d.getFullYear()+'-'+monthNumber})
            d.setMonth(d.getMonth() - 1);
        }
        
        // console.log('monthsArray',monthsArray);
        const daysInMonth = getDaysInMonth( moment(req.body.month).format('YYYY'),  moment(req.body.month).format('MM'));
        console.log('daysInMonth',daysInMonth); 
        
        var monthDateArray = [];
        for (let index = 1 ; index <= daysInMonth; index++) {
            index = (index < 10) ? '0' + index.toString() : index.toString();
            monthDateArray.push(moment(req.body.month).format('YYYY')+'-'+moment(req.body.month).format('MM')+'-'+index)
        }    
        
        if(req.body.type == 'month'){
            
            req.body.monthDateArray = monthDateArray;
            var getAttendanceDataList =  await SFA_masterModel.getAttendanceMonthDataList(req.body);
            var getAttendanceDataCount = getAttendanceDataList.length;
            
        }else{
            var getTotalSalesExecutiveUser =  await SFA_masterModel.getTotalSalesExecutiveUser(req.body)
            var getAttendanceDataCount = await SFA_masterModel.getAttendanceDataCount(req.body)
            console.log('getAttendanceDataCount',getAttendanceDataCount);
            
            
            var getAttendanceTodayDataCount = await SFA_masterModel.getAttendanceTodayDataCount(req.body)
            console.log('getAttendanceTodayDataCount',getAttendanceTodayDataCount);
            
            var getAttendanceDataList = [];
            if(getAttendanceDataCount > 0){
                getAttendanceDataList =  await SFA_masterModel.getAttendanceDataList(req.body);
                // console.log('getAttendanceDataList',getAttendanceDataList);
            }
        }
        
        
        res.status(200).json({
            message: "Attendance List",
            status: "success",
            result : {
                data: getAttendanceDataList,
                totalCount : getAttendanceDataCount,
                attendanceCount : {
                    totalMember : getTotalSalesExecutiveUser,
                    totalPresent : getAttendanceTodayDataCount
                },
                months : monthsArray,
                monthDates : monthDateArray
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getAttendanceDataListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.limit = 1000000;
        req.body.start = 0;
        var getAttendanceDataList =  await SFA_masterModel.getAttendanceDataList(req.body);
        // console.log('getAttendanceDataList',getAttendanceDataList);
        
        var getAttendanceDataListExcel = [];
        if(getAttendanceDataList.length){
            var i = getAttendanceDataList.length - 1;
            
            // console.log('filter', req.body.filter.cust_type_id);
            
            var file_name = 'Attendance-Report-Data';
            
            getAttendanceDataList.forEach(async (element,index) => {
                
                
                if(req.body.filter.attend_date){
                    var data = {
                        'S.No.' : index + 1, 
                        'Emp Code' : element.emp_code ? element.emp_code : '-',
                        'Reporting Head' : element.reporting_manager ? element.reporting_manager : '',
                        'Name' : element.user_name ? element.user_name : '-',
                        'Date': element.attend_date ? moment(element.attend_date).format('YYYY-MM-DD') : '',
                        'Office Location' : element.branch ? element.branch : '-',
                        'Visits': element.checkin_count ? element.checkin_count : 0, 
                    };
                }else{
                    var data = {
                        'S.No.' : index + 1, 
                        'Emp Code' : element.emp_code ? element.emp_code : '-',
                        'Office Location' : element.branch ? element.branch : '-',
                        'Name' : element.user_name ? element.user_name : '-',
                        'Date': element.attend_date ? moment(element.attend_date).format('YYYY-MM-DD') : '',
                        'Visits': element.checkin_count ? element.checkin_count : 0, 
                        'Distance (KM)': element.google_km ? element.google_km : 0,
                    };
                }
                
                getAttendanceDataListExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Attendance List Excel",
                        status: "success",
                        result : {
                            data: getAttendanceDataListExcel,
                            file_name : file_name
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Attendance List Excel",
                status: "success",
                result : {
                    data: getAttendanceDataListExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getAttendanceDwrData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        var dwrReportData = [];
        var mapLatLngUserData = [];
        
        req.body.attendance_id = decrypt(req.body.attendance_id)
        
        var getAttendanceDataDetail = await SFA_masterModel.getAttendanceDataDetail(req.body);
        // console.log('getAttendanceDataDetail',getAttendanceDataDetail);
        
        if(getAttendanceDataDetail.id){
            
            req.body.report_date = getAttendanceDataDetail.attend_date;
            req.body.user_id = getAttendanceDataDetail.user_id;
            
            getAttendanceDataDetail.counterData = {
                checkinCount : await SFA_masterModel.getCheckinDataCount({'filter' : {'user_id' : req.body.user_id, 'checkin_date' : req.body.report_date}}),
                followupCount :  await SFA_masterModel.getFollowupDataCount({'filter' : {'user_id' : req.body.user_id, 'followup_date' : req.body.report_date}}),
                expenseCount : await SFA_masterModel.getExpenseDataCount({'filter' : {'user_id' : req.body.user_id, 'created_on' : req.body.report_date}}),
                leaveCount : await SFA_masterModel.getLeaveDataCount({'filter' : {'user_id' : req.body.user_id,  'created_on' : req.body.report_date}}),
                primaryOrderCount : await Order.getOrderCount({'filter' : {'user_id' : req.body.user_id, 'order_type' : 'Primary',  'created_on' : req.body.report_date}}),
                secondaryOrderCount : await Order.getOrderCount({'filter' : {'user_id' : req.body.user_id, 'order_type' : 'Secondary',  'created_on' : req.body.report_date}}),
            };
            
            
            var attendanceStartData = {
                start_address : getAttendanceDataDetail.start_address,
                start_lat : getAttendanceDataDetail.start_lat,
                start_lng : getAttendanceDataDetail.start_lng,
                start_time : getAttendanceDataDetail.start_time
            }
            
            mapLatLngUserData.push({'lat' : getAttendanceDataDetail.start_lat, 'lng' : getAttendanceDataDetail.start_lng, 'address' : getAttendanceDataDetail.start_address, 'detail' : attendanceStartData});
            dwrReportData.push(attendanceStartData);
        }
        
        req.body.limit = 1000;
        req.body.start = 0;
        req.body.filter = {
            checkin_date : req.body.report_date,
            user_id : req.body.user_id,
            order_by : 'ASC'
        };
        
        var getCheckinListData =  await SFA_masterModel.getCheckinDataList(req.body);
        // console.log('getCheckinListData',getCheckinListData.length);
        
        if(getCheckinListData.length){
            getCheckinListData.map(x=>{
                mapLatLngUserData.push({'lat' : x.start_lat, 'lng' : x.start_lng, 'address' : x.start_address, 'detail' : x});
                mapLatLngUserData.push({'lat' : x.stop_lat, 'lng' : x.stop_lng, 'address' : x.stop_address, 'detail' : x});
                dwrReportData.push(x);
            });
        }
        
        if(getAttendanceDataDetail.id && getAttendanceDataDetail.stop_lat){
            // console.log('here');
            var attendanceStartData = {
                stop_address : getAttendanceDataDetail.stop_address,
                stop_lat : getAttendanceDataDetail.stop_lat,
                stop_lng : getAttendanceDataDetail.stop_lng,
                stop_time : getAttendanceDataDetail.stop_time
            }
            mapLatLngUserData.push({'lat' : getAttendanceDataDetail.stop_lat, 'lng' : getAttendanceDataDetail.stop_lng, 'address' : getAttendanceDataDetail.stop_address, 'detail' : attendanceStartData});
            dwrReportData.push(attendanceStartData);
        }
        
        
        res.status(200).json({
            message: "Attendance DWR Data",
            status: "success",
            result : {
                data: dwrReportData,
                attendanceData : getAttendanceDataDetail,
                mapLatLng : mapLatLngUserData
            }
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getCheckinDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        req.body.image_app_path = res.locals.image_app_path;
        
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var getCheckinDataCount = await SFA_masterModel.getCheckinDataCount(req.body)
        
        var getCheckinDataList = [];
        if(getCheckinDataCount > 0){
            getCheckinDataList =  await SFA_masterModel.getCheckinDataList(req.body);
            // console.log('getCheckinDataList',getCheckinDataList);
        }
        
        res.status(200).json({
            message: "Checkin List",
            status: "success",
            result : {
                data: getCheckinDataList,
                totalCount : getCheckinDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getCheckinDataListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.limit = 1000000;
        req.body.start = 0;
        
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var  getCheckinDataList =  await SFA_masterModel.getCheckinDataList(req.body);
        // console.log('getCheckinDataList',getCheckinDataList);
        
        
        var getCheckinDataListExcel = [];
        if(getCheckinDataList.length){
            var i = getCheckinDataList.length - 1;
            
            // console.log('filter', req.body.filter.cust_type_id);
            
            var file_name = 'Checkin-Report-Data';
            
            getCheckinDataList.forEach(async (element,index) => {
                
                var data = {
                    'S.No.' : index + 1, 
                    'Emp Code' : element.emp_code ? element.emp_code : '-',
                    'Visited By' :  element.user_name ? element.user_name : '-',
                    'Reporting Head' : element.reporting_manager ? element.reporting_manager : '-',
                    'Office Location' : element.branch ? element.branch : '-',
                    'Customer' :  element.company_name ? element.company_name : '-',
                    'Date' : element.checkin_date ? moment(element.checkin_date).format('YYYY-MM-DD') : '',
                    'Check-In Time' : element.start_time ? moment(element.start_time).format('hh:mm A') : '',
                    'Check-Out Time' :  element.stop_time ? moment(element.stop_time).format('hh:mm A') : '',
                    'Outcome Description' : element.remark ? element.remark : '',
                    'Address' : element.start_address ? element.start_address : '',
                    'Territory' : element.branch ? element.branch : ''
                };
                
                getCheckinDataListExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Checkin List Excel",
                        status: "success",
                        result : {
                            data: getCheckinDataListExcel,
                            file_name : file_name
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Checkin List Excel",
                status: "success",
                result : {
                    data: getCheckinDataListExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getLeaveDataListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Pending'},
            {'name' : 'Approved'},
            {'name' : 'Rejected'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getLeaveDataCount({'filter' : {'status' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Leave List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Leave List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
    
};


exports.getLeaveDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var getLeaveDataCount = await SFA_masterModel.getLeaveDataCount(req.body)
        
        var getLeaveDataList = [];
        if(getLeaveDataCount > 0){
            getLeaveDataList =  await SFA_masterModel.getLeaveDataList(req.body);
            // console.log('getLeaveDataList',getLeaveDataList);
        }
        
        res.status(200).json({
            message: "Leave List",
            status: "success",
            result : {
                data: getLeaveDataList,
                totalCount : getLeaveDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateLeaveStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var getLeaveDataDetail = await SFA_masterModel.getLeaveDataDetail(req.body)
        // console.log('getLeaveDataDetail',getLeaveDataDetail);
        
        var updateLeaveStatus = await SFA_masterModel.updateLeaveStatus(req.body)
        
        if(req.body.status == 'Approved' && req.body.leave_id){
            
            var title = 'Hey, your leave status is Approved by the admin';
            var description = 'Enjoy your Leave';
            var sendNotificationDataToUserBody = {
                user_id: getLeaveDataDetail.user_id ? getLeaveDataDetail.user_id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'leave_approved',
                leave_id : req.body.leave_id
            }
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        if(updateLeaveStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Leave status updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Leave status not updated",
                status: "success",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getTravelPlanTypeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var  getTravelPlanTypeData =  await SFA_masterModel.getTravelPlanTypeData(req.body);
        // console.log('getTravelPlanTypeData',getTravelPlanTypeData);
        
        res.status(200).json({
            message: "Travel type data",
            status: "success",
            result : {
                data: getTravelPlanTypeData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addTravelPlanData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var addTravelPlanData = await SFA_masterModel.addTravelPlanData(req.body);
        console.log('addTravelPlanData',addTravelPlanData);
        
        
        if(addTravelPlanData.insertId){
            
            var title = 'Hey, new travel plan created for you.';
            var description = 'Enjoy your Travel Plan';
            var sendNotificationDataToUserBody = {
                user_id: req.body.sales_user.id ? req.body.sales_user.id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'travel_plan_add',
                travel_plan_id : addTravelPlanData.insertId?addTravelPlanData.insertId:0,
                redirect : 1,
                route : addTravelPlanData.insertId?'travel-detail;id='+addTravelPlanData.insertId:0
            };
            
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        
        if(addTravelPlanData.insertId){
            await db.commit();
            res.status(200).json({
                message: "Travel Plan data added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Travel Plan data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getTravelPlanListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Pending'},
            {'name' : 'Approved'},
            {'name' : 'Rejected'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getTravelPlanDataCount({'filter' : {'status' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Travel List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Travel List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
    
};


exports.getTravelPlanDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var getTravelPlanDataCount = await SFA_masterModel.getTravelPlanDataCount(req.body)
        
        var getTravelPlanDataList = [];
        if(getTravelPlanDataCount > 0){
            getTravelPlanDataList =  await SFA_masterModel.getTravelPlanDataList(req.body);
            // console.log('getTravelPlanDataList',getTravelPlanDataList);
        }
        
        res.status(200).json({
            message: "Travel Plan List",
            status: "success",
            result : {
                data: getTravelPlanDataList,
                totalCount : getTravelPlanDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateTravelPlanStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var updateTravelPlanStatus = await SFA_masterModel.updateTravelPlanStatus(req.body)
        
        if(updateTravelPlanStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Travel Plan status updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Travel Plan status not updated",
                status: "success",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.detailTravelPlanData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        req.body.travel_plan_id = decrypt(req.body.travel_plan_id);
        
        
        var detailTravelPlanData = await SFA_masterModel.detailTravelPlanData(req.body);
        console.log('detailTravelPlanData',detailTravelPlanData);
        
        await db.commit();
        res.status(200).json({
            message: "Travel Plan data detail",
            status: "success",
            data : {
                detail : detailTravelPlanData
            }
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCustomerNetworkType = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        var getCustomerNetworkTypeListData = [];
        getCustomerNetworkTypeListData =  await SFA_masterModel.getCustomerNetworkTypeList(req.body);
        // console.log('getCustomerNetworkTypeListData',getCustomerNetworkTypeListData);
        
        res.status(200).json({
            message: "Customer network type list",
            status: "success",
            result : {
                data: getCustomerNetworkTypeListData
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getDistributorNetworkList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getDistributorNetworkCount = await SFA_masterModel.getDistributorNetworkCount(req.body)
        
        var getDistributorNetworkList = [];
        if(getDistributorNetworkCount > 0){
            getDistributorNetworkList =  await SFA_masterModel.getDistributorNetworkList(req.body);
            // console.log('getDistributorNetworkList',getDistributorNetworkList);
        }
        
        res.status(200).json({
            message: "Network List",
            status: "success",
            result : {
                data: getDistributorNetworkList,
                totalCount : getDistributorNetworkCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSalesUserList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.role = 2;
        var getSalesUserCount = await SFA_masterModel.getSalesUserCount(req.body)
        
        var getSalesUserList = [];
        if(getSalesUserCount > 0){
            getSalesUserList =  await SFA_masterModel.getSalesUserList(req.body);
            // console.log('getSalesUserList',getSalesUserList);
        }
        
        res.status(200).json({
            message: "Sales User List",
            status: "success",
            result : {
                data: getSalesUserList,
                totalCount : getSalesUserCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getReportingManagerUserList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // req.body.role = 5;
        var getSalesUserCount = await SFA_masterModel.getSalesUserCount(req.body)
        
        var getSalesUserList = [];
        if(getSalesUserCount > 0){
            getSalesUserList =  await SFA_masterModel.getSalesUserList(req.body);
            // console.log('getSalesUserList',getSalesUserList);
        }
        
        res.status(200).json({
            message: "Sales User List",
            status: "success",
            result : {
                data: getSalesUserList,
                totalCount : getSalesUserCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getFollowupListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Today'},
            {'name' : 'Upcoming'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getFollowupDataCount({'filter' : {'followup_type' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Followup List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Followup List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
    
};


exports.getFollowupDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var getFollowupDataCount = await SFA_masterModel.getFollowupDataCount(req.body)
        
        var getFollowupDataList = [];
        if(getFollowupDataCount > 0){
            getFollowupDataList =  await SFA_masterModel.getFollowupDataList(req.body);
            // console.log('getFollowupDataList',getFollowupDataList);
        }
        
        res.status(200).json({
            message: "Followup List",
            status: "success",
            result : {
                data: getFollowupDataList,
                totalCount : getFollowupDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addFollowUpData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var addFollowUpData = await SFA_masterModel.addFollowUpData(req.body);
        console.log('addFollowUpData',addFollowUpData);
        
        
        if(addFollowUpData.insertId){
            
            var title = 'Hey, new followup created for you.';
            var description = 'Enjoy your followup';
            var sendNotificationDataToUserBody = {
                user_id: req.body.sales_user.id ? req.body.sales_user.id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'followup_add',
                followup_id : addFollowUpData.insertId?addFollowUpData.insertId:0,
                redirect : 1,
                route : addFollowUpData.insertId?'followup-detail;id='+addFollowUpData.insertId:0
            };
            
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        
        if(addFollowUpData.insertId){
            await db.commit();
            res.status(200).json({
                message: "Followup data added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Followup data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateFollowupData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var getLeaveDataDetail = await SFA_masterModel.getLeaveDataDetail(req.body)
        // console.log('getLeaveDataDetail',getLeaveDataDetail);
        
        var updateFollowupData = await SFA_masterModel.updateFollowupData(req.body)
        
        if(req.body.followup_id){
            
            var title = 'Hey, followup created for you.';
            var description = 'Enjoy your followup';
            var sendNotificationDataToUserBody = {
                user_id: req.body.sales_user.id ? req.body.sales_user.id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'followup_update',
                followup_id : req.body.followup_id ? req.body.followup_id : 0
            }
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        if(updateFollowupData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Leave status updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Leave status not updated",
                status: "success",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.deleteFollowupData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var deleteFollowupData = await SFA_masterModel.deleteFollowupData(req.body)
        
        if(deleteFollowupData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Followup data deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Followup data not deleted",
                status: "error"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getMasterExpenseTypeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var data ={
            type:[
                {label:'Local Conveyance',value:'Local Conveyance'},
                {label:'Outstation Travel',value:'Outstation Travel'},
                {label:'Other',value:'Other'}
                // {label:'Sales Promotion Activity',value:'Sales Promotion Activity'},
                // {label:'Outstation Travel',value:'Outstation Travel'},
                // {label:'Misc Exp.',value:'Misc Exp.'},
                // {label:'Local Conveyance',value:'Local Conveyance'},
            ],
            travel_mode:[
                {label:'Car',value:'Car'},
                {label:'Bike',value:'Bike'},
            ],
            conveyance_travel_mode:[
                {label:'Car',value:'Car'},
                {label:'Bike',value:'Bike'},
            ],
            nature_of_expense:[
                {label:'Car',value:'Car'},
                {label:'Bike',value:'Bike'},
            ]
        };
        
        res.status(200).json({
            message: "Master Expense Type List",
            status: "success",
            result : {
                data: data,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getExpenseDataCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Pending'},
            {'name' : 'Approved'},
            {'name' : 'Rejected'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                
                element.count = await SFA_masterModel.getExpenseDataCount({'filter' : {'status' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Expense List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            await db.commit();
            res.status(200).json({
                message: "Expense List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getExpenseDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var getExpenseDataCount = await SFA_masterModel.getExpenseDataCount(req.body)
        
        var getExpenseDataList = [];
        if(getExpenseDataCount > 0){
            getExpenseDataList =  await SFA_masterModel.getExpenseDataList(req.body);
            // console.log('getExpenseDataList',getExpenseDataList);
        }
        
        res.status(200).json({
            message: "Expense List",
            status: "success",
            result : {
                data: getExpenseDataList,
                totalCount : getExpenseDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getExpenseDataDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.expense_id = decrypt(req.body.expense_id);
        
        var getExpenseData = await SFA_masterModel.getExpenseData(req.body)
        
        var getExpenseDataDetail =  await SFA_masterModel.getExpenseDataDetail(req.body);
        // console.log('getExpenseDataList',getExpenseDataList);
        
        res.status(200).json({
            message: "Expense Detail",
            status: "success",
            result : {
                expenseData: getExpenseData,
                expenseDataDetail : getExpenseDataDetail
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.addExpenseData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var addExpenseData = await SFA_masterModel.addExpenseData(req.body);
        console.log('addExpenseData',addExpenseData);
        
        if(addExpenseData.insertId){
            
            var title = 'Hey, new expense created for you.';
            var description = 'Enjoy your expense';
            var sendNotificationDataToUserBody = {
                user_id: req.body.user_id.id ? req.body.user_id.id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'expense_add',
                expense_id : addExpenseData.insertId?addExpenseData.insertId:0,
                redirect : 1,
                route : addExpenseData.insertId?'expenses-detail;id='+addExpenseData.insertId:0
            };
            
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
            
            if(req.body.misc_array?.length){
                
                for (let index = 0; index < req.body.misc_array.length; index++) {
                    const element = req.body.misc_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'MISC';
                    element.sfa_app_user_id =req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type =req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name =req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';
                    
                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData',addExpenseDetailData);
                }
            }
            
            if(req.body.local_conveyance_array?.length){
                for (let index = 0; index < req.body.local_conveyance_array.length; index++) {
                    const element = req.body.local_conveyance_array[index];
                    element.expense_type = 'Local Conveyance';
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.sfa_app_user_id =req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type =req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name =req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';
                    
                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData',addExpenseDetailData);
                }
            }
            
            if(req.body.food_array?.length){
                for (let index = 0; index < req.body.food_array.length; index++) {
                    const element = req.body.food_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Food';
                    element.sfa_app_user_id =req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type =req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name =req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';
                    
                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData',addExpenseDetailData);
                }
            }
            
            if(req.body.hotel_array?.length){
                for (let index = 0; index < req.body.hotel_array.length; index++) {
                    const element = req.body.hotel_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Hotel';
                    element.sfa_app_user_id =req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type =req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name =req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';
                    
                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData',addExpenseDetailData);
                }
            }
            
            
            if(req.body.travel_entitlement_array?.length){
                for (let index = 0; index < req.body.travel_entitlement_array.length; index++) {
                    const element = req.body.travel_entitlement_array[index];
                    element.expense_id = addExpenseData.insertId ? addExpenseData.insertId : 0;
                    element.expense_type = 'Travel Entitlement';
                    element.sfa_app_user_id =req.body.sfa_app_user_id ? req.body.sfa_app_user_id : 0;
                    element.sfa_app_user_type =req.body.sfa_app_user_type ? req.body.sfa_app_user_type : '';
                    element.sfa_app_user_name =req.body.sfa_app_user_name ? req.body.sfa_app_user_name : '';
                    
                    var addExpenseDetailData = await SFA_masterModel.addExpenseDetailData(element);
                    console.log('addExpenseDetailData',addExpenseDetailData);
                }
            }
            
            await db.commit();
            res.status(200).json({
                message: "Expense data added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Expense data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateExpenseStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction(); 
        
        var getExpenseData = await SFA_masterModel.getExpenseData(req.body);
        console.log('getExpenseData',getExpenseData);
        
        
        var updateExpenseStatus = await SFA_masterModel.updateExpenseStatus(req.body)
        
        if(req.body.status == 'Approved' && req.body.expense_id){
            
            var title = 'Hey, your expense status is Approved by the admin';
            var description = 'Enjoy your app';
            var sendNotificationDataToUserBody = {
                user_id: getExpenseData.user_id ? getExpenseData.user_id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'expense_approved',
                expense_id : req.body.expense_id
            }
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        if(req.body.status == 'Rejected' && req.body.expense_id){
            
            var title = 'Hey, your expense status is Rejected by the admin';
            var description = 'Enjoy your app';
            var sendNotificationDataToUserBody = {
                user_id: getExpenseData.user_id ? getExpenseData.user_id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'expense_rejected',
                expense_id : req.body.expense_id
            }
            // console.log('sendNotificationDataToUserBody',sendNotificationDataToUserBody);
            // var sendNotificationDataToUser =  await Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            var sendNotificationDataToUser =  Notification.sendNotificationDataToUser(sendNotificationDataToUserBody);
            console.log('sendNotificationDataToUser',sendNotificationDataToUser);
            
        }
        
        
        if(updateExpenseStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Expense status updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Expense status not updated",
                status: "success",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getEnquiryDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getEnquiryDataCount = await SFA_masterModel.getEnquiryDataCount(req.body)
        
        var getEnquiryDataList = [];
        if(getEnquiryDataCount > 0){
            getEnquiryDataList =  await SFA_masterModel.getEnquiryDataList(req.body);
            // console.log('getEnquiryDataList',getEnquiryDataList);
        }
        
        res.status(200).json({
            message: "Enquiry List",
            status: "success",
            result : {
                data: getEnquiryDataList,
                totalCount : getEnquiryDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getPopAndGiftIssueTypeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var  getPopAndGiftIssueTypeData =  await SFA_masterModel.getPopAndGiftIssueTypeData(req.body);
        // console.log('getPopAndGiftIssueTypeData',getPopAndGiftIssueTypeData);
        
        res.status(200).json({
            message: "Pop and Gift issue type data",
            status: "success",
            result : {
                data: getPopAndGiftIssueTypeData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getPopAndGiftDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        
        var getPopAndGiftDataCount = await SFA_masterModel.getPopAndGiftDataCount(req.body)
        
        var getPopAndGiftDataList = [];
        if(getPopAndGiftDataCount > 0){
            getPopAndGiftDataList =  await SFA_masterModel.getPopAndGiftDataList(req.body);
            // console.log('getPopAndGiftDataList',getPopAndGiftDataList);
        }
        
        res.status(200).json({
            message: "Pop And Gift List",
            status: "success",
            result : {
                data: getPopAndGiftDataList,
                totalCount : getPopAndGiftDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.addPopAndGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var addPopAndGiftData = await SFA_masterModel.addPopAndGiftData(req.body);
        console.log('addPopAndGiftData',addPopAndGiftData);
        
        if(addPopAndGiftData.insertId){
            
            var stockData = {
                pop_gift_id : addPopAndGiftData.insertId ? addPopAndGiftData.insertId : 0,
                stock : req.body.qty ? req.body.qty : 0,
                type : 'stockin',
                remark : 'Stock Added'
            };
            
            var popAndGiftStockLedgerMaintaindata = await SFA_masterModel.popAndGiftStockLedgerMaintaindata(stockData);                        
            console.log('popAndGiftStockLedgerMaintaindata',popAndGiftStockLedgerMaintaindata);
            
            await db.commit();
            res.status(200).json({
                message: "Pop And Gift data added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Pop And Gift data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updatePopAndGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.pop_gift_id = decrypt(req.body.pop_gift_id);
        
        var updatePopAndGiftData = await SFA_masterModel.updatePopAndGiftData(req.body);
        console.log('updatePopAndGiftData',updatePopAndGiftData);
        
        if(updatePopAndGiftData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Pop And Gift data updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Pop And Gift data not updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updatePopAndGiftStockData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.pop_gift_id = decrypt(req.body.pop_gift_id);
        
        var updatePopAndGiftStockData = await SFA_masterModel.updatePopAndGiftStockData(req.body);
        console.log('updatePopAndGiftStockData',updatePopAndGiftStockData);
        
        if(updatePopAndGiftStockData.affectedRows){
            
            var stockData = {
                pop_gift_id : req.body.pop_gift_id ? req.body.pop_gift_id : 0,
                stock : req.body.qty ? req.body.qty : 0,
                type : 'stockin',
                remark : 'Stock Updated'
            };
            
            var popAndGiftStockLedgerMaintaindata = await SFA_masterModel.popAndGiftStockLedgerMaintaindata(stockData);                        
            console.log('popAndGiftStockLedgerMaintaindata',popAndGiftStockLedgerMaintaindata);
            
            await db.commit();
            res.status(200).json({
                message: "Pop And Gift data updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Pop And Gift data not updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deletePopAndGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.pop_gift_id = decrypt(req.body.pop_gift_id);
        
        var deletePopAndGiftData = await SFA_masterModel.deletePopAndGiftData(req.body);
        console.log('deletePopAndGiftData',deletePopAndGiftData);
        
        if(deletePopAndGiftData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Pop And Gift data deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Pop And Gift data not deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.detailPopAndGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        
        req.body.pop_gift_id = decrypt(req.body.pop_gift_id);
        
        var detailPopAndGiftData = await SFA_masterModel.detailPopAndGiftData(req.body);
        console.log('detailPopAndGiftData',detailPopAndGiftData);
        
        var getGiftAssignDataDetail = await SFA_masterModel.getGiftAssignDataDetail(req.body);
        console.log('getGiftAssignDataDetail',getGiftAssignDataDetail);
        
        var getGiftStockDataDetail = await SFA_masterModel.getGiftStockDataDetail(req.body);
        console.log('getGiftStockDataDetail',getGiftStockDataDetail);
        
        var getGiftRequestDetail = await SFA_masterModel.getGiftRequestDetail(req.body);
        console.log('getGiftRequestDetail',getGiftRequestDetail);
        
        await db.commit();
        res.status(200).json({
            message: "Pop and Gift data detail",
            status: "success",
            data : {
                detail : detailPopAndGiftData,
                assignedGiftDetail : getGiftAssignDataDetail,
                stockDataDetail : getGiftStockDataDetail,
                requestedGiftDetail : getGiftRequestDetail
            }
        });
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.assignPopAndGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        if(req.body.itemInfo.length){
            
            var i = req.body.itemInfo.length - 1;
            req.body.itemInfo.forEach(async element => {
                
                element.sfa_web_user_id = req.body.sfa_web_user_id;
                element.sfa_web_user_name = req.body.sfa_web_user_name;
                element.sfa_web_user_type = req.body.sfa_web_user_type;
                element.description = req.body.delivery_note;
                element.pop_gift_id = element.id;
                
                if(req.body.type.cust_type_id){
                    element.customer_id = req.body.typeData.id;
                    element.cust_type_id = req.body.type.cust_type_id;
                    element.name = req.body.typeData.name;
                    element.type = req.body.type.type;
                }else{
                    element.user_id = req.body.typeData.id;
                    element.name = req.body.typeData.name;
                    element.type = req.body.type.type;
                }
                
                var assignPopAndGiftData = await SFA_masterModel.assignPopAndGiftData(element);
                console.log('assignPopAndGiftData',assignPopAndGiftData);
                
                var stockData = {
                    pop_gift_id : element.pop_gift_id ? element.pop_gift_id : 0,
                    stock : element.input_qty ? element.input_qty : 0,
                    gift_assigned_id : assignPopAndGiftData.insertId ? assignPopAndGiftData.insertId : 0,
                    type : 'stockout',
                    remark : 'Stock Assigned'
                };
                
                var popAndGiftStockLedgerMaintaindata = await SFA_masterModel.popAndGiftStockLedgerMaintaindata(stockData);                        
                console.log('popAndGiftStockLedgerMaintaindata',popAndGiftStockLedgerMaintaindata);
                
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Pop and Gift assigned",
                        status: "success",
                    });
                }
                i--;
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Pop and Gift not assigned",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getEventPlanDataCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        var statusArrayData = [
            {'name' : 'Pending'},
            {'name' : 'Approved'},
            {'name' : 'Completed'},
            {'name' : 'Cancelled'},
            {'name' : 'Rejected'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await SFA_masterModel.getEventPlanDataCount({'filter' : {'status' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Event Plan List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            await db.commit();
            res.status(200).json({
                message: "Event Plan List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getEventPlanDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        
        var getEventPlanDataCount = await SFA_masterModel.getEventPlanDataCount(req.body)
        console.log('getEventPlanDataCount',getEventPlanDataCount);
        
        var getEventPlanDataList = [];
        if(getEventPlanDataCount > 0){
            getEventPlanDataList =  await SFA_masterModel.getEventPlanDataList(req.body);
            // console.log('getEventPlanDataList',getEventPlanDataList);
        }
        
        res.status(200).json({
            message: "Event and Plan List",
            status: "success",
            result : {
                data: getEventPlanDataList,
                count : getEventPlanDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateEventPlanStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        // req.body.event_plan_id = decrypt(req.body.event_plan_id);
        
        var updateEventPlanStatus = await SFA_masterModel.updateEventPlanStatus(req.body);
        console.log('updateEventPlanStatus',updateEventPlanStatus);
        
        if(updateEventPlanStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Event Plan status updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Event Plan status not updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.deleteEventPlanData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        
        req.body.event_plan_id = decrypt(req.body.event_plan_id);
        
        var deleteEventPlanData = await SFA_masterModel.deleteEventPlanData(req.body);
        console.log('deleteEventPlanData',deleteEventPlanData);
        
        if(deleteEventPlanData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Event Plan deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Event Plan not deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.detailEventPlanData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        req.body.image_app_path = res.locals.image_app_path;
        req.body.event_plan_id = decrypt(req.body.event_plan_id);
        
        var detailEventPlanData = await SFA_masterModel.detailEventPlanData(req.body);
        console.log('detailEventPlanData',detailEventPlanData);
        
        req.body.type = 'participant';
        var detailEventPlanParticipantData = await SFA_masterModel.detailEventPlanParticipantData(req.body);
        console.log('detailEventPlanParticipantData',detailEventPlanParticipantData);
        
        req.body.type = 'gallery';
        var detailEventPlanGalleryData = await SFA_masterModel.detailEventPlanParticipantData(req.body);
        console.log('detailEventPlanGalleryData',detailEventPlanGalleryData);
        
        await db.commit();
        res.status(200).json({
            message: "Event plan data detail",
            status: "success",
            data : {
                detail : detailEventPlanData,
                participantData : detailEventPlanParticipantData,
                galleryData : detailEventPlanGalleryData,
            }
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getMasterSearchData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        req.body.image_app_path = res.locals.image_app_path;
        
        var getMasterSearchDataList = await SFA_masterModel.getMasterSearchDataList(req.body);
        
        res.status(200).json({
            message: "Master search List",
            status: "success",
            result : {
                data: getMasterSearchDataList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.trackUserLocation = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        // req.body.attendance_id = decrypt(req.body.attendance_encrypt_id)
        req.body.user_id = decrypt(req.body.user_id)
        
        var getSystemUserDetail = await User.getSystemUserDetail(req.body);
        // console.log('getSystemUserDetail',getSystemUserDetail);
        
        var trackUserLocationCount = await SFA_masterModel.trackUserLocationCount(req.body)
        console.log('trackUserLocationCount',trackUserLocationCount);
        
        var trackUserLocationList = [];
        if(trackUserLocationCount > 0){
            trackUserLocationList =  await SFA_masterModel.trackUserLocationList(req.body);
            // console.log('trackUserLocationList',trackUserLocationList);
        }
        
        res.status(200).json({
            message: "Track User List",
            status: "success",
            result : {
                detail : getSystemUserDetail,
                data: trackUserLocationList,
                count : trackUserLocationCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};
