const Dashboard = require("./../../models/web/dashboardModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');

exports.getLmsDashboardData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        
        var getDashboardCountData = await Dashboard.getDashboardCountData(req.body);
        console.log('getDashboardCountData',getDashboardCountData);
        
        
        var getTopDistributorNetwork =  await Dashboard.getTopDistributorNetwork({"filter" : {"cust_type_id" : 4}});
        // console.log('getTopDistributorNetwork',getTopDistributorNetwork);
        
        var getTopRetailerNetwork =  await Dashboard.getTopDistributorNetwork({"filter" : {"cust_type_id" : 10}});
        // console.log('getTopDistributorNetwork',getTopDistributorNetwork);

                
        var getTopDistributorNetwork =  await Dashboard.getTopDistributorNetwork({"filter" : {"cust_type_id" : 1}});
        // console.log('getTopDistributorNetwork',getTopDistributorNetwork);
        

        res.status(200).json({
            message: "Dashboard Data",
            status: "success",
            result : {
                data: {
                    retailerData : getTopRetailerNetwork,
                    distributorData : getTopDistributorNetwork,
                },
                countData : getDashboardCountData
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getLmsStateWiseRetailerData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        req.body.cust_type_id = 10;
        var retailerStateWise = await Dashboard.getDisNetworkDataStateWise(req.body);
        console.log('getLmsStateWiseRetailerData',retailerStateWise);
        
        res.status(200).json({
            message: "Dashboard Graph Data - Retailer State wise",
            status: "success",
            result : {
                carpenterBranchWise : retailerStateWise
            }
            
        });
            
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getLmsBranchWiseArchitectData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {

        req.body.cust_type_id = 1;
        var getArchitectBranchWise = await Dashboard.getDisNetworkDataStateWise(req.body);
        // console.log('getLeadDataStateWise',getLeadDataStateWise);
        
        res.status(200).json({
            message: "Dashboard Graph Data - Distributor Branch wise",
            status: "success",
            result : {
                architectBranchWise : getArchitectBranchWise
            }
            
        });
            
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};




//  SFA code start 

exports.getSfaDashboardData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;

        var getSfaDashboardCountData = await Dashboard.getSfaDashboardCountData(req.body);
        // console.log('getSfaDashboardCountData',getSfaDashboardCountData);
     
        var getLatestCheckinUserData =  await Dashboard.getLatestCheckinUserData(req.body);
        // console.log('getLatestCheckinUserData',getLatestCheckinUserData);
        
        var getTopSalesExecutive =  await Dashboard.getTopSalesExecutive(req.body);
        // console.log('getTopSalesExecutive',getTopSalesExecutive);
        
        res.status(200).json({
            message: "Dashboard Data",
            status: "success",
            result : {
                data: getTopSalesExecutive,
                checkinData : getLatestCheckinUserData,
                leadCountData : getSfaDashboardCountData
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSfaDashboardDataLeadSourceWise = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        var getLeadDataSourceWise = await Dashboard.getLeadDataSourceWise(req.body);
        console.log('getLeadDataSourceWise',getLeadDataSourceWise);
        
        res.status(200).json({
            message: "Dashboard Bar Graph - Source Wise Lead",
            status: "success",
            result : {
                leadSourceBarGraph : getLeadDataSourceWise
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSfaDashboardDataLeadStateWise = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        req.body.is_lead = "1";
        var getLeadDataStateWise = await Dashboard.getDisNetworkDataStateWise(req.body);
        console.log('getLeadDataStateWise',getLeadDataStateWise);
        
        res.status(200).json({
            message: "Dashboard Graph Data - Lead State wise",
            status: "success",
            result : {
                leadStateWiseBarGraph : getLeadDataStateWise
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getSfaDashboardDataDrStateWise = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {

        req.body.is_lead = "0";
        var getLeadDataStateWise = await Dashboard.getDisNetworkDataStateWise(req.body);
        // console.log('getLeadDataStateWise',getLeadDataStateWise);
        
        res.status(200).json({
            message: "Dashboard Graph Data - DR State wise",
            status: "success",
            result : {
                drStateWiseBarGraph : getLeadDataStateWise
            }
            
        });
            
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSfaDashboardDataCustTypeWise = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        req.body.is_lead = "0";
        var getDisNetworkDataCustTypeWise = await Dashboard.getDisNetworkDataCustTypeWise(req.body);
        console.log('getDisNetworkDataCustTypeWise',getDisNetworkDataCustTypeWise);
        
        res.status(200).json({
            message: "Dashboard Graph - Cust Type Pie Chart",
            status: "success",
            result : {
                drNetworkPieChart : getDisNetworkDataCustTypeWise
            }
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getSfaDashboardDataSalesWorkingStateWise = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {        


        var getSalesUserWorkingState = await Dashboard.getSalesUserWorkingState(req.body);
        // console.log('getSalesUserWorkingState',getSalesUserWorkingState);

        var getSalesUserWorkingStateDrillDownGraph = await Dashboard.getSalesUserWorkingStateDrillDownGraph(req.body);
        // console.log('getSalesUserWorkingStateDrillDownGraph',getSalesUserWorkingStateDrillDownGraph);

        var saleUserWorkingStateDrillDownBarGraph = {
            data : getSalesUserWorkingState,
            series: getSalesUserWorkingStateDrillDownGraph
        };
        
        res.status(200).json({
            message: "Dashboard Graph Data",
            status: "success",
            result : {
                saleUserWorkingStateDrillDownBarGraph : saleUserWorkingStateDrillDownBarGraph
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSfaDashboardDataSalesWorkingLocation = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        req.body.for_sales_user = 1;
        var getSalesUserWorkingLatLng = await Dashboard.getSalesUserWorkingLatLng(req.body);
        // console.log('getSalesUserWorkingLatLng',getSalesUserWorkingLatLng);

        res.status(200).json({
            message: "Dashboard Graph Data",
            status: "success",
            result : {
                salesUserMapLocation : getSalesUserWorkingLatLng
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

//  SFA code end 
