const Invoice = require("../../models/web/invoiceModel");
const Network = require("../../models/web/networkModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');


exports.getDrListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getDrList =  await Invoice.getDrList(req.body);
        // console.log('getDrList',getDrList);
        
        res.status(200).json({
            message: "DR List",
            status: "success",
            result : {
                data: getDrList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getDrOrderList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getDrOrderList =  await Invoice.getDrOrderList(req.body);
        // console.log('getDrOrderList',getDrOrderList);
        
        res.status(200).json({
            message: "DR Order List",
            status: "success",
            result : {
                data: getDrOrderList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getDrOrderItemDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // console.log('body ==>', req.body);
        var getDrOrderItemDetail =  await Invoice.getDrOrderItemDetail(req.body);
        // console.log('getDrOrderItemDetail',getDrOrderItemDetail);
        
        res.status(200).json({
            message: "DR Order Item Detail List",
            status: "success",
            result : {
                data: getDrOrderItemDetail,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getInvoiceNo = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getInvoiceNo =  await Invoice.getInvoiceNo(req.body);
        console.log('getInvoiceNo',getInvoiceNo);
        if(getInvoiceNo.invoice_no != null){
            const result = getInvoiceNo.invoice_no.split('-');
            var invoice_no = parseInt(result[2]) + 1;
            // console.log('invoice_no',invoice_no);        
            getInvoiceNo.invoice_no = result[0].concat("-", result[1]).concat("-",invoice_no);
        }else{
            getInvoiceNo.invoice_no = 'SI-INVOICE-0001';
            invoice_no = '0001';
        }
        // add prefix now
        getInvoiceNo.invoice_no = req.body.sfa_web_user_prefix+'-INVOICE-'+invoice_no;
        
        res.status(200).json({
            message: "Invoice number",
            status: "success",
            result : {
                data: getInvoiceNo,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getInvoiceList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getInvoiceCount = await Invoice.getInvoiceCount(req.body)
        
        var getInvoiceList = [];
        if(getInvoiceCount > 0){
            getInvoiceList =  await Invoice.getInvoiceList(req.body);
            // console.log('getInvoiceList',getInvoiceList);
        }
        
        res.status(200).json({
            message: "Invoice List",
            status: "success",
            result : {
                data: getInvoiceList,
                totalCount : getInvoiceCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addInvoiceData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // console.log('body ==>', req.body);
        
        req.body.order_id = req.body.orderIds.toString();
        
        var addInvoiceData =  await Invoice.addInvoiceData(req.body);
        console.log('addInvoiceData',addInvoiceData);
        
        
        if(addInvoiceData.insertId){
            
            var i = req.body.invoiceItem.length - 1;
            if(req.body.invoiceItem.length){
                req.body.invoiceItem.forEach(async element => {
                    element.invoice_id = addInvoiceData.insertId;
                    element.customer_id = req.body.customer_id;
                    
                    var addInvoiceItemData =  await Invoice.addInvoiceItemData(element);
                    // console.log('addInvoiceItemData',addInvoiceItemData);
                    element.order_item_id = element.id;
                    var updateOrderItemQty = await Invoice.updateOrderItemQty(element) 
                    console.log('updateOrderItemQty',updateOrderItemQty);
                    
                    
                    var stockData = {
                        product_id : element.product_id ? element.product_id : 0,
                        customer_id : req.body.customer_id ? req.body.customer_id : 0,
                        stock : element.dispatch_qty ? element.dispatch_qty : 0,
                        invoice_no : req.body.invoice_no ? req.body.invoice_no : '',
                        invoice_id : element.invoice_id ? element.invoice_id : '',
                        type : 'stockout'
                    };
                    
                    var stockLedgerMaintaindata = await Network.stockLedgerMaintaindata(stockData);                        
                    console.log('stockLedgerMaintaindata',stockLedgerMaintaindata);
                    
                    if(i==0){
                        
                        var logData = {
                            customer_id : req.body.customer_id,
                            type : 'Invoice Added',
                            description : 'Invoice added'
                        }
                        var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
                        // console.log('distributionNetworkLogData',distributionNetworkLogData);
                        
                        if(req.body.orderIds.length){
                            req.body.orderIds.forEach(async x => {
                                var checkOrderPendingQty = await Invoice.checkOrderPendingQty({'order_id' : x}) 
                                console.log('checkOrderPendingQty',checkOrderPendingQty, 'order id', x);
                                
                                if(checkOrderPendingQty.pending_qty == 0){
                                    var updateOrderStatus = await Invoice.updateOrderStatus({'order_id' : x}) 
                                    console.log('updateOrderStatus',updateOrderStatus);
                                }
                            });
                        }
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Invoice Created",
                            status: "success",
                        });
                    }
                    i--; 
                });
            }else{
                await db.rollback();
                res.status(200).json({
                    message: "Invoice item empty",
                    status: "error"
                });
            }
            
        }else{
            await db.rollback()
            res.status(200).json({
                message: "Invoice not Created",
                status: "error"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getInvoiceDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.invoice_id = decrypt(req.body.invoice_id);
        
        var getInvoiceBasicDetail =  await Invoice.getInvoiceBasicDetail(req.body);
        // console.log('getInvoiceBasicDetail',getInvoiceBasicDetail);
        var getInvoiceItemDetail =  await Invoice.getInvoiceItemDetail(req.body);
        
        
        await db.commit();
        res.status(200).json({
            message: "Invoice Detail",
            status: "success",
            result : {
                data : getInvoiceBasicDetail,
                orderItem : getInvoiceItemDetail
            }
        });
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.deleteInvoiceData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.invoice_id = decrypt(req.body.invoice_id);
        
        var deleteInvoiceData =  await Invoice.deleteInvoiceData(req.body);
        // console.log('deleteInvoiceData',deleteInvoiceData);
        var deleteInvoiceDataItem =  await Invoice.deleteInvoiceDataItem(req.body);
        console.log('deleteInvoiceDataItem',deleteInvoiceDataItem);

        if(deleteInvoiceData.affectedRows && deleteInvoiceDataItem.affectedRows){
            
            var getInvoiceBasicDetail = await Invoice.getInvoiceBasicDetail(req.body);
            console.log('getInvoiceBasicDetail',getInvoiceBasicDetail);

            var getInvoiceItemDetail =  await Invoice.getInvoiceItemDetail(req.body);
            console.log('getInvoiceItemDetail',getInvoiceItemDetail);
            
            var i = getInvoiceItemDetail.length - 1;

            if(getInvoiceItemDetail.length){
                getInvoiceItemDetail.forEach(async element => {
                    // element.po_id = addPurchaseOrderData.insertId;
                    
                    var stockData = {
                        product_id : element.product_id ? element.product_id : 0,
                        vendor_id : element.customer_id ? element.customer_id : 0,
                        stock : element.qty ? element.qty : 0,
                        invoice_no : getInvoiceBasicDetail.invoice_no ? getInvoiceBasicDetail.invoice_no : null,
                        invoice_id : element.invoice_id ? element.invoice_id : 0,
                        type : 'stockreverse'
                    };
                    
                    var stockLedgerMaintaindata = await Network.stockLedgerMaintaindata(stockData);                        
                    // console.log('stockLedgerMaintaindata',stockLedgerMaintaindata);
                    
                    if(i==0){
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Invoice Data Deleted",
                            status: "success",
                        });

                    }
                    i--; 
                });
            }else{
                await db.rollback();
                res.status(200).json({
                    message: "Invoice item not found",
                    status: "error"
                });
            }
        }else{
            await db.rollback();
            res.status(200).json({
                message : 'Invoice Data not Deleted',
                status: "error",
            });   
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};
