const Master = require("./../../models/web/masterModel");
const Network = require("./../../models/web/networkModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');
var base64ToFilePath = require('../../config/genericFn/base64ToFilePath');


exports.getBrandList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getBrandCount = await Master.getBrandCount(req.body)
        
        var getBrandListData = [];
        if (getBrandCount > 0) {
            getBrandListData = await Master.getBrandList(req.body);
            // console.log('getBrandListData',getBrandListData);
        }
        
        res.status(200).json({
            message: "Brand List",
            status: "success",
            result: {
                data: getBrandListData,
                totalCount: getBrandCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.addBrandData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addBrand = await Master.addBrandData(req.body);
        // console.log('addBrand',addBrand);
        
        if (addBrand.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Brand Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Brand not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateBrandData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateBrandData = await Master.updateBrandData(req.body);
        // console.log('updateBrandData',updateBrandData);
        
        if (updateBrandData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Brand Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Brand not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteBrandData = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var deleteBrandData = await Master.deleteBrandData(req.body);
        console.log('deleteBrandData', deleteBrandData);
        if (deleteBrandData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Brand Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Brand not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
}


exports.getBrandDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getBrandDetail = await Master.getBrandDetail(req.body);
        // console.log('getBrandDetail',getBrandDetail);
        
        if (getBrandDetail.id != undefined) {
            res.status(200).json({
                message: "Brand Detail",
                status: "success",
                data: {
                    detail: getBrandDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Brand Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// holiday code start

exports.getHolidayList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getHolidayCount = await Master.getHolidayCount(req.body)
        
        var getHolidayListData = [];
        if (getHolidayCount > 0) {
            getHolidayListData = await Master.getHolidayListData(req.body);
            // console.log('getHolidayListData',getHolidayListData);
        }
        
        res.status(200).json({
            message: "Holiday List",
            status: "success",
            result: {
                data: getHolidayListData,
                totalCount: getHolidayCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.addHolidayData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addHolidayData = await Master.addHolidayData(req.body);
        // console.log('addHolidayData',addHolidayData);
        
        if (addHolidayData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Holiday Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Holiday not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateHolidayData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateHolidayData = await Master.updateHolidayData(req.body);
        // console.log('updateHolidayData',updateHolidayData);
        
        if (updateHolidayData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Holiday Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Holiday not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getHolidayDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getHolidayDetail = await Master.getHolidayDetail(req.body);
        // console.log('getHolidayDetail',getHolidayDetail);
        
        if (getHolidayDetail.id != undefined) {
            res.status(200).json({
                message: "Holiday Detail",
                status: "success",
                data: {
                    detail: getHolidayDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Holiday Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getProductGroupList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductGroupCount = await Master.getProductGroupCount(req.body)
        
        var getProductGroupList = [];
        if (getProductGroupCount > 0) {
            getProductGroupList = await Master.getProductGroupList(req.body);
            // console.log('getProductGroupList',getProductGroupList);
        }
        
        res.status(200).json({
            message: "Product Group List",
            status: "success",
            result: {
                data: getProductGroupList,
                totalCount: getProductGroupCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addProductGroupData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductGroup = await Master.addProductGroupData(req.body);
        console.log('addProductGroup', addProductGroup);
        
        if (addProductGroup.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Group Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Group not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductGroupData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductGroup = await Master.updateProductGroupData(req.body);
        // console.log('updateProductGroup',updateProductGroup);
        
        if (updateProductGroup.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Group Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Group not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.deleteProductGroupData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductGroupData = await Master.deleteProductGroupData(req.body);
        console.log('deleteProductGroupData', deleteProductGroupData);
        
        if (deleteProductGroupData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Group Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Group not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductGroupDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductGroupDetail = await Master.getProductGroupDetail(req.body);
        // console.log('getProductGroupDetail',getProductGroupDetail);
        
        if (getProductGroupDetail.id != undefined) {
            res.status(200).json({
                message: "Product Group Detail",
                status: "success",
                data: {
                    detail: getProductGroupDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Group Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// product category start


exports.getProductCategoryList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var getProductCategoryCount = await Master.getProductCategoryCount(req.body)
        
        var getProductCategoryList = [];
        if (getProductCategoryCount > 0) {
            getProductCategoryList = await Master.getProductCategoryList(req.body);
            // console.log('getProductCategoryList',getProductCategoryList);
        }
        
        res.status(200).json({
            message: "Product Category List",
            status: "success",
            result: {
                data: getProductCategoryList,
                totalCount: getProductCategoryCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addProductCategoryData = async function (req, res) {
    console.log("data ==>", req.body)
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductCategoryData = await Master.addProductCategoryData(req.body);
        // console.log('addProductCategoryData',addProductCategoryData);
        
        if (addProductCategoryData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Category Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Group not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductCategoryData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductCategoryData = await Master.updateProductCategoryData(req.body);
        // console.log('updateProductCategoryData',updateProductCategoryData);
        
        if (updateProductCategoryData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Group Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Group not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteProductCategoryData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductCategoryData = await Master.deleteProductCategoryData(req.body);
        // console.log('updateProductCategoryData',deleteProductCategoryData);
        
        if (deleteProductCategoryData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Category Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Category not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductCategoryDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductCategoryDetail = await Master.getProductCategoryDetail(req.body);
        // console.log('getProductCategoryDetail',getProductCategoryDetail);
        
        if (getProductCategoryDetail.id != undefined) {
            res.status(200).json({
                message: "Product Category Detail",
                status: "success",
                data: {
                    detail: getProductCategoryDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Group Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// product category end

// product Type Start

exports.getProductTypeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductTypeCount = await Master.getProductTypeCount(req.body)
        
        var getProductTypeList = [];
        if (getProductTypeCount > 0) {
            getProductTypeList = await Master.getProductTypeList(req.body);
            // console.log('getProductCategoryList',getProductCategoryList);
        }
        
        res.status(200).json({
            message: "Product Type List",
            status: "success",
            result: {
                data: getProductTypeList,
                totalCount: getProductTypeCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.addProductTypeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductTypeData = await Master.addProductTypeData(req.body);
        console.log('addProductTypeData', addProductTypeData);
        
        if (addProductTypeData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Type Added",
                status: "success",
                data: addProductTypeData
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Type not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.updateProductTypeData = async function (req, res) {
    console.log(req.body);
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductTypeData = await Master.updateProductTypeData(req.body);
        console.log('updateProductTypeData', updateProductTypeData);
        
        if (updateProductTypeData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Type Updated",
                status: "success",
                detail: updateProductTypeData
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Type not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getProductTypeDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductTypeDetail = await Master.getProductTypeDetail(req.body);
        // console.log('getProductCategoryDetail',getProductCategoryDetail);
        
        if (getProductTypeDetail.id != undefined) {
            res.status(200).json({
                message: "Product Category Detail",
                status: "success",
                data: {
                    detail: getProductTypeDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Group Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteProductTypeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductTypeData = await Master.deleteProductTypeData(req.body);
        // console.log('updateProductCategoryData',deleteProductCategoryData);
        
        if (deleteProductTypeData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Type Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Type not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

// product Type End 

// Product Size Start

exports.getProductSizeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductSizeCount = await Master.getProductSizeCount(req.body)
        
        var getProductSizeList = [];
        if (getProductSizeCount > 0) {
            getProductSizeList = await Master.getProductSizeList(req.body);
            // console.log('getProductSizeList',getProductSizeList);
        }
        
        res.status(200).json({
            message: "Product Size List",
            status: "success",
            result: {
                data: getProductSizeList,
                totalCount: getProductSizeCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.addProductSizeData = async function (req, res) {
    console.log(req.body)
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductSizeData = await Master.addProductSizeData(req.body);
        console.log('addProductSizeData', addProductSizeData);
        
        if (addProductSizeData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Type Added",
                status: "success",
                data: addProductSizeData
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Type not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.updateProductSizeData = async function (req, res) {
    console.log(req.body);
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductSizeData = await Master.updateProductSizeData(req.body);
        console.log('updateProductSizeData', updateProductSizeData);
        
        if (updateProductSizeData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Type Updated",
                status: "success",
                detail: updateProductSizeData
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Type not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteProductSizeData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductSizeData = await Master.deleteProductSizeData(req.body);
        // console.log('deleteProductData',deleteProductData);
        
        if (deleteProductSizeData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Size Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Size not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getProductSizeDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductSizeDetail = await Master.getProductSizeDetail(req.body);
        // console.log('getProductSizeDetail',getProductSizeDetail);
        
        if (getProductSizeDetail.id != undefined) {
            res.status(200).json({
                message: "Product Size Detail",
                status: "success",
                data: {
                    detail: getProductSizeDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Size Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getProductColorList = async function (req, res) {
    console.log("data==>", req.body)
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductColorCount = await Master.getProductColorCount(req.body)
        
        var getProductColorList = [];
        if (getProductColorCount > 0) {
            getProductColorList = await Master.getProductColorList(req.body);
            // console.log('getProductUnitList',getProductUnitList);
        }
        
        res.status(200).json({
            message: "Product Color List",
            status: "success",
            result: {
                data: getProductColorList,
                totalCount: getProductColorCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.addProductColorData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductColor = await Master.addProductColorData(req.body);
        // console.log('addProductUnit',addProductUnit);
        
        if (addProductColor.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Color Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Color not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductColorData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductColor = await Master.updateProductColorData(req.body);
        // console.log('updateProductUnit',updateProductUnit);
        
        if (updateProductColor.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Color Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Color not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteProductColorData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductColorData = await Master.deleteProductColorData(req.body);
        // console.log('deleteProductUnitData',deleteProductUnitData);
        
        if (deleteProductColorData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Color Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Color not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductColorDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductColorDetail = await Master.getProductColorDetail(req.body);
        // console.log('getProductUnitDetail',getProductUnitDetail);
        
        if (getProductColorDetail.id != undefined) {
            res.status(200).json({
                message: "Product Color Detail",
                status: "success",
                data: {
                    detail: getProductColorDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Color Detail",
                status: "notfound",
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductUnitList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductUnitCount = await Master.getProductUnitCount(req.body)
        
        var getProductUnitList = [];
        if (getProductUnitCount > 0) {
            getProductUnitList = await Master.getProductUnitList(req.body);
            
        }
        
        res.status(200).json({
            message: "Product Unit List",
            status: "success",
            result: {
                data: getProductUnitList,
                totalCount: getProductUnitCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getBeatList = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getBeatListCount = await Master.getBeatListCount(req.body)
        
        var getBeatList = [];
        if (getBeatListCount > 0) {
            getBeatList = await Master.getBeatList(req.body);
            // console.log('getBeatList',getBeatList);
        }
        
        res.status(200).json({
            message: "Beat Plan List",
            status: "success",
            result: {
                data: getBeatList,
                totalCount: getBeatListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getBeatPlanData = async function (req, res) {
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var startBeatDateMonth = '1';
        var getBeatPlanData = [];
        var beatYearMonthArray = []
        var beatYearMonthDisplayArray = []
        for (let index = startBeatDateMonth; index <= moment(new Date()).format('M'); index++) {
            index = (index < 10) ? '0' + index.toString() : index.toString();
            // console.log('startBeatDateMonth', index);  
            beatYearMonthArray.push(moment(new Date()).format('YYYY') + '-' + index);
        }
        
        req.body.beat_month = req.body.beat_month ? req.body.beat_month : moment(new Date()).format('YYYY') + '-' + moment(new Date()).format('MM')
        var getBeatPlanData = [];
        if (req.body.beat_plan_id) {
            req.body.beat_plan_id = decrypt(req.body.beat_plan_id)
        }
        getBeatPlanData = await Master.getBeatPlanData(req.body);
        
        if (beatYearMonthArray.length) {
            beatYearMonthArray.forEach(element => {
                beatYearMonthDisplayArray.push(moment(element).format('MMM-Y'));
            });
        }
        
        res.status(200).json({
            message: "Beat Plan Detail",
            status: "success",
            result: {
                data: getBeatPlanData,
                beatYearMonthArray: beatYearMonthArray,
                beatYearMonthDisplayArray: beatYearMonthDisplayArray
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addBeatData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var checkSalesUserAlreadyAssign = await Master.checkSalesUserAlreadyAssign(req.body);
        console.log('checkSalesUserAlreadyAssign==>', checkSalesUserAlreadyAssign);
        
        if (checkSalesUserAlreadyAssign.id) {
            req.body.beat_plan_id = checkSalesUserAlreadyAssign.id
            var updateBeatData = await Master.updateBeatData(req.body);
            // console.log('updateBeatData',updateBeatData);
            if (updateBeatData.affectedRows) {
                var addBeatDetailData = await Master.addBeatDetailData(req.body);
            }
            
        } else {
            var addBeatData = await Master.addBeatData(req.body);
            // console.log('addBeatData',addBeatData);
            if (addBeatData.insertId) {
                req.body.beat_plan_id = addBeatData.insertId;
                var addBeatDetailData = await Master.addBeatDetailData(req.body);
            }
        }
        
        if (addBeatDetailData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Beat data added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Beat data not Added",
                status: "failed",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteBeatData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteBeatData = await Master.deleteBeatData(req.body);
        // console.log('deleteBeatData',deleteBeatData);
        
        if (deleteBeatData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Beat Plan Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Beat Plan not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};
exports.deleteBeatPlanData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteBeatPlanData = await Master.deleteBeatPlanData(req.body);
        // console.log('deleteBeatPlanData',deleteBeatPlanData);
        
        var getBeatPlanData = await Master.getBeatPlanData(req.body);
        // console.log('getBeatPlanData',getBeatPlanData);
        if (!getBeatPlanData.length) {
            console.log('getBeatPlanData', getBeatPlanData);
            var deleteBeatData = await Master.deleteBeatData(req.body);
            console.log('deleteBeatData', deleteBeatData);
        }
        
        if (deleteBeatPlanData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Beat Plan Detail Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Beat Plan Detail not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addProductUnitData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductUnit = await Master.addProductUnitData(req.body);
        // console.log('addProductUnit',addProductUnit);
        
        if (addProductUnit.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product Unit Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Unit not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductUnitData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductUnit = await Master.updateProductUnitData(req.body);
        // console.log('updateProductUnit',updateProductUnit);
        
        if (updateProductUnit.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Unit Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Unit not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.deleteProductUnitData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductUnitData = await Master.deleteProductUnitData(req.body);
        // console.log('deleteProductUnitData',deleteProductUnitData);
        
        if (deleteProductUnitData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Unit Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product Unit not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductUnitDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductUnitDetail = await Master.getProductUnitDetail(req.body);
        // console.log('getProductUnitDetail',getProductUnitDetail);
        
        if (getProductUnitDetail.id != undefined) {
            res.status(200).json({
                message: "Product Unit Detail",
                status: "success",
                data: {
                    detail: getProductUnitDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product Unit Detail",
                status: "notfound",
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};




exports.getProductHsnList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductHsnCount = await Master.getProductHsnCount(req.body)
        
        var getProductHsnList = [];
        if (getProductHsnCount > 0) {
            getProductHsnList = await Master.getProductHsnList(req.body);
            // console.log('getProductHsnList',getProductHsnList);
        }
        
        res.status(200).json({
            message: "Product HSN List",
            status: "success",
            result: {
                data: getProductHsnList,
                totalCount: getProductHsnCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addProductHsnData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addProductHsn = await Master.addProductHsnData(req.body);
        // console.log('addProductHsn',addProductHsn);
        
        if (addProductHsn.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Product HSN Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product HSN not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductHsnData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductHsn = await Master.updateProductHsnData(req.body);
        // console.log('updateProductHsn',updateProductHsn);
        
        if (updateProductHsn.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product HSN Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product HSN not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getHsnDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getHsnDetail = await Master.getHsnDetail(req.body);
        // console.log('getHsnDetail',getHsnDetail);
        
        if (getHsnDetail.id != undefined) {
            res.status(200).json({
                message: "Product HSN detail",
                status: "success",
                data: {
                    detail: getHsnDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Product HSN detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductThicknessList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getProductThicknessCount = await Master.getProductThicknessCount(req.body)
        
        var getProductThicknessList = [];
        if (getProductThicknessCount > 0) {
            getProductThicknessList = await Master.getProductThicknessList(req.body);
            // console.log('getProductThicknessList',getProductThicknessList);
        }
        
        res.status(200).json({
            message: "Product Thickness List",
            status: "success",
            result: {
                data: getProductThicknessList,
                totalCount: getProductThicknessCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getRoleList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getRoleCount = await Master.getRoleCount(req.body)
        
        var getRoleListData = [];
        if (getRoleCount > 0) {
            getRoleListData = await Master.getRoleListData(req.body);
            // console.log('getRoleListData',getRoleListData);
        }
        
        res.status(200).json({
            message: "Role List",
            status: "success",
            result: {
                data: getRoleListData,
                totalCount: getRoleCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addRoleData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addRoleData = await Master.addRoleData(req.body);
        // console.log('addRoleData',addRoleData);
        
        if (addRoleData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Role data Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Role data not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateRoleData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateRoleData = await Master.updateRoleData(req.body);
        // console.log('updateRoleData',updateRoleData);
        
        if (updateRoleData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Role Data Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Role Data not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getRoleDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getRoleDetail = await Master.getRoleDetail(req.body);
        // console.log('getRoleDetail',getRoleDetail);
        
        if (getRoleDetail.id != undefined) {
            res.status(200).json({
                message: "Role Data Detail",
                status: "success",
                data: {
                    detail: getRoleDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Role Data Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalMasterList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getPostalMasterCount = await Master.getPostalMasterCount(req.body)
        
        var getPostalMasterList = [];
        if (getPostalMasterCount > 0) {
            getPostalMasterList = await Master.getPostalMasterList(req.body);
            // console.log('getPostalMasterList',getProductHsnList);
        }
        
        res.status(200).json({
            message: "Postal Master List",
            status: "success",
            result: {
                data: getPostalMasterList,
                totalCount: getPostalMasterCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalStateList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalStateList = [];
        getPostalStateList = await Master.getPostalStateList(req.body);
        // console.log('getPostalStateList',getPostalStateList);
        
        res.status(200).json({
            message: "Postal Master state list",
            status: "success",
            result: {
                data: getPostalStateList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalDistrictList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalDistrictList = [];
        getPostalDistrictList = await Master.getPostalDistrictList(req.body);
        // console.log('getPostalDistrictList',getPostalDistrictList);
        
        res.status(200).json({
            message: "Postal Master district list",
            status: "success",
            result: {
                data: getPostalDistrictList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalCityList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalCityList = [];
        getPostalCityList = await Master.getPostalCityList(req.body);
        // console.log('getPostalCityList',getPostalCityList);
        
        res.status(200).json({
            message: "Postal Master city list",
            status: "success",
            result: {
                data: getPostalCityList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalAreaList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalAreaList = [];
        getPostalAreaList = await Master.getPostalAreaList(req.body);
        // console.log('getPostalAreaList',getPostalAreaList);
        
        res.status(200).json({
            message: "Postal Master Area list",
            status: "success",
            result: {
                data: getPostalAreaList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getPostalZoneList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalZoneList = [];
        getPostalZoneList = await Master.getPostalZoneList(req.body);
        // console.log('getPostalZoneList',getPostalZoneList);
        
        res.status(200).json({
            message: "Postal Master zone list",
            status: "success",
            result: {
                data: getPostalZoneList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getPostalBeatList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalBeatList = [];
        getPostalBeatList = await Master.getPostalBeatList(req.body);
        // console.log('getPostalZoneList',getPostalZoneList);
        
        res.status(200).json({
            message: "Postal Master beat list",
            status: "success",
            result: {
                data: getPostalBeatList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

exports.getPostalPincodeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalPincodeList = [];
        getPostalPincodeList = await Master.getPostalPincodeList(req.body);
        // console.log('getPostalPincodeList',getPostalPincodeList);
        
        res.status(200).json({
            message: "Postal Master pincode list",
            status: "success",
            result: {
                data: getPostalPincodeList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getDetailByPincode = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getDetailByPincode = [];
        getDetailByPincode = await Master.getDetailByPincode(req.body);
        // console.log('getDetailByPincode',getDetailByPincode);
        
        res.status(200).json({
            message: "Postal Master pincode data",
            status: "success",
            result: {
                data: getDetailByPincode
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addPostalMasterData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addPostalMasterData = await Master.addPostalMasterData(req.body);
        // console.log('addPostalMasterData',addPostalMasterData);
        
        if (addPostalMasterData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Postal Master Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Postal Master not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updatePostalMasterData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updatePostalMasterData = await Master.updatePostalMasterData(req.body);
        // console.log('updatePostalMasterData',updatePostalMasterData);
        
        if (updatePostalMasterData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Postal Master Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Postal Master not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getPostalMasterDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getPostalMasterDetail = [];
        getPostalMasterDetail = await Master.getPostalMasterDetail(req.body);
        // console.log('getPostalMasterDetail',getPostalMasterDetail);
        
        res.status(200).json({
            message: "Postal Master detail",
            status: "success",
            result: {
                data: getPostalMasterDetail
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// Product controller code start

exports.getProductCodeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getProductCodeList = await Master.getProductCodeList(req.body);
        // console.log('getProductCodeList',getProductCodeList);
        
        
        res.status(200).json({
            message: "Product Code List",
            status: "success",
            result: {
                data: getProductCodeList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var getProductCount = await Master.getProductCount(req.body)
        
        var getProductList = [];
        if (getProductCount > 0) {
            getProductList = await Master.getProductList(req.body);
            // console.log('getProductList',getProductList);
        }
        
        res.status(200).json({
            message: "Product List",
            status: "success",
            result: {
                data: getProductList,
                totalCount: getProductCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getProductListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        req.body.limit = 1000000;
        req.body.start = 0;
        
        var getProductList = await Master.getProductList(req.body);
        // console.log('getProductList',getProductList);
        
        
        var getProductListExcel = [];
        if (getProductList.length) {
            var i = getProductList.length - 1;
            
            console.log('filter', req.body.filter.cust_type_id);
            
            var file_name = 'Product-Master-Data';
            
            getProductList.forEach(async (element, index) => {
                
                var data = {
                    'S.No.': index + 1,
                    'Date': element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00',
                    // 'Brand': element.brand ? element.brand : '',
                    'Category' : element.product_category ? element.product_category : '',
                    'Sub-Category': element.product_group ? element.product_group : '',
                    'Product Name': element.product_name ? element.product_name : '',
                    'Product Code': element.product_code ? element.product_code : '',
                    'Batch Code': element.batch_code ? element.batch_code : '',
                    'Description' : element.description ? element.description : '',
                    'Product Size': element.product_size ? element.product_size : '',
                    // 'Unit': element.unit ? element.unit : '',
                    'Retaler Point': element.retailer_point ? element.retailer_point : '0',
                    'Distributor Point' : element.distributor_point ? element.distributor_point :'0',
                    'Sales User Point' : element.sales_user_point ? element.sales_user_point : '0',
                    // 'Sale Price': element.sale_price ? element.sale_price : '',
                    'status': element.status ? element.status : '0',
                    'Updated On': element.updated_on ? element.updated_on : '',
                };
                
                
                getProductListExcel.push(data);
                
                if (i == 0) {
                    res.status(200).json({
                        message: "Product List Excel",
                        status: "success",
                        result: {
                            data: getProductListExcel,
                            file_name: file_name
                        }
                    });
                }
                
                i--;
            });
        } else {
            res.status(200).json({
                message: "Product List Excel",
                status: "success",
                result: {
                    data: getProductListExcel,
                }
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addProductData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var addProductData = await Master.addProductData(req.body);
        req.body.product_id = addProductData.insertId
        
        if (addProductData.insertId) {
            if (req.body.product_color?.length) {
                var i = req.body.product_color.length - 1;
                req.body.product_color.forEach(async x => {
                    var assigncolor = await Master.assigncolor({ 'color': x, 'product_id': addProductData.insertId ? addProductData.insertId : 0 });
                    console.log('assigncolor', assigncolor);
                    i--;
                })
            }
            if(req.body.image_path?.length){

                var i = req.body.image_path.length - 1;
                req.body.image_path.forEach(async x => {
                    console.log("x------->",x) 
                    var product_image = await Master.addproductimage({ 'filename': x, 'product_id': addProductData.insertId ? addProductData.insertId : 0 });
                    console.log('product_image', product_image);
                    i--;
                })
       
            }
            await db.commit();
               return res.status(200).json({
                     message: "Product Added",
                     status: "success",
            });
            
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.importProductData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        
        if (req.body.importData.length) {
            
            var i = req.body.importData.length - 1;
            req.body.importData.forEach(async element => {
                
                element.sfa_web_user_id = req.body.sfa_web_user_id;
                element.sfa_web_user_name = req.body.sfa_web_user_name;
                element.sfa_web_user_type = req.body.sfa_web_user_type;
                
                var addProductData = await Master.addProductData(element);
                // console.log('addProductData',addProductData);
                
                if (i == 0) {
                    res.status(200).json({
                        message: "Data added",
                        status: "success",
                    });
                }
                i--;
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Data not added",
                status: "error"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateProductData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateProductData = await Master.updateProductData(req.body);
        // console.log('updateProductData',updateProductData);
        if (updateProductData.affectedRows) {
            
    
            
            if (req.body.product_color?.length) {
                
                var deleteAssignedColorData = await Master.deleteAssignedColorData(req.body);
                console.log('deleteAssignedColorData',deleteAssignedColorData);

                
                var i = req.body.product_color.length - 1;
                req.body.product_color.forEach(async x => {
                    var assigncolor = await Master.assigncolor({ 'color': x, 'product_id': req.body.product_id ? req.body.product_id : 0 });
                    console.log('assigncolor', assigncolor);
                    i--;
                })

            }
            if(req.body.image_path?.length){

                var i = req.body.image_path.length - 1;
                req.body.image_path.forEach(async x => {
                    console.log("x------->",x) 
                    var deleteProductimage = await Master.deleteProductimage(req.body);
                    console.log('deleteProductimage',deleteProductimage);

                    var product_image = await Master.addproductimage({ 'filename': x, 'product_id': req.body.product_id ? req.body.product_id : 0 });
                    console.log('product_image', product_image);
                    i--;
                })
       
            }
           
          
                await db.commit();
                res.status(200).json({
                    message: "Product Updated",
                    status: "success",
                });
            
           



        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.deleteProductData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteProductData = await Master.deleteProductData(req.body);
        // console.log('deleteProductData',deleteProductData);
        
        var deleteAssignedColorData = await Master.deleteAssignedColorData(req.body);
        console.log('deleteAssignedColorData',deleteAssignedColorData);
        
        if (deleteProductData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Product Deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Product not Deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.getProductDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        req.body.product_id = decrypt(req.body.product_id);
        req.body.image_app_path = res.locals.image_app_path;
        
        var getProductDetail = await Master.getProductDetail(req.body);
        console.log('getProductDetail',getProductDetail);
        
        // var getProductColorDetail = await Master.getProductColorDetail(req.body);
        // console.log('getProductColorDetail',getProductColorDetail);
        
        if (getProductDetail.id != undefined) {
            res.status(200).json({
                message: "Product detail",
                status: "success",
                data: {
                    detail: getProductDetail,
                }
            });
        } else {
            res.status(200).json({
                message: "Product detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

// Product controller code end



// feedback controller code start

exports.getFeedbackCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var statusArray = [
            { 'type': 'unread', 'name': 'Un Read', 'count': 0 },
            { 'type': 'read', 'name': 'Read', 'count': 0 }
        ]
        
        if (statusArray.length) {
            var i = statusArray.length - 1;
            statusArray.forEach(async element => {
                element.count = await Master.getFeedbackCount(element);
                
                if (i == 0) {
                    res.status(200).json({
                        message: "Feedback Count",
                        status: "success",
                        result: {
                            data: statusArray,
                        }
                    });
                }
                i--;
            });
        } else {
            res.status(200).json({
                message: "Feedback Count",
                status: "success",
                result: {
                    data: statusArray,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getFeedbackList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getFeedbackListCount = await Master.getFeedbackListCount(req.body)
        
        var getFeedbackListData = [];
        if (getFeedbackListCount > 0) {
            getFeedbackListData = await Master.getFeedbackList(req.body);
            console.log('getFeedbackListData------>',getFeedbackListData);
        }
        
        res.status(200).json({
            message: "Feedback List",
            status: "success",
            result: {
                data: getFeedbackListData,
                totalCount: getFeedbackListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getFeedbackDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var getFeedbackDetailData = await Master.getFeedbackDetailData(req.body);
        // console.log('getFeedbackDetailData',getFeedbackDetailData);
        
        
        var markReadMessage = await Master.markReadMessage(req.body);
        console.log('markReadMessage', markReadMessage);
        
        res.status(200).json({
            message: "Feedback Detail",
            status: "success",
            result: {
                data: getFeedbackDetailData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.sendFeedbackMessage = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        console.log('body ==>', req.body);
        
        var sendFeedbackMessage = await Master.sendFeedbackMessage(req.body);
        console.log('sendFeedbackMessage', sendFeedbackMessage);
        
        var markReadMessage = await Master.markReadMessage(req.body);
        console.log('markReadMessage', markReadMessage);
        
        if (sendFeedbackMessage.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Feedback message sent",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Chat message sent",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// feedback controller code end


// site controller code start


exports.getSiteList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var getSiteListCount = await Master.getSiteListCount(req.body)
        
        var getSiteListData = [];
        if (getSiteListCount > 0) {
            getSiteListData = await Master.getSiteListData(req.body);
            // console.log('getSiteListData',getSiteListData);
        }
        
        res.status(200).json({
            message: "Site List",
            status: "success",
            result: {
                data: getSiteListData,
                totalCount: getSiteListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getSiteDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        req.body.site_id = decrypt(req.body.site_id);
        
        var getSiteDetail = await Master.getSiteDetail(req.body);
        console.log('getSiteDetail', getSiteDetail);
        
        res.status(200).json({
            message: "Site Detail",
            status: "success",
            result: {
                data: getSiteDetail,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


// site controller code end




// banner code start

exports.getBannerList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        var getBannerCount = await Master.getBannerCount(req.body)
        
        var getBannerListData = [];
        if (getBannerCount > 0) {
            getBannerListData = await Master.getBannerListData(req.body);
            // console.log('getBannerListData',getBannerListData);
        }
        
        res.status(200).json({
            message: "Banner List",
            status: "success",
            result: {
                data: getBannerListData,
                totalCount: getBannerCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addBannerData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var addBannerData = await Master.addBannerData(req.body);
        // console.log('addBannerData',addBannerData);
        
        if (addBannerData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Banner data Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Banner data not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateBannerData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var updateBannerData = await Master.updateBannerData(req.body);
        // console.log('updateBannerData',updateBannerData);
        
        if (updateBannerData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Banner Data Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Banner Data not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



exports.deleteBannerData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var deleteBannerData = await Master.deleteBannerData(req.body);
        // console.log('deleteBannerData',deleteBannerData);
        
        if (deleteBannerData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Banner Data deleted",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Banner Data not deleted",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getBannerDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getBannerDetail = await Master.getBannerDetail(req.body);
        // console.log('getBannerDetail',getBannerDetail);
        
        if (getBannerDetail.id != undefined) {
            res.status(200).json({
                message: "Banner Data Detail",
                status: "success",
                data: {
                    detail: getBannerDetail
                }
            });
        } else {
            res.status(200).json({
                message: "Banner Data Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

// banner code end



exports.base64ToFile = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    console.log('herererer');
    // var base64ToFile = await Master.base64ToFile(req.body);
    // console.log('base64ToFile', base64ToFile);
    var base64ToImage = await base64ToFilePath.base64ToImage(req.body);
    console.log('base64ToImage', base64ToImage);
    
    res.status(200).json({
        message: 'Uploaded',
        status: "success",
        response: base64ToImage
    });
    
};

exports.unlinkFile = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    var unlinkFile = await Master.unlinkFile(req.body);
    console.log('unlinkFile', unlinkFile);
    
    res.status(200).json({
        message: 'Unlinked',
        status: "success",
        response: unlinkFile
    });
};


// referral point master


exports.getReferralPointMaster = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        
        var getReferralPointMaster = await Master.getReferralPointMaster(req.body);
        // console.log('getReferralPointMaster',getReferralPointMaster);
        
        await db.commit();
        res.status(200).json({
            message: "Point Master data",
            status: "success",
            result: {
                data: getReferralPointMaster
            }
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateReferralPointMaster = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        // console.log('body ==>', req.body);
        
        var updateReferralPointMaster = await Master.updateReferralPointMaster(req.body);
        console.log('updateReferralPointMaster', updateReferralPointMaster);
        
        if (updateReferralPointMaster.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Point Master Updated",
                status: "success"
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Point Master Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};



// Content code start


exports.getContentTypeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getContentTypeList = [];
        getContentTypeList = await Master.getContentTypeList(req.body);
        // console.log('getContentTypeList',getContentTypeList);
        
        res.status(200).json({
            message: "Content Type List",
            status: "success",
            result: {
                data: getContentTypeList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getContentDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var getContentDataCount = await Master.getContentDataCount(req.body)
        
        var getContentListData = [];
        if (getContentDataCount > 0) {
            getContentListData = await Master.getContentListData(req.body);
            // console.log('getContentListData',getContentListData);
        }
        
        res.status(200).json({
            message: "Content List Data",
            status: "success",
            result: {
                data: getContentListData,
                totalCount: getContentDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.addContentData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        var addContentData = await Master.addContentData(req.body);
        // console.log('addContentData',addContentData);
        
        if (addContentData.insertId) {
            await db.commit();
            res.status(200).json({
                message: "Content data Added",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Content data not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.updateContentData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;
        
        req.body.content_id = decrypt(req.body.content_id);
        
        var updateContentData = await Master.updateContentData(req.body);
        // console.log('updateContentData',updateContentData);
        
        if (updateContentData.affectedRows) {
            await db.commit();
            res.status(200).json({
                message: "Content Data Updated",
                status: "success",
            });
        } else {
            await db.rollback();
            res.status(200).json({
                message: "Content Data not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};


exports.getContentDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        req.body.content_id = decrypt(req.body.content_id);
        
        var getContentDetail = await Master.getContentDetail(req.body);
        // console.log('getContentDetail',getContentDetail);
        
        if (getContentDetail.id != undefined) {
            res.status(200).json({
                message: "Contact Data Detail",
                status: "success",
                data: {
                    detail: getContentDetail
                }
            });
        } else {
            res.status(200).json({
                message: "FAQs Data Detail",
                status: "notfound",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};

// Content code end

exports.getBankList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        
        var getBankList = [];
        getBankList = await Master.getBankList(req.body);
        console.log('getBankList', getBankList);
        
        res.status(200).json({
            message: "Bank List",
            status: "success",
            result: {
                data: getBankList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err: err
        });
        return false;
    }
};
