const Network = require("../../models/web/networkModel");
const Invoice = require("../../models/web/invoiceModel");
const Notification = require("../../models/web/notificationModel");
const SFA_masterModel = require("../../models/web/SFA_masterModel");
const Order = require("../../models/web/orderModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');


exports.getCustomerNetworkType = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        var getCustomerNetworkTypeListData = [];
        getCustomerNetworkTypeListData =  await Network.getCustomerNetworkTypeList(req.body);
        // console.log('getCustomerNetworkTypeListData',getCustomerNetworkTypeListData);
        
        res.status(200).json({
            message: "Customer network type list",
            status: "success",
            result : {
                data: getCustomerNetworkTypeListData
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};




exports.getLeadSourceListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        // console.log('body ==>', req.body);
        
        var  getLeadSourceListData =  await Network.getLeadSourceListData(req.body);
        // console.log('getLeadSourceListData',getLeadSourceListData);
        
        res.status(200).json({
            message: "Lead source data",
            status: "success",
            result : {
                data: getLeadSourceListData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getLeadStatusListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        // console.log('body ==>', req.body);
        
        var  getLeadStatusListData =  await Network.getLeadStatusListData(req.body);
        // console.log('getLeadStatusListData',getLeadStatusListData);
        
        res.status(200).json({
            message: "Lead Status List",
            status: "success",
            result : {
                data: getLeadStatusListData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getSystemUserList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getSystemUserList =  await Network.getSystemUserList(req.body);
        // console.log('getSystemUserList',getSystemUserList);
        
        res.status(200).json({
            message: "System User List",
            status: "success",
            result : {
                data: getSystemUserList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getSalesUserList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getSalesUserList =  await Network.getSalesUserList(req.body);
        // console.log('getSalesUserList',getSalesUserList);
        
        res.status(200).json({
            message: "Sales User List",
            status: "success",
            result : {
                data: getSalesUserList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getNetworkListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        
        var statusArrayData = [
            {'status' : 'All'},
            // {'status' : 'Verified'},
            {'status' : 'Pending'},
            {'status' : 'Approved'},
            // {'status' : 'Disqualified'},
            // {'status' : 'Win'},
            {'status' : 'Lost'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.cust_type_id = req.body.cust_type_id;
                element.count = await Network.getNetworkCountByStatus(element);
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Network List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Network List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getNetworkList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        
        // user_assignedDrIds
        if(req.body.filter.encrypt_user_id){
            req.body.filter.user_id = decrypt(req.body.filter.encrypt_user_id);
            req.body.user_id = req.body.filter.user_id;
            var getAssignedDrIds = await Network.getDistributionNetworkUserAssigned({user_id : req.body.user_id})
            console.log('getAssignedDrIds',getAssignedDrIds);
            req.body.user_assignedDrIds = getAssignedDrIds ? getAssignedDrIds : '';
        }
        
        
        var getNetworkCount = await Network.getNetworkCount(req.body)
        
        var getNetworkList = [];
        if(getNetworkCount > 0){
            getNetworkList =  await Network.getNetworkList(req.body);
            // console.log('getNetworkList',getNetworkList);
        }
        
        res.status(200).json({
            message: "Network List",
            status: "success",
            result : {
                data: getNetworkList,
                totalCount : getNetworkCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getNetworkListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.limit = 1000000;
        req.body.start = 0;
        var getNetworkList =  await Network.getNetworkList(req.body);
        
        var getNetworkListExcel = [];
        if(getNetworkList.length){
            var i = getNetworkList.length - 1;
            
            console.log('filter', req.body.filter.cust_type_id);
            
            var file_name = req.body.filter.cust_type_id == 1 ? 'Distributor-Network-Data' : 'Retailer-Network-Data';
            
            getNetworkList.forEach(async (element,index) => {
                
                element.customer_id = element.id;
                // console.log('element', element.id);
                // var getDistributionNetworkDealerDetail = await Network.getDistributionNetworkDealerDetail(element);
                // console.log('getDistributionNetworkDealerDetail',getDistributionNetworkDealerDetail);
                // if(getDistributionNetworkDealerDetail.length){
                //     element.dealer_counter_name = getDistributionNetworkDealerDetail[0].name ? getDistributionNetworkDealerDetail[0].name : '';
                //     element.dealer_counter_cp_name = getDistributionNetworkDealerDetail[0].cp_name ? getDistributionNetworkDealerDetail[0].cp_name : '';
                //     element.dealer_counter_mobile = getDistributionNetworkDealerDetail[0].mobile ? getDistributionNetworkDealerDetail[0].mobile : '';
                //     element.dealer_counter_status = 'Active';
                //     element.dealer_counter_address = getDistributionNetworkDealerDetail[0].address ? getDistributionNetworkDealerDetail[0].address : '';
                // }
                
                var getDistributionNetworkDocumentDetail = await Network.getDistributionNetworkDocumentDetail(element);
                // console.log('getDistributionNetworkDocumentDetail',getDistributionNetworkDocumentDetail);
                if(getDistributionNetworkDocumentDetail.length){
                    element.doc_type = getDistributionNetworkDocumentDetail[0].doc_type ? getDistributionNetworkDocumentDetail[0].doc_type : '';
                    element.doc_type_value = getDistributionNetworkDocumentDetail[0].doc_type_value ? getDistributionNetworkDocumentDetail[0].doc_type_value : '';
                }
                
                
                var getDistributionNetworkAssignedUserDetail = await Network.getDistributionNetworkAssignedUserDetail(element);
                console.log('getDistributionNetworkAssignedUserDetail',getDistributionNetworkAssignedUserDetail);
                if(getDistributionNetworkAssignedUserDetail.length){
                    element.se_name = getDistributionNetworkAssignedUserDetail[0].user_name ? getDistributionNetworkAssignedUserDetail[0].user_name : '';
                    element.se_code = getDistributionNetworkAssignedUserDetail[0].user_code ? getDistributionNetworkAssignedUserDetail[0].user_code : '';
                }
                
                var data = {
                    'S.No.' : index + 1, 
                    'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                    'Created By' : element.created_by_name ?  element.created_by_name :'',
                    // 'Company Name' : element.first_name ? element.first_name +' '+ element.last_name : '', 
                    'Company Name': element.company_name ? element.company_name :'',
                    'Email' : element.email ? element.email : '',
                    'Mobile': element.mobile ? element.mobile : '',
                    'Address': element.street ? element.street : '',
                    'State': element.state_name ? element.state_name : '', 
                    'District': element.district_name ? element.district_name : '', 
                    'City': element.city ? element.city : '', 
                    'Pincode': element.pincode ? element.pincode : 0, 
                    'Document Type': element.doc_type ? element.doc_type : '', 
                    'Document Number': element.doc_type_value ? element.doc_type_value : '', 
                    // 'Dealer Counter Name': element.dealer_counter_name ? element.dealer_counter_name : '', 
                    // 'Dealer Contact Person':  element.dealer_counter_cp_name ? element.dealer_counter_cp_name : '',
                    // 'Dealer Mobile No.': element.dealer_counter_mobile ? element.dealer_counter_mobile : '', 
                    // 'Dealer Status': element.dealer_counter_status ? element.dealer_counter_status : '', 
                    // 'Dealer Address': element.dealer_counter_address ? element.dealer_counter_address : '', 
                    // 'Sales Executive': element.se_name ? element.se_name + ' - ' + element.se_code : '', 
                    'Wallet Points': element.wallet_points ? element.wallet_points : 0, 
                    'Assigned Sale User': element.se_name ? element.se_name :'',
                    // 'Assigned Distributor': element.se_name ? element.se_name :'',
                    
                    // 'Gifts Request': element.redeem_request ? element.redeem_request : 0,
                    'Status': element.status ? element.status : '', 
                    'Action Date':  element.updated_on ? moment(element.updated_on).format('YYYY-MM-DD') : '0000-00-00', 
                    // 'Days to Verify': '', 
                    // 'Scanned Points': '', 
                    // 'Referral Points': '', 
                };
                
                
                getNetworkListExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Network List Excel",
                        status: "success",
                        result : {
                            data: getNetworkListExcel,
                            file_name : file_name
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Network List Excel",
                status: "success",
                result : {
                    data: getNetworkListExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.importDistributionNetwork = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        if(req.body.importData.length){
            
            var i = req.body.importData.length - 1;
            req.body.importData.forEach(async element => {
                
                element.sfa_web_user_id = req.body.sfa_web_user_id;
                element.sfa_web_user_name = req.body.sfa_web_user_name;
                element.sfa_web_user_type = req.body.sfa_web_user_type;
                
                var addDistributionNetworkData =  await Network.addDistributionNetworkData(element);
                console.log('addDistributionNetworkData',addDistributionNetworkData);
                
                if(i == 0){
                    res.status(200).json({
                        message: "Data added",
                        status: "success",
                    });
                }
                i--;
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if(e.code == 'ER_DUP_ENTRY'){
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>',myArray[1])
            var key = myArray[1];
            console.log('key',key);
            key = key.replace(/'/g, "");
            console.log('key',key);
            if(key == 'tg_customer.tg_customer_mobile_UN'){
                errMessage = 'Duplicate entry for mobile number - '+req.body.basicDetail.mobile;
            }else if(key == 'tg_customer.tg_customer_UN'){
                errMessage = 'Duplicate entry for email - '+req.body.basicDetail.email;
            }else{
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage',errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err : err
        });
        return false;
    }
};


exports.addDistributionNetwork = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        req.body.basicDetail.sfa_web_user_id = req.body.sfa_web_user_id;
        req.body.basicDetail.sfa_web_user_name = req.body.sfa_web_user_name;
        req.body.basicDetail.sfa_web_user_type = req.body.sfa_web_user_type;
        
        var addDistributionNetworkData =  await Network.addDistributionNetworkData(req.body.basicDetail);
        console.log('addDistributionNetworkData',addDistributionNetworkData);
        
        if(addDistributionNetworkData.insertId){
            
            if(req.body.basicDetail?.contactDetail?.length){
                req.body.customer_id = addDistributionNetworkData.insertId;
                var addDistributionNetworkContactData =  await Network.addDistributionNetworkContactData(req.body);
                console.log('addDistributionNetworkContactData',addDistributionNetworkContactData);    
            }
            
            if(req.body.brandDetail?.length){
                req.body.customer_id = addDistributionNetworkData.insertId;
                var addDistributionNetworkBrandData =  await Network.assignDistributionNetworkBrandData(req.body);
                console.log('addDistributionNetworkBrandData',addDistributionNetworkBrandData);    
            }
            
            
            if(req.body.basicDetail?.assignedUser?.length){
                req.body.customer_id = addDistributionNetworkData.insertId;
                var assignDistributionNetworkUserData =  await Network.assignDistributionNetworkUserData(req.body);
                console.log('assignDistributionNetworkUserData',assignDistributionNetworkUserData);    
            }
            
            
            var logData = {
                customer_id : addDistributionNetworkData.insertId,
                type : req.body.basicDetail.cust_type+ ' Details Added',
                description : 'Details added',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            req.body.customer_id = addDistributionNetworkData.insertId ? addDistributionNetworkData.insertId : 0;
            
            var documentData = {
                customer_id : req.body.customer_id ? req.body.customer_id : 0,
                doc_type : req.body.basicDetail.doc_type ? req.body.basicDetail.doc_type : '',
                doc_type_value : req.body.doc_no ? req.body.basicDetail.doc_no : '',
                doc_filename : req.body.basicDetail.doc_image ? req.body.basicDetail.doc_image : '',
                doc_filename_1 : req.body.basicDetail.doc_image_1 ? req.body.basicDetail.doc_image_1 : '',
            };
            
            var addCustomerDocument = await Network.addCustomerDocument(documentData);
            console.log('addCustomerDocument',addCustomerDocument);
            
            
            req.body.basicDetail.customer_id = addDistributionNetworkData.insertId ? addDistributionNetworkData.insertId : 0;
            var addCustomerDealer = await Network.addNetworkDealerData(req.body.basicDetail);
            console.log('addCustomerDealer',addCustomerDealer);
            
            await db.commit();
            res.status(200).json({
                message: "Data added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Data not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if(e.code == 'ER_DUP_ENTRY'){
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>',myArray[1])
            var key = myArray[1];
            console.log('key',key);
            key = key.replace(/'/g, "");
            console.log('key',key);
            if(key == 'tg_customer_mobile_UN'){
                errMessage = 'Duplicate entry for mobile number - '+req.body.basicDetail.mobile;
            }else if(key == 'tg_customer_UN'){
                errMessage = 'Duplicate entry for email - '+req.body.basicDetail.email;
            }else{
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage',errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err : err
        });
        return false;
    }
};


exports.addDistributionNetworkLogs = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        // req.body.customer_id = decrypt(req.body.customer_id);
        
        var logData = {
            customer_id : req.body.customer_id,
            type :  'Remark Added',
            description : req.body.remark,
            created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
            created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
            created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '',
        }
        
        var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
        // console.log('distributionNetworkLogData',distributionNetworkLogData);
        
        if(distributionNetworkLogData.insertId){
            
            await db.commit();
            res.status(200).json({
                message: "Logs added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Logs not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        var errMessage = 'Error Occured';
        if(e.code == 'ER_DUP_ENTRY'){
            var sqlMessage = e.sqlMessage.replace(/'/, "")
            const myArray = sqlMessage.split("for key ");
            console.log('myArray ==>',myArray[1])
            var key = myArray[1];
            console.log('key',key);
            key = key.replace(/'/g, "");
            console.log('key',key);
            if(key == 'tg_customer.tg_customer_mobile_UN'){
                errMessage = 'Duplicate entry for mobile number - '+req.body.basicDetail.mobile;
            }else if(key == 'tg_customer.tg_customer_UN'){
                errMessage = 'Duplicate entry for email - '+req.body.basicDetail.email;
            }else{
                errMessage = 'Error Occured';
            }
        }
        console.log('errMessage',errMessage);
        res.status(200).json({
            message: errMessage,
            status: "error",
            err : err
        });
        return false;
    }
};


exports.updateDistributionNetwork = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();      
        
        req.body.basicDetail.cust_id = decrypt(req.body.basicDetail.cust_id);
        req.body.basicDetail.customer_id  = req.body.basicDetail.cust_id ;
        req.body.customer_id  = req.body.basicDetail.cust_id ;
        
        var updateDistributionNetworkData =  await Network.updateDistributionNetworkData(req.body.basicDetail);
        console.log('updateDistributionNetworkData',updateDistributionNetworkData);
        
        if(updateDistributionNetworkData.affectedRows){
            
            // if(req.body.contactDetail?.length){
            //     req.body.customer_id = addDistributionNetworkData.insertId;
            //     var addDistributionNetworkContactData =  await Network.addDistributionNetworkContactData(req.body);
            //     console.log('addDistributionNetworkContactData',addDistributionNetworkContactData);    
            // }
            
            // if(req.body.brandDetail?.length){
            //     req.body.customer_id = addDistributionNetworkData.insertId;
            //     var addDistributionNetworkBrandData =  await Network.assignDistributionNetworkBrandData(req.body);
            //     console.log('addDistributionNetworkBrandData',addDistributionNetworkBrandData);    
            // }
            
            
            console.log('bodyy --->', req.body);
            // return false;
            
            if(req.body.assignedUser?.length){
                var updateDistributionAssignedSalesUser = await Network.updateDistributionAssignedSalesUser(req.body);
                console.log('updateDistributionAssignedSalesUser',updateDistributionAssignedSalesUser);
                
                
                var addDistributionNetworkContactData =  await Network.assignDistributionNetworkUserData(req.body);
                console.log('addDistributionNetworkContactData',addDistributionNetworkContactData);    
            }
            
            var logData = {
                customer_id : req.body.basicDetail.customer_id,
                type : req.body.basicDetail.cust_type+ ' Details Added',
                description : 'Details added',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            var documentData = {
                customer_id : req.body.customer_id ? req.body.customer_id : 0,
                doc_type : req.body.basicDetail.doc_type ? req.body.basicDetail.doc_type : '',
                doc_type_value : req.body.doc_no ? req.body.basicDetail.doc_no : '',
                doc_filename : req.body.basicDetail.doc_image ? req.body.basicDetail.doc_image : '',
                doc_filename_1 : req.body.basicDetail.doc_image_1 ? req.body.basicDetail.doc_image_1 : '',
            };
            
            var deleteCustomerDocument = await Network.deleteCustomerDocument(req.body);
            console.log('deleteCustomerDocument',deleteCustomerDocument);
            
            var addCustomerDocument = await Network.addCustomerDocument(documentData);
            console.log('addCustomerDocument',addCustomerDocument);
            
            
            var deleteNetworkDealerData = await Network.deleteNetworkDealerData(req.body.basicDetail);
            console.log('deleteNetworkDealerData',deleteNetworkDealerData);
            
            req.body.basicDetail.customer_id = req.body.customer_id ? req.body.customer_id : 0;
            var addCustomerDealer = await Network.addNetworkDealerData(req.body.basicDetail);
            console.log('addCustomerDealer',addCustomerDealer);
            
            await db.commit();
            res.status(200).json({
                message: "Customer network added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Customer network not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.assignedSalesUserDistributionNetwork = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        if(req.body.userIds.length){
            var i = req.body.userIds.length - 1;
            var updateDistributionAssignedSalesUser = await Network.updateDistributionAssignedSalesUser(req.body);
            console.log('updateDistributionAssignedSalesUser',updateDistributionAssignedSalesUser);
            req.body.userIds.forEach(async element => {
                
                element.sfa_web_user_id = req.body.sfa_web_user_id;
                element.sfa_web_user_name = req.body.sfa_web_user_name;
                element.sfa_web_user_type = req.body.sfa_web_user_type;
                element.customer_id = req.body.customer_id;
                
                var addDistributionAssignedSalesUser =  await Network.addDistributionAssignedSalesUser(element);
                console.log('addDistributionAssignedSalesUser',addDistributionAssignedSalesUser);
                
                if(i == 0){
                    
                    var logData = {
                        customer_id : req.body.customer_id,
                        type : 'Assigned sales users',
                        description : 'Basic Details updated',
                        created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                        created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                        created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
                    }
                    var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
                    console.log('distributionNetworkLogData',distributionNetworkLogData);
                    
                    await db.commit();
                    res.status(200).json({
                        message: "Sales User Assigned",
                        status: "success",
                    });
                }
                i--;
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Sales User not Assigned",
                status: "error",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateDistributionNetworkBasicDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        req.body.basicDetail.customer_id = req.body.customer_id;
        var updateDistributionNetworkData =  await Network.updateDistributionNetworkData(req.body.basicDetail);
        console.log('updateDistributionNetworkData',updateDistributionNetworkData);
        
        if(updateDistributionNetworkData.affectedRows){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : req.body.basicDetail.cust_type+ ' Basic Details Updated',
                description : 'Basic Details updated',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Basic Details Updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Basic Details not Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateDistributionNetworkContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        var updateDistributionNetworkContactData =  await Network.updateDistributionNetworkContactData(req.body);
        console.log('updateDistributionNetworkContactData',updateDistributionNetworkContactData);
        
        if(updateDistributionNetworkContactData.affectedRows){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Contact Details Updated',
                description : 'Contact Details updated',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deleteDistributionNetworkContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        var deleteDistributionNetworkContactData =  await Network.deleteDistributionNetworkContactData(req.body);
        console.log('deleteDistributionNetworkContactData',deleteDistributionNetworkContactData);
        
        if(deleteDistributionNetworkContactData.affectedRows){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Contact Details Deleted',
                description : 'Contact Details Deleted',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addDistributionNetworkContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        req.body.customer_id = decrypt(req.body.customer_id);
        
        if(req.body.contactDetail.length){
            var addDistributionNetworkContactData =  await Network.addDistributionNetworkContactData(req.body);
            console.log('addDistributionNetworkContactData',addDistributionNetworkContactData);    
        }
        
        if(addDistributionNetworkContactData.insertId){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Contact Details Added',
                description : 'Contact Details Added',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getDistributionNetworkDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        req.body.customer_id = decrypt(req.body.customer_id);
        req.body.image_app_path = res.locals.image_app_path;
        
        var getBasicDetail = await Network.getDistributionNetworkBasicDetail(req.body);
        // console.log('getBasicDetail',getBasicDetail);
        
        var getContactDetail = await Network.getDistributionNetworkContactDetail(req.body);
        // console.log('getContactDetail',getContactDetail);
        
        var getDealerDetail = await Network.getDistributionNetworkDealerDetail(req.body);
        // console.log('getDistributionNetworkDealerDetail',getDistributionNetworkDealerDetail);
        
        var getDocumentDetail = await Network.getDistributionNetworkDocumentDetail(req.body);
        // console.log('getDocumentDetail',getDocumentDetail);
        
        var getAssignedBrandDetail = await Network.getAssignedBrandDetail(req.body);
        // console.log('getAssignedBrandDetail',getAssignedBrandDetail);
        
        var getAssignedUserDetail = await Network.getAssignedUserDetail(req.body);
        // console.log('getAssignedUserDetail',getAssignedUserDetail);
        
        var getAssignedSalesUserDetail = await Network.getAssignedSalesUserDetail(req.body);
        // console.log('getAssignedSalesUserDetail',getAssignedSalesUserDetail);
        
        
        var getDealerPartnerDetail = await Network.getDealerPartnerDetail(req.body);
        // console.log('getDealerPartnerDetail',getDealerPartnerDetail);
        
        
        var getLogDetail = await Network.getDistributionNetworkLogDetail(req.body);
        // console.log('getLogDetail',getLogDetail);
        
        // var getBonusListData = await Network.getDrPointListData({"filter" : {"customer_id" : req.body.customer_id}});
        // // console.log('getLogDetail',getLogDetail);
        
        res.status(200).json({
            message: "Network detail",
            status: "success",
            data :{
                basicDetail : getBasicDetail,
                contactDetail : getContactDetail,
                logDetail : getLogDetail,
                assignedBrandDetail : getAssignedBrandDetail,
                assignedUserDetail : getAssignedUserDetail,
                documentDetail : getDocumentDetail,
                dealerDetail : getDealerDetail,
                salesUserDetail : getAssignedSalesUserDetail,
                partnerDetail : getDealerPartnerDetail
            }
        });
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.deleteDrData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        req.body.customer_id = decrypt(req.body.customer_id);
        
        var deleteDrData = await Network.deleteDrData(req.body);
        // console.log('deleteDrData',deleteDrData);
        
        if(deleteDrData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Distribution network deleted",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Distribution network not deleted",
                status: "failed",
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateDrStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.customer_id = decrypt(req.body.customer_id);
        
        if(req.body.status == 'Verified'){
            req.body.is_lead = "0";
        }else{
            req.body.is_lead = "1";
        }
        
        var updateDrStatus =  await Network.updateDrStatus(req.body);
        // console.log('updateDrStatus',updateDrStatus);
        
        
        // LMS Code here start
        if(req.body.status == 'Verified'){
            
            // wallet point code start
            var getReferralBonusPoint =  await Network.getReferralBonusPoint(req.body);
            console.log('getReferralBonusPoint',getReferralBonusPoint);
            
            var referral_user_point = getReferralBonusPoint.referral_user_point ? getReferralBonusPoint.referral_user_point : 0;
            console.log('referral_user_point',referral_user_point);
            var referral_owner_point = getReferralBonusPoint.referral_owner_point ? getReferralBonusPoint.referral_owner_point : 0;
            console.log('referral_user_point',referral_owner_point);
            var welcome_point = getReferralBonusPoint.welcome_point ? getReferralBonusPoint.welcome_point : 0;
            console.log('welcome_point',welcome_point);
            
            
            
            
            var getBasicDetail = {};
            getBasicDetail = await Network.getDistributionNetworkBasicDetail(req.body);
            // console.log('getBasicDetail',getBasicDetail);
            
            if(getBasicDetail.cust_type_id == 10){
                var sales_user_point = getReferralBonusPoint.retailer_sales_user_point ? getReferralBonusPoint.retailer_sales_user_point : 0;
                console.log('welcome_point',sales_user_point);
            }else{
                var sales_user_point = getReferralBonusPoint.retailer_distributor_point ? getReferralBonusPoint.retailer_distributor_point : 0;
                console.log('welcome_point',sales_user_point);
            }
            
            var first_name = getBasicDetail.first_name ? getBasicDetail.first_name : ''
            
            var findReferredCode = {};
            if(req.body.referred_by_code){
                findReferredCode =  await Network.findReferredCode(req.body);
                console.log('findReferredCode',findReferredCode);
                console.log('findReferredCode',findReferredCode.id);
                
                if(findReferredCode.id){
                    
                    var sendReferralOwnerPointData = {
                        customer_id : findReferredCode.id ? findReferredCode.id :0,
                        against_customer_id : req.body.customer_id,
                        type : 'referral_owner',
                        point : referral_owner_point
                    }
                    
                    var sendReferralOwnerPoint = await Network.sendWalletPoints(sendReferralOwnerPointData);
                    console.log('sendReferralOwnerPoint',sendReferralOwnerPoint);
                    
                    
                    var title = 'Hey, your referral code used by '+first_name;
                    var description = 'Enjoy your wallet points';
                    var sendNotificationDataBody = {
                        customer_id:  findReferredCode.id ? findReferredCode.id :0,
                        title:  title ? title : '',
                        description : description ? description : '',
                        remark : description ? description : '',
                        notification_type : 'referral_user',
                    }
                    
                    var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
                    console.log('sendNotificationData',sendNotificationData);
                    
                    var sendReferralUserPointData = {
                        customer_id :  req.body.customer_id,
                        against_customer_id :  findReferredCode.id ? findReferredCode.id :0,
                        type : 'referral_user',
                        point : referral_user_point
                    };
                    
                    var sendReferralUserPoint = await Network.sendWalletPoints(sendReferralUserPointData);
                    console.log('sendReferralUserPoint',sendReferralUserPoint);
                    
                    var title = 'Hey '+first_name+', you get bonus in your wallet using signup referral code';
                    var description = 'Enjoy your wallet points';
                    var sendNotificationDataBody = {
                        customer_id: req.body.customer_id ? req.body.customer_id : 0,
                        title:  title ? title : '',
                        description : description ? description : '',
                        remark : description ? description : '',
                        notification_type : 'referral_user',
                    }
                    
                    var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
                    console.log('sendNotificationData',sendNotificationData);
                    
                }
            }else{
                
                var sendWelcomeUserPointData = {
                    customer_id : req.body.customer_id,
                    type : 'welcome_bonus',
                    point : welcome_point
                };
                
                var sendWelcomeUserPoint = await Network.sendWalletPoints(sendWelcomeUserPointData);
                console.log('sendWelcomeUserPoint',sendWelcomeUserPoint);
                
                var title = 'Hey '+first_name+', your profile status is verified from the admin';
                var description = 'Enjoy your welcome bonus wallet points';
                var sendNotificationDataBody = {
                    customer_id: req.body.customer_id ? req.body.customer_id : 0,
                    title:  title ? title : '',
                    description : description ? description : '',
                    remark : description ? description : '',
                    notification_type : 'welcome_bonus',
                }
                
                var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
                console.log('sendNotificationData',sendNotificationData);
                
                // send points to sales user code start
                if(getBasicDetail.created_by_type == 'app' && getBasicDetail.created_by){

                    var sendWelcomeUserPointData = {
                        user_id : getBasicDetail.created_by ? getBasicDetail.created_by : 0,
                        against_customer_id : getBasicDetail.id ? getBasicDetail.id : 0,
                        type : 'data_added',
                        point : sales_user_point
                    };
                    
                    var sendWelcomeUserPoint = await Network.sendWalletPoints(sendWelcomeUserPointData);
                    console.log('sendWelcomeUserPoint',sendWelcomeUserPoint);
                    
                    var title = 'Hey your added ' + getBasicDetail.company_name +' - ' +getBasicDetail.cust_type + ' status is verified from the admin';
                    var description = 'Enjoy your bonus wallet points';
                    var sendNotificationDataBody = {
                        user_id : getBasicDetail.created_by ? getBasicDetail.created_by : 0,
                        title:  title ? title : '',
                        description : description ? description : '',
                        remark : description ? description : '',
                        notification_type : 'data_added',
                    }
                    
                    var sendNotificationData =  await Notification.sendNotificationDataToUser(sendNotificationDataBody);
                    console.log('sendNotificationData',sendNotificationData);    
                }
                
                // send points to sales user code end
            }
            
            // wallet point code end
            
            
            
            var referredData = {
                id : req.body.customer_id,
                referred_by : findReferredCode.id ? findReferredCode.id :0,
                referred_by_name : findReferredCode.first_name ? findReferredCode.first_name :'',
                referred_by_code : req.body.referred_by_code ? req.body.referred_by_code : '',
            };
            
            
            var updateReferredDetails = await Network.updateReferredDetails(referredData);
            console.log('updateReferredDetails',updateReferredDetails);
            
        }
        
        
        if(updateDrStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Network Status updated",
                status: "success",
            });
        }else{
            await db.rollback()
            res.status(200).json({
                message: "Network Status not updated",
                status: "error",
            });
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateDrLoginStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.customer_id = decrypt(req.body.customer_id);
        
        var updateDrLoginStatus =  await Network.updateDrLoginStatus(req.body);
        // console.log('updateDrLoginStatus',updateDrLoginStatus);
        
        if(updateDrLoginStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Login Block Status updated",
                status: "success",
            });
        }else{
            await db.rollback()
            res.status(200).json({
                message: "Login Block Status not updated",
                status: "error",
            });
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addDrBonusPoint = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        // req.body.customer_id = decrypt(req.body.customer_id);
        
        req.body.type = 'bonus_point';
        var addBonusPointData =  await Network.addBonusPointData(req.body);
        console.log('addBonusPointData',addBonusPointData);    
        
        if(addBonusPointData.insertId){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Bonus Points Added',
                description : 'Bonus Points Added',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Bonus Points Added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Bonus Points not Added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getDrOrderList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.customer_id = decrypt(req.body.customer_id);
        
        req.body.filter = {
            customer_id : req.body.customer_id,
        }
        
        var getOrderCount = await Order.getOrderCount(req.body)
        console.log('getOrderCount',getOrderCount);
        var getOrderList = [];
        if(getOrderCount > 0){
            getOrderList =  await Order.getOrderList(req.body);
            // console.log('getOrderList',getOrderList);
        }
        
        res.status(200).json({
            message: "DR Order List",
            status: "success",
            result : {
                data: getOrderList,
                totalCount : getOrderCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getDrInvoiceList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.customer_id = decrypt(req.body.customer_id);
        
        req.body.filter = {
            customer_id : req.body.customer_id,
        }
        var getInvoiceCount = await Invoice.getInvoiceCount(req.body)
        
        var getInvoiceList = [];
        if(getInvoiceCount > 0){
            getInvoiceList =  await Invoice.getInvoiceList(req.body);
            // console.log('getInvoiceList',getInvoiceList);
        }
        
        res.status(200).json({
            message: "DR Invoice List",
            status: "success",
            result : {
                data: getInvoiceList,
                totalCount : getInvoiceCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getDrProductDiscountList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        if(req.body.customer_id){
            req.body.customer_id = decrypt(req.body.customer_id);
            
            req.body.filter = {
                customer_id : req.body.customer_id,
            }
        }
        
        
        var getProductDiscountListCount = await Network.getProductDiscountListCount(req.body)
        
        var getProductDiscountList = [];
        if(getProductDiscountListCount > 0){
            getProductDiscountList =  await Network.getProductDiscountList(req.body);
            // console.log('getProductDiscountList',getProductDiscountList);
        }
        
        res.status(200).json({
            message: "DR Product Discount List",
            status: "success",
            result : {
                data: getProductDiscountList,
                totalCount : getProductDiscountListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getDrPointList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        
        if(req.body.customer_id){
            req.body.customer_id = decrypt(req.body.customer_id);
            
            req.body.filter = {
                customer_id : req.body.customer_id,
            }
        }
        
        if(req.body.customer_encrypt_id){
            req.body.customer_id = decrypt(req.body.customer_encrypt_id);
            
            req.body.filter = {
                customer_id : req.body.customer_id,
            }
        }
        
        var getDrPointListCount = await Network.getDrPointListCount(req.body)
        
        var getDrPointList = [];
        if(getDrPointListCount > 0){
            getDrPointList =  await Network.getDrPointListData(req.body);
            // console.log('getDrPointList',getDrPointList);
        }
        
        res.status(200).json({
            message: "DR Point List",
            status: "success",
            result : {
                data: getDrPointList,
                totalCount : getDrPointListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getDrFollowupList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.customer_id = decrypt(req.body.customer_id);
        
        req.body.filter = {
            customer_id : req.body.customer_id,
        }
        
        var getFollowupDataCount = await SFA_masterModel.getFollowupDataCount(req.body)
        
        var getFollowupDataList = [];
        if(getFollowupDataCount > 0){
            getFollowupDataList =  await SFA_masterModel.getFollowupDataList(req.body);
            // console.log('getFollowupDataList',getFollowupDataList);
        }
        
        res.status(200).json({
            message: "Followup List",
            status: "success",
            result : {
                data: getFollowupDataList,
                totalCount : getFollowupDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getDrCheckinList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {        
        req.body.customer_id = decrypt(req.body.customer_id);
        
        req.body.filter = {
            customer_id : req.body.customer_id,
        }
        
        var getCheckinDataCount = await SFA_masterModel.getCheckinDataCount(req.body)
        
        var getCheckinDataList = [];
        if(getCheckinDataCount > 0){
            getCheckinDataList =  await SFA_masterModel.getCheckinDataList(req.body);
            // console.log('getCheckinDataList',getCheckinDataList);
        }
        
        res.status(200).json({
            message: "Checkin List",
            status: "success",
            result : {
                data: getCheckinDataList,
                totalCount : getCheckinDataCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addDistributionNetworkProductDiscount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        if(req.body.type == 'detail'){
            req.body.customer_id = decrypt(req.body.customer_id);            
        }
        
        var addDistributionNetworkProductDiscount =  await Network.addDistributionNetworkProductDiscount(req.body);
        console.log('addDistributionNetworkProductDiscount',addDistributionNetworkProductDiscount);    
        
        if(addDistributionNetworkProductDiscount.insertId){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Product Discount Added',
                description : 'Product Discount Added',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Product Discount Added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Product Discount not Added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateDistributionNetworkProductDiscount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        if(req.body.type == 'detail'){
            req.body.customer_id = decrypt(req.body.customer_id);            
        }
        var updateDistributionNetworkProductDiscount =  await Network.updateDistributionNetworkProductDiscount(req.body);
        console.log('updateDistributionNetworkProductDiscount',updateDistributionNetworkProductDiscount);    
        
        if(updateDistributionNetworkProductDiscount.affectedRows){
            
            var logData = {
                customer_id : req.body.customer_id,
                type : 'Product Discount Updated',
                description : 'Product Discount Updated',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : 0,
                created_by_type : req.body.sfa_web_user_type ? req.body.sfa_web_user_type : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Product Discount Updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Product Discount not Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};




// sheet upload module start


exports.getSheetUploadList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getSheetUploadCount = await Network.getSheetUploadCount(req.body)
        
        var getSheetUploadList = [];
        if(getSheetUploadCount > 0){
            getSheetUploadList =  await Network.getSheetUploadList(req.body);
            // console.log('getSheetUploadList',getSheetUploadList);
        }
        
        res.status(200).json({
            message: "Sheet Upload List",
            status: "success",
            result : {
                data: getSheetUploadList,
                totalCount : getSheetUploadCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateSheetPoints = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var updateSheetPoints = await Network.updateSheetPoints(req.body)
        
        if(updateSheetPoints.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Sheet data updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Sheet data not updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



// sheet upload module end



// project list module start


exports.getProjectList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getProjectListCount = await Network.getProjectListCount(req.body)
        
        var getProjectList = [];
        if(getProjectListCount > 0){
            getProjectList =  await Network.getProjectList(req.body);
            // console.log('getProjectList',getProjectList);
        }
        
        res.status(200).json({
            message: "Project List",
            status: "success",
            result : {
                data: getProjectList,
                totalCount : getProjectListCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



// project list module end

