const Notification = require("./../../models/web/notificationModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');


exports.getNotificationDataList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        
        if(req.body.type == 'sfa'){
            var getNotificationDataCount = await Notification.getNotificationDataCount_salesUser(req.body)
            console.log('getNotificationDataCount',getNotificationDataCount);
            var getNotificationDataList = [];
            if(getNotificationDataCount > 0){
                getNotificationDataList =  await Notification.getNotificationDataList_salesUser(req.body);
                console.log('getNotificationDataList',getNotificationDataList);
            }
        }else{
            var getNotificationDataCount = await Notification.getNotificationDataCount(req.body)
            console.log('getNotificationDataCount',getNotificationDataCount);
            var getNotificationDataList = [];
            if(getNotificationDataCount > 0){
                getNotificationDataList =  await Notification.getNotificationDataList(req.body);
                // console.log('getNotificationDataList',getNotificationDataList);
            }
        }
        
        res.status(200).json({
            message: "Notification List",
            status: "success",
            result : {
                data: getNotificationDataList,
                totalCount : getNotificationDataCount
            }
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getNotificationDataListDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        
        if(req.body.type == 'sfa'){
            
            var getNotificationDataDetail =  await Notification.getNotificationDataListDetail_salesUser(req.body);
            // console.log('getNotificationDataList',getNotificationDataList);
            
        }else{
            var getNotificationDataDetail =  await Notification.getNotificationDataListDetail(req.body);
            // console.log('getNotificationDataList',getNotificationDataList);
            
        }
        
        res.status(200).json({
            message: "Notification List",
            status: "success",
            result : {
                detail: getNotificationDataDetail,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.sendNotificationData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        if(req.body.customerIds?.length){
            
            req.body.notification_code = 'notification-'+generateRandomString(3);
            
            var i = req.body.customerIds.length - 1;
            req.body.customerIds.forEach(async element => {
                req.body.image_app_path = res.locals.image_app_path;
                
                var sendNotificationDataBody = {
                    customer_id: element.customer_id ? element.customer_id :0,
                    notification_code : req.body.notification_code ? req.body.notification_code : null,
                    title: req.body.title ? req.body.title : '',
                    remark: req.body.remark ? req.body.remark.replace(/<\/?[^>]+(>|$)/g, "") : '',
                    image: req.body.image ? req.body.image : '',
                    notification_type: req.body.notification_type ? req.body.notification_type : '',
                    image_app_path : res.locals.image_app_path
                }
                
                var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
                // console.log('sendNotificationData',sendNotificationData);
                
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Notification sent",
                        status: "success"
                    });
                }
                
                i--;
            });
        }else if(req.body.userIds?.length){
            req.body.notification_code = 'notification-'+generateRandomString(3);
            
            var i = req.body.userIds.length - 1;
            req.body.userIds.forEach(async element => {
                req.body.image_app_path = res.locals.image_app_path;
                
                var sendNotificationDataBody = {
                    user_id: element.user_id ? element.user_id :0,
                    notification_code : req.body.notification_code ? req.body.notification_code : null,
                    title: req.body.title ? req.body.title : '',
                    remark: req.body.remark ? req.body.remark.replace(/<\/?[^>]+(>|$)/g, "") : '',
                    image: req.body.image ? req.body.image : '',
                    notification_type: req.body.notification_type ? req.body.notification_type : '',
                    image_app_path : res.locals.image_app_path
                }
                
                var sendNotificationData =  await Notification.sendNotificationDataToUser(sendNotificationDataBody);
                // console.log('sendNotificationData',sendNotificationData);
                
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Notification sent",
                        status: "success"
                    });
                }
                
                i--;
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Notification sent",
                status: "error"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getNotificationTypeList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getNotificationTypeList = [];
        
        getNotificationTypeList =  await Notification.getNotificationTypeList(req.body);
        // console.log('getNotificationTypeList',getNotificationTypeList);
        
        res.status(200).json({
            message: "Notification Type List",
            status: "success",
            result : {
                data: getNotificationTypeList
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


function generateRandomString(n) {
    let randomString           = '';
    let characters       = 'abcdefghtyrqrva';
    
    for ( let i = 0; i < n; i++ ) {
        randomString += characters.charAt(Math.floor(Math.random()*characters.length));
    }
    return randomString;
}