const Offer = require("../../models/web/offerModel");
const Network = require("../../models/web/networkModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');
const Notification = require("../../models/web/notificationModel");


exports.getOfferCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusDataArray = [
            {"type" : 'all', 'name' : 'All', 'count' : 0},
            {"type" : 'active', 'name' : 'Active', 'count' : 0},
            {"type" : 'deactive', 'name' : 'Deactive', 'count' : 0}
        ]
        
        if(statusDataArray.length){
            var i = statusDataArray.length - 1;
            statusDataArray.forEach(async element => {
                element.filter = {};
                element.count =  await Offer.getOfferCount(element)
                if(i==0){
                    res.status(200).json({
                        message: "Offer Tab Count",
                        status: "success",
                        result : {
                            data: statusDataArray,
                        }  
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Offer Tab Count",
                status: "success",
                result : {
                    data: statusDataArray,
                }  
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getOfferList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getOfferCount = await Offer.getOfferCount(req.body)
        
        var getOfferListData = [];
        if(getOfferCount > 0){
            getOfferListData =  await Offer.getOfferListData(req.body);
            // console.log('getOfferListData',getOfferListData);
        }
        
        res.status(200).json({
            message: "Offer List",
            status: "success",
            result : {
                data: getOfferListData,
                totalCount : getOfferCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getOfferListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.limit = 100000;
        req.body.start = 0;
        
        var getOfferListData =  await Offer.getOfferListData(req.body);
        // console.log('getOfferListData',getOfferListData);
        
        var getOfferListDataExcel = [];
        if(getOfferListData.length){
            var i = getOfferListData.length - 1;
            
            getOfferListData.forEach((element,index) => {
                var data = {
                    'S.No.' : index + 1, 
                    'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                    'Title' : element.title ? element.title : '', 
                    'Offer Code': element.code ? element.code : '', 
                    'Offer Start Date': element.start_date ? moment(element.start_date).format('YYYY-MM-DD') : '0000-00-00', 
                    'Offer End Date': element.end_date ? moment(element.end_date).format('YYYY-MM-DD') : '0000-00-00', 
                    'Total Coupon': element.couponCount ? element.couponCount : 0, 
                    'Total Gift': element.giftCount ? element.giftCount : 0, 
                    'Status': element.status == 1 ? 'Active' : 'Inactive', 
                };
                getOfferListDataExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Offer List Excel",
                        status: "success",
                        result : {
                            data: getOfferListDataExcel,
                            file_name : 'Offer-List-Data'
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Offer List Excel",
                status: "success",
                result : {
                    data: getOfferListDataExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addOfferData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var addOffer = await Offer.addOfferData(req.body);
        // console.log('addOffer',addOffer);
        
        req.body.offer_id = addOffer.insertId ? addOffer.insertId : 0;
        
        if(req.body.offer_id){
          
            if(req.body.bannerData.length > 0){
                req.body.bannerData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferBanner = await Offer.addOfferBannerData(element);
                    // console.log('addOfferBanner',addOfferBanner);
                });   
            }
            
            if(req.body.schemeData.length > 0){
                req.body.schemeData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferSchemeData = await Offer.addOfferSchemeData(element);
                    // console.log('addOfferSchemeData',addOfferSchemeData);
                });   
            }
            
            if(req.body.giftData.length > 0){
                req.body.giftData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferGift = await Offer.addOfferGiftData(element);
                    // console.log('addOfferGift',addOfferGift);
                });
            }
            
            if(req.body.areaData.length > 0){
                req.body.areaData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferArea = await Offer.addOfferAreaData(element);
                    // console.log('addOfferArea',addOfferArea);
                });
            }
            
            
            var summaryData = {
                offer_id : req.body.offer_id ? req.body.offer_id : 0,   
                title : 'Offer Added',   
                description : 'Offer Added - Description Here',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            
            var addOfferSummary = await Offer.addOfferSummaryData(summaryData);
            // console.log('addOfferSummary',addOfferSummary);
            
        }
        
        
        if(addOffer.insertId){
            await db.commit();
            res.status(200).json({
                message: "Offer Added",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Offer not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateOfferData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.offer_id = req.body.id ? req.body.id : 0;
        
        var updateOfferData = await Offer.updateOfferData(req.body);
        // console.log('updateOfferData',updateOfferData);
        
        
        if(req.body.offer_id){
            
            var updateCouponValidUpto = Offer.updateCouponValidUpto(req.body);
            console.log('updateCouponValidUpto',updateCouponValidUpto);
            
            if(req.body.bannerData.length > 0){
                var deleteOfferBannerData = await Offer.deleteOfferBannerData(req.body);
                console.log('deleteOfferBannerData',deleteOfferBannerData);
                req.body.bannerData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    // element.banner_id = element.id ? element.id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferBannerData = await Offer.addOfferBannerData(element);
                    // console.log('addOfferBannerData',addOfferBannerData);
                });   
            }
            
            if(req.body.schemeData.length > 0){
                var deleteOfferSchemeData = await Offer.deleteOfferSchemeData(req.body);
                console.log('deleteOfferSchemeData',deleteOfferSchemeData);
                req.body.schemeData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    // element.banner_id = element.id ? element.id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferSchemeData = await Offer.addOfferSchemeData(element);
                    // console.log('addOfferSchemeData',addOfferSchemeData);
                });   
            }
            
            if(req.body.giftData.length > 0){
                var deleteOfferGiftData = await Offer.deleteOfferGiftData(req.body);
                console.log('deleteOfferGiftData',deleteOfferGiftData);
                
                req.body.giftData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferGiftData = await Offer.addOfferGiftData(element);
                    // console.log('addOfferGiftData',addOfferGiftData);
                });
            }
            
            if(req.body.areaData.length > 0){
                var deleteOfferAreaData = await Offer.deleteOfferAreaData(req.body);
                console.log('deleteOfferAreaData',deleteOfferAreaData);
                
                req.body.areaData.forEach(async element => {
                    element.offer_id = req.body.offer_id ? req.body.offer_id : 0;
                    element.sfa_web_user_id =  req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '';
                    element.sfa_web_user_name =  req.body.sfa_web_user_name ? req.body.sfa_web_user_name : '';
                    var addOfferArea = await Offer.addOfferAreaData(element);
                    // console.log('addOfferArea',addOfferArea);
                });
            }
            
            
            var summaryData = {
                offer_id : req.body.offer_id ? req.body.offer_id : 0,   
                title : 'Offer Updated',   
                description : 'Offer Updated - Description Here',
                created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '',
                created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
            }
            
            var addOfferSummary = await Offer.addOfferSummaryData(summaryData);
            // console.log('addOfferSummary',addOfferSummary);
            
        }
        
        
        if(updateOfferData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Offer Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Offer not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateOfferGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.offer_id = req.body.id ? req.body.id : 0;
        
        var updateOfferGiftData = await Offer.updateOfferGiftData(req.body);
        // console.log('updateOfferGiftData',updateOfferGiftData);
        
        var summaryData = {
            offer_id : req.body.offer_id ? req.body.offer_id : 0,   
            title : 'Offer Gift Updated',   
            description : 'Offer Updated - Description Here',
            created_by : req.body.sfa_web_user_id ? req.body.sfa_web_user_id : '',
            created_by_name : req.body.sfa_web_user_name ? req.body.sfa_web_user_name : ''
        }
        
        var addOfferSummary = await Offer.addOfferSummaryData(summaryData);
        // console.log('addOfferSummary',addOfferSummary);
        
        if(updateOfferGiftData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Offer Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Offer not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getOfferDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        req.body.offer_id = decrypt(req.body.offer_id);
        
        var getOfferDetail = await Offer.getOfferDetail(req.body);
        // console.log('getOfferDetail',getOfferDetail);
        
        var getOfferBanner = await Offer.getOfferBannerData(req.body);
        // console.log('getOfferBanner',getOfferBanner);
        
        var getOfferScheme = await Offer.getOfferSchemeData(req.body);
        // console.log('getOfferScheme',getOfferScheme);
        
        var getOfferGift = await Offer.getOfferGiftData(req.body);
        // console.log('getOfferGift',getOfferGift);
        
        var getOfferArea = await Offer.getOfferAreaData(req.body);
        // console.log('getOfferArea',getOfferArea);
        
        var getOfferSummary = await Offer.getOfferSummaryData(req.body);
        // console.log('getOfferSummary',getOfferSummary);
        
        res.status(200).json({
            message: "Offer Detail",
            status: "success",
            result : {
                data: getOfferDetail,
                bannerData : getOfferBanner,
                schemeData : getOfferScheme,
                giftData : getOfferGift,
                areaData : getOfferArea,
                summaryData : getOfferSummary,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deleteOfferData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        req.body.offer_id = decrypt(req.body.offer_id);
        
        var deleteOfferData = await Offer.deleteOfferData(req.body);
        // console.log('deleteOfferData',deleteOfferData);
        var deleteOfferGiftData = await Offer.deleteOfferGiftData(req.body);
        // console.log('deleteOfferGiftData',deleteOfferGiftData);
        
        
        if(deleteOfferData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Offer Deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Offer Deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateOfferStatusData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.offer_id = decrypt(req.body.offer_id);
        
        var updateOfferStatusData = await Offer.updateOfferStatusData(req.body);
        // console.log('updateOfferStatusData',updateOfferStatusData);
        
        if(updateOfferStatusData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Offer status Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Offer status not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getGiftList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        var getGiftCount = await Offer.getGiftCount(req.body)
        
        var getGiftListData = [];
        if(getGiftCount > 0){
            getGiftListData =  await Offer.getGiftListData(req.body);
            // console.log('getGiftListData',getGiftListData);
        }
        
        res.status(200).json({
            message: "Gift List",
            status: "success",
            result : {
                data: getGiftListData,
                totalCount : getGiftCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getGiftListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        
        req.body.limit = 100000;
        req.body.start = 0;
        
        var  getGiftListData =  await Offer.getGiftListData(req.body);
        // console.log('getGiftListData',getGiftListData);
        
        var getGiftListDataExcel = [];
        if(getGiftListData.length){
            var i = getGiftListData.length - 1;
            
            getGiftListData.forEach((element,index) => {
                var data = {
                    'S.No.' : index + 1, 
                    'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                    'Title' : element.title ? element.title : '', 
                    'Points': element.points ? element.points : 0, 
                    'Retailer Redeem Requests': element.retailer_redeem_req ? element.retailer_redeem_req : 0, 
                    'Distributor Redeem Requests': element.distributor_redeem_req ? element.distributor_redeem_req : 0, 
                    'Sales User Redeem Requests': element.sales_user_redeem_req ? element.sales_user_redeem_req : 0, 
                    'Pending Requests': element.pending_redeem_req ? element.pending_redeem_req : 0,
                    'Approved Requests':  element.approved_redeem_req ? element.approved_redeem_req : 0,
                    'Reject Requests':  element.reject_redeem_req ? element.reject_redeem_req : 0,
                    'Total Requests': element.total_redeem_req ? element.total_redeem_req : 0,
                };
                getGiftListDataExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Gift List excel",
                        status: "success",
                        result : {
                            data: getGiftListDataExcel,
                            file_name : 'Gift-List-Data'
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Gift List excel",
                status: "success",
                result : {
                    data: getGiftListDataExcel,
                }
            });
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getGiftListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All','status' : ""},
            {'name' : 'Active', 'status' : 1},
            {'name' : 'Inactive', 'status' : 0}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await Offer.getGiftCount({'filter' : {'statusName' : element.name, 'status' : element.status}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Gift List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Gift List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.addGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var addGiftData = await Offer.addGiftData(req.body);
        // console.log('addGiftData',addGiftData);
        
        if(addGiftData.insertId){
            
            await db.commit();
            res.status(200).json({
                message: "Gift Added",
                status: "success",
            });
            
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Gift not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.updateGiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.image_app_path = res.locals.image_app_path;
        req.body.gift_id = decrypt(req.body.gift_id);
        
        var updateGiftData = await Offer.updateGiftData(req.body);
        // console.log('updateGiftData',updateGiftData);
        
        if(updateGiftData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Gift updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Gift data not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.deletegiftData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        req.body.gift_id = decrypt(req.body.gift_id);
        
        var deleteGiftData = await Offer.deletegiftData(req.body);
        // console.log('deleteOfferData',deleteOfferData);
        // var deleteOfferGiftData = await Offer.deleteOfferGiftData(req.body);
        // console.log('deleteOfferGiftData',deleteOfferGiftData);
        
        
        if(deleteGiftData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Gift Deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Gift Deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.updateGiftStatusData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.gift_id = decrypt(req.body.gift_id);
        
        var updateGiftStatusData = await Offer.updateGiftStatusData(req.body);
        // console.log('updateOfferStatusData',updateOfferStatusData);
        
        if(updateGiftStatusData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Gift status Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Gift status not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getGiftDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        req.body.gift_id = decrypt(req.body.gift_id);
        
        var getGiftDetail = await Offer.getGiftDetail(req.body);
        // console.log('getGiftDetail',getGiftDetail);
        
        res.status(200).json({
            message: "Gift Detail",
            status: "success",
            result : {
                data: getGiftDetail
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};




exports.addCouponData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.coupon_ref = 'COUPON-'+generateRandomString(8);
        
        for (let index = 0; index < parseInt(req.body.no_of_code); index++) {
            const element = req.body;
            element.coupon_code = generateRandomString(10);
            var addCouponData = await Offer.addCouponData(element);
            // console.log('addCouponData',addCouponData);
        }
        
        if(addCouponData.insertId){
            await db.commit();
            res.status(200).json({
                message: "Coupon Added",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not Added",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateCouponData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.coupon_id = decrypt(req.body.coupon_id);
        
        var updateCouponData = await Offer.updateCouponData(req.body);
        // console.log('updateCouponData',updateCouponData);
        
        if(updateCouponData.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Coupon updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.reopenCouponData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        // console.log('body ==>', req.body);
        if(req.body.length){
            var i = req.body.length - 1;
            
            req.body.forEach(async element => {
                // console.log('element',element);
                
                element.coupon_id = decrypt(element.coupon_id);
                element.customer_id = decrypt(element.customer_id);
                
                // console.log('element',element);
                var getCouponDetail = await Offer.getCouponDetail(element);
                // console.log('getCouponDetail',getCouponDetail);
                
                if(getCouponDetail.reopen == 0){
                    element.sfa_web_user_name = req.body.sfa_web_user_name;
                    element.sfa_web_user_id = req.body.sfa_web_user_id;
                    var reopenCouponData = await Offer.reopenCouponData(element);
                    // console.log('reopenCouponData',reopenCouponData);
                    
                    if(reopenCouponData.affectedRows){
                        
                        var sendReferralOwnerPointData = {
                            customer_id : element.customer_id,
                            coupon_id : element.coupon_id,
                            type : 'coupon_reopen',
                            point : -element.point
                        }
                        console.log('sendReferralOwnerPointData',sendReferralOwnerPointData);
                        var sendReferralOwnerPoint = await Network.sendWalletPoints(sendReferralOwnerPointData);
                        console.log('sendReferralOwnerPoint',sendReferralOwnerPoint);
                        
                    }
                }
                
                if(i == 0){
                    await db.commit();
                    res.status(200).json({
                        message: "Coupon reopened",
                        status: "success",
                    });
                }
                i--;
            });
            
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not updated",
                status: "failed",
            });
        }
        
        
        return false;
        
        req.body.coupon_id = decrypt(req.body.coupon_id);
        req.body.customer_id = decrypt(req.body.customer_id);
        
        var getCouponDetail = await Offer.getCouponDetail(req.body);
        console.log('getCouponDetail',getCouponDetail);
        
        if(getCouponDetail.reopen == 1){
            await db.rollback();
            res.status(200).json({
                message: "Coupon already reopened",
                status: "error",
            });
            return false;
        }
        
        var reopenCouponData = await Offer.reopenCouponData(req.body);
        // console.log('reopenCouponData',reopenCouponData);
        
        if(reopenCouponData.affectedRows){
            
            var sendReferralOwnerPointData = {
                customer_id : req.body.customer_id,
                coupon_id : req.body.coupon_id,
                type : 'coupon_reopen',
                point : -req.body.point
            }
            
            var sendReferralOwnerPoint = await Network.sendWalletPoints(sendReferralOwnerPointData);
            console.log('sendReferralOwnerPoint',sendReferralOwnerPoint);
            
            await db.commit();
            res.status(200).json({
                message: "Coupon reopened",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deleteMultipleCoupon = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // req.body.coupon_id = decrypt(req.body.coupon_id);
        var i = req.body.couponIdArray.length - 1;
        if(req.body.couponIdArray.length){
            req.body.couponIdArray.forEach(async element => {
                // console.log('element',element);
                req.body.tg_coupon_id = element;
                var deleteMultipleCoupon = await Offer.deleteMultipleCoupon(req.body);
                // console.log('updateCouponData',deleteMultipleCoupon);
                
                if(i==0){
                    await db.commit();
                    res.status(200).json({
                        message: "Coupon deleted",
                        status: "success",
                    });
                }
                i--;
            });    
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not deleted",
                status: "failed",
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deleteCouponGroup = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // req.body.coupon_id = decrypt(req.body.coupon_id);
        
        var deleteCouponGroup = await Offer.deleteCouponGroup(req.body);
        // console.log('deleteCouponGroup',deleteCouponGroup);
        
        if(deleteCouponGroup.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Coupon deleted",
                status: "success",
            });   
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Coupon not deleted",
                status: "failed",
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        
        console.log('filter', req.body.filter.used_by);
        
        
        
        var getCustTypeById = 0;
        if(req.body.filter.used_by != undefined){
            
            getCustTypeById = await Network.getCustTypeById({id : req.body.filter.used_by});
            console.log('getCustTypeById',getCustTypeById);
            if(getCustTypeById.cust_type_id == 3){
                req.body.filter.used_by_carpenter = req.body.filter.used_by;
                req.body.filter.used_by = 0;
            }else if(getCustTypeById.cust_type_id == 4){
                req.body.filter.used_by_architect = req.body.filter.used_by;
                req.body.filter.used_by = 0;
            }else{
                req.body.filter.used_by = req.body.filter.used_by;
            }
        }
        
        if(req.body.filter.statusName == 'Scanned'){
            
            getCustTypeById = await Network.getCustTypeById({id : decrypt(req.body.customer_encrypt_id)});
            console.log('getCustTypeById',getCustTypeById);
            
            if(getCustTypeById.cust_type_id == 3){
                req.body.filter.statusName = 'Scanned By Carpenter';
            }else if(getCustTypeById.cust_type_id == 4){
                req.body.filter.statusName = 'Scanned By Architect';
            }else{
                req.body.filter.statusName = '';
            }
        }
        
        if(req.body.customer_encrypt_id){
            var getCustTypeById = await Network.getCustTypeById({id : decrypt(req.body.customer_encrypt_id)});
            console.log('getCustTypeById',getCustTypeById);
            if(getCustTypeById.cust_type_id == 3){
                req.body.filter.used_by_carpenter =decrypt(req.body.customer_encrypt_id);
                req.body.customer_encrypt_id = '';
            }else if(getCustTypeById.cust_type_id == 4){
                req.body.filter.used_by_architect = decrypt(req.body.customer_encrypt_id);
                req.body.customer_encrypt_id = '';
            }else{
                req.body.filter.used_by = decrypt(req.body.customer_encrypt_id);
            }
        }
        
        // console.log('filter', req.body.filter);
        
        var getCouponCount = await Offer.getCouponCount(req.body)
        
        var getCouponListData = [];
        if(getCouponCount > 0){
            req.body.cust_type_id = getCustTypeById.cust_type_id ? getCustTypeById.cust_type_id : 0
            getCouponListData =  await Offer.getCouponListData(req.body);
            // console.log('getCouponListData',getCouponListData);
        }
        
        res.status(200).json({
            message: "Coupon List",
            status: "success",
            result : {
                data: getCouponListData,
                totalCount : 0,
                totalCount : getCouponCount,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        req.body.limit = 10000000;
        req.body.start = 0;
        
        var getCouponListData =  await Offer.getCouponListData(req.body);
        // console.log('getCouponListData',getCouponListData);
        
        
        var getCouponListDataExcel = [];
        if(getCouponListData.length){
            var i = getCouponListData.length - 1;
            
            getCouponListData.forEach((element,index) => {
                
                if(req.body.filter.statusName == 'Available'){
                    var data = {
                        'S.No.' : index + 1, 
                        'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                        'Product': element.product_name ? element.product_name : '',
                        'Product Code': element.product_code ? element.product_code : '', 
                        'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                        'Offer' : element.offer_title ? element.offer_title : '', 
                        'Offer code' : element.offer_code ? element.offer_code : '', 
                        'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                        'Retailer Points': element.retailer_point ? element.retailer_point : 0, 
                        'Distributor Points': element.distributor_point ? element.distributor_point : 0,
                        'Sales User Points': element.sales_user_point ? element.sales_user_point : 0, 
                        // 'Remarks': ''
                    };
                }else if(req.body.filter.statusName == 'Scanned' || req.body.filter.statusName == 'Scanned By Retailer' || req.body.filter.statusName == 'Scanned By Carpenter'){ 
                    
                    var cust_type = req.body.filter.statusName == 'Scanned By Retailer' ? 'Retailer' : 'Carpenter';
                    if(req.body.filter.statusName == 'Scanned By Carpenter'){
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Used Date': element.used_by_carpenter_on ? moment(element.used_by_carpenter_on).format('YYYY-MM-DD') : '', 
                            'Used By': element.used_by_carpenter_name ? element.used_by_carpenter_name : '', 
                            'Carpenter Mobile No.': element.used_by_carpenter_mobile ? element.used_by_carpenter_mobile : '', 
                            'Carpenter City': element.city ? element.city : '', 
                            'Carpenter District': element.district_name ? element.district_name : '', 
                            'Carpenter State': element.state_name ? element.state_name : '', 
                            'Carpenter Pincode': element.pincode ? element.pincode : '', 
                            'Points':  element.carpenter_point ? element.carpenter_point : 0
                        };
                    }else if(req.body.filter.statusName == 'Scanned By Retailer'){
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00',
                            'Product': element.product_name ? element.product_name: '',
                            'Product': element.product_code ? element.product_code: '',
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Scan Date': element.used_by_retailer_on ? moment(element.used_by_retailer_on).format('YYYY-MM-DD') : '', 
                            'Scan By': element.used_by_retailer_name ? element.used_by_retailer_name : '', 
                            'retailer Mobile No.': element.used_by_retailer_mobile ? element.used_by_retailer_mobile : '',
                            'retailer District': element.used_by_retailer_district_name ? element.used_by_retailer_district_name : '', 
                            'retailer State': element.used_by_retailer_state_name ? element.used_by_retailer_state_name : '', 
                            'Points':  element.retailer_point ? element.retailer_point : 0
                        };
                    }else{
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Used Date': element.used_on ? moment(element.used_on).format('YYYY-MM-DD') : '', 
                            'Used By': element.used_by_name ? element.used_by_name : '', 
                            'Mobile No.': element.used_by_mobile ? element.used_by_mobile : '', 
                            'City': element.city ? element.city : '', 
                            'District': element.district_name ? element.district_name : '', 
                            'State': element.state_name ? element.state_name : '', 
                            'Pincode': element.pincode ? element.pincode : '', 
                            'Points':  element.point ? element.point : 0
                        };
                    }
                    
                }else{
                    var data = {
                        'S.No.' : index + 1, 
                        'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                        'Product': element.product_name ? element.product_name : '',
                        'Product Code': element.product_code ? element.product_code : '', 
                        'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                        'Offer' : element.offer_title ? element.offer_title : '', 
                        'Offer code' : element.offer_code ? element.offer_code : '', 
                        'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                        'Retailer Points': element.retailer_point ? element.retailer_point : 0, 
                        'Distributor Points': element.distributor_point ? element.distributor_point : 0,
                        'Sales User Points': element.sales_user_point ? element.sales_user_point : 0, 
                    };
                }
                
                
                getCouponListDataExcel.push(data);
                
                if(i==0){
                    var file_name = req.body.filter.statusName ? req.body.filter.statusName +'-Coupon-Data' : 'Coupon-Data';
                    
                    res.status(200).json({
                        message: "Coupon List Excel",
                        status: "success",
                        result : {
                            data: getCouponListDataExcel,
                            file_name : file_name
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Coupon List Excel",
                status: "success",
                result : {
                    data: getCouponListDataExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponRedeemRequestListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        req.body.limit = 10000000;
        req.body.start = 0;
        
        var getRedeemRequestDetail = await Offer.getRedeemRequestDetail({"redeem_request_id" : req.body.filter.redeem_request_id});
        // console.log('getRedeemRequestDetail',getRedeemRequestDetail);
        
        var getCouponListData =  await Offer.getCouponListData(req.body);
        // console.log('getCouponListData',getCouponListData);
        
        
        var getCouponListDataExcel = [];
        if(getCouponListData.length){
            var i = getCouponListData.length - 1;
            
            getCouponListData.forEach((element,index) => {
                
                if(req.body.filter.statusName == 'Available'){
                    var data = {
                        'S.No.' : index + 1, 
                        'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                        'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                        'Category Name': element.product_category ? element.product_category : '', 
                        'Product Name': element.product_name ? element.product_name : '', 
                        'Offer' : element.offer_title ? element.offer_title : '', 
                        'Offer code' : element.offer_code ? element.offer_code : '', 
                        'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                        'Carpenter Points': element.carpenter_point ? element.carpenter_point : 0, 
                        'Architect Points': element.architect_point ? element.architect_point : 0, 
                        'Remarks': ''
                    };
                }else if(req.body.filter.statusName == 'Scanned' || req.body.filter.statusName == 'Scanned By Architect' || req.body.filter.statusName == 'Scanned By Carpenter'){ 
                    
                    var cust_type = req.body.filter.statusName == 'Scanned By Architect' ? 'Architect' : 'Carpenter';
                    if(req.body.filter.statusName == 'Scanned By Carpenter'){
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Used Date': element.used_by_carpenter_on ? moment(element.used_by_carpenter_on).format('YYYY-MM-DD') : '', 
                            'Used By': element.used_by_carpenter_name ? element.used_by_carpenter_name : '', 
                            'Carpenter Mobile No.': element.used_by_carpenter_mobile ? element.used_by_carpenter_mobile : '', 
                            'Carpenter City': element.city ? element.city : '', 
                            'Carpenter District': element.district_name ? element.district_name : '', 
                            'Carpenter State': element.state_name ? element.state_name : '', 
                            'Carpenter Pincode': element.pincode ? element.pincode : '', 
                            'Redeem Date' : getRedeemRequestDetail.created_on ? moment(getRedeemRequestDetail.created_on).format('YYYY-MM-DD') : '0000-00-00',
                            'Gift' : getRedeemRequestDetail.gift_title ? getRedeemRequestDetail.gift_title : '',
                            'Points':  element.carpenter_point ? element.carpenter_point : 0
                        };
                    }else if(req.body.filter.statusName == 'Scanned By Architect'){
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Used Date': element.used_on ? moment(element.used_on).format('YYYY-MM-DD') : '', 
                            'Used By': element.used_by_architect_name ? element.used_by_architect_name : '', 
                            'Architect Mobile No.': element.used_by_architect_mobile ? element.used_by_architect_mobile : '', 
                            'Architect City': element.city ? element.city : '', 
                            'Architect District': element.district_name ? element.district_name : '', 
                            'Architect State': element.state_name ? element.state_name : '', 
                            'Architect Pincode': element.pincode ? element.pincode : '', 
                            'Redeem Date' : getRedeemRequestDetail.created_on ? moment(getRedeemRequestDetail.created_on).format('YYYY-MM-DD') : '0000-00-00',
                            'Gift' : getRedeemRequestDetail.gift_title ? getRedeemRequestDetail.gift_title : '',
                            'Points':  element.architect_point ? element.architect_point : 0
                        };
                    }else{
                        var data = {
                            'S.No.' : index + 1, 
                            'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                            'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                            'Offer' : element.offer_title ? element.offer_title : '', 
                            'Offer code' : element.offer_code ? element.offer_code : '', 
                            'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                            'Used Date': element.used_on ? moment(element.used_on).format('YYYY-MM-DD') : '', 
                            'Used By': element.used_by_name ? element.used_by_name : '', 
                            'Mobile No.': element.used_by_mobile ? element.used_by_mobile : '', 
                            'City': element.city ? element.city : '', 
                            'District': element.district_name ? element.district_name : '', 
                            'State': element.state_name ? element.state_name : '', 
                            'Pincode': element.pincode ? element.pincode : '', 
                            'Redeem Date' : getRedeemRequestDetail.created_on ? moment(getRedeemRequestDetail.created_on).format('YYYY-MM-DD') : '0000-00-00',
                            'Gift' : getRedeemRequestDetail.gift_title ? getRedeemRequestDetail.gift_title : '',
                            'Points':  element.point ? element.point : 0
                        };
                    }
                    
                }else{
                    var data = {
                        'S.No.' : index + 1, 
                        'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                        'Coupon Code' : element.coupon_code ? element.coupon_code : '', 
                        'Category Name': element.product_category ? element.product_category : '', 
                        'Product Name': element.product_name ? element.product_name : '', 
                        'Offer' : element.offer_title ? element.offer_title : '', 
                        'Offer code' : element.offer_code ? element.offer_code : '', 
                        'Offer Valid Upto' : element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '', 
                        'Carpenter Points': element.carpenter_point ? element.carpenter_point : 0, 
                        'Architect Points': element.architect_point ? element.architect_point : 0, 
                        'Remarks': ''
                    };
                }
                
                
                getCouponListDataExcel.push(data);
                
                if(i==0){
                    var file_name = req.body.filter.statusName ? req.body.filter.statusName +'-Coupon-Data' : 'Coupon-Data';
                    
                    res.status(200).json({
                        message: "Coupon List Excel",
                        status: "success",
                        result : {
                            data: getCouponListDataExcel,
                            file_name : file_name
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Coupon List Excel",
                status: "success",
                result : {
                    data: getCouponListDataExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponGrouppedList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        var getCouponGrouppedCount = await Offer.getCouponGrouppedCount(req.body)
        
        var getCouponGrouppedList = [];
        if(getCouponGrouppedCount > 0){
            getCouponGrouppedList =  await Offer.getCouponGrouppedList(req.body);
            // console.log('getCouponGrouppedList',getCouponGrouppedList);
        }
        
        res.status(200).json({
            message: "Coupon groupped list",
            status: "success",
            result : {
                data: getCouponGrouppedList,
                totalCount : getCouponGrouppedCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponGrouppedListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        req.body.limit = 100000;
        req.body.start = 0;
        
        var getCouponGrouppedList =  await Offer.getCouponGrouppedList(req.body);
        // console.log('getCouponGrouppedList',getCouponGrouppedList);
        
        res.status(200).json({
            message: "Coupon groupped list excel",
            status: "success",
            result : {
                data: getCouponGrouppedList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getCouponListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Available'},
            {'name' : 'Scanned By Retailer'},
            // {'name' : 'Scanned By Distributor'},
            // {'name' : 'Scanned By Sales User'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await Offer.getCouponCount({'filter' : {'statusName' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Coupon List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Coupon List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getRedeemRequestList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        
        var getRedeemRequestCount = await Offer.getRedeemRequestCount(req.body)
        
        var getRedeemRequestListData = [];
        if(getRedeemRequestCount > 0){
            getRedeemRequestListData =  await Offer.getRedeemRequestListData(req.body);
            // console.log('getRedeemRequestListData',getRedeemRequestListData);
        }
        
        res.status(200).json({
            message: "Redeem Request List",
            status: "success",
            result : {
                data: getRedeemRequestListData,
                totalCount : getRedeemRequestCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getRedeemRequestListExcel = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();
        req.body.image_app_path = res.locals.image_app_path;  
        req.body.limit = 100000;
        req.body.start = 0;
        var getRedeemRequestListData =  await Offer.getRedeemRequestListData(req.body);
        // console.log('getRedeemRequestListData',getRedeemRequestListData);
        
        var getRedeemRequestListDataExcel = [];
        if(getRedeemRequestListData.length){
            var i = getRedeemRequestListData.length - 1;
            
            getRedeemRequestListData.forEach((element,index) => {
                var data = {
                    'S.No.' : index + 1, 
                    'Date' : element.created_on ? moment(element.created_on).format('YYYY-MM-DD') : '0000-00-00', 
                    'Type' : element.cust_type_name ? element.cust_type_name : '',
                    'Name' : element.first_name ? element.first_name + ' '+ element.last_name : '', 
                    'Req. ID': element.redeem_request_id ? element.redeem_request_id : '', 
                    'Mobile No.': element.mobile ? element.mobile : '',
                    'State': element.state_name ? element.state_name : '', 
                    'A/C Status': element.redeem_status ? element.redeem_status : '', 
                    'Gift': element.gift_title ? element.gift_title : '', 
                    'Gift value': element.gift_point ? element.gift_point : 0, 
                    'Offer': element.offer_title ? element.offer_title : '', 
                    'Valid Upto': element.offer_end_date ? moment(element.offer_end_date).format('YYYY-MM-DD') : '0000-00-00', 
                    'Redeem Status': element.redeem_status ? element.redeem_status : '', 
                    'Reject Reason': '',
                    'Gift Status': element.gift_status ? element.gift_status : '', 
                    'Shipped Type': element.shipment_type ? element.shipment_type : '', 
                    'Shipped Information': element.shipment_address ? element.shipment_address : '', 
                    'Estimated Date': element.shipment_delivery_time ? moment(element.shipment_delivery_time).format('YYYY-MM-DD') : '0000-00-00', 
                };
                getRedeemRequestListDataExcel.push(data);
                
                if(i==0){
                    res.status(200).json({
                        message: "Redeem Request List Excel",
                        status: "success",
                        result : {
                            data: getRedeemRequestListDataExcel,
                            file_name : 'Redeem-Request-Data'
                        }
                    });
                }
                
                i--;
            });
        }else{
            res.status(200).json({
                message: "Redeem Request List Excel",
                status: "success",
                result : {
                    data: getRedeemRequestListDataExcel,
                }
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getRedeemRequestListCount = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var statusArrayData = [
            {'name' : 'All'},
            {'name' : 'Pending'},
            {'name' : 'Approved'},
            {'name' : 'Rejected'},
            // {'name' : 'Suspect'},
            // {'name' : 'Dispatched'},
            // {'name' : 'Received'}
        ]
        
        if(statusArrayData.length){
            var i = statusArrayData.length - 1;
            statusArrayData.forEach(async element => {
                element.count = await Offer.getRedeemRequestCount({'filter' : {'statusName' : element.name}});
                // console.log('element.count',element.count);
                if(i == 0){
                    res.status(200).json({
                        message: "Redeem Request List Count",
                        status: "success",
                        result : {
                            data: statusArrayData,
                        }
                    });
                }
                i--;
            });
        }else{
            res.status(200).json({
                message: "Coupon List Count",
                status: "success",
                result : {
                    data: statusArrayData,
                }
            });
        }
        
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getRedeemRequestDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.image_app_path = res.locals.image_app_path;
        req.body.redeem_request_id = decrypt(req.body.redeem_request_id);
        
        var getRedeemRequestDetail = await Offer.getRedeemRequestDetail(req.body);
        // console.log('getRedeemRequestDetail',getRedeemRequestDetail);
        
        res.status(200).json({
            message: "Redeem Request Detail",
            status: "success",
            result : {
                data: getRedeemRequestDetail
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateRedeemStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.redeem_request_id = decrypt(req.body.redeem_request_id);
        
        var updateRedeemStatus = await Offer.updateRedeemStatus(req.body);
        // console.log('updateRedeemStatus',updateRedeemStatus);
        
        // if(req.body.redeem_status == 'Approved' && req.body.customer_id){
            
            // var title = 'Hey, your redeem request status is Approved by the admin';
            // var description = 'Enjoy your gift';
            // var sendNotificationDataBody = {
            //     customer_id: req.body.customer_id ? req.body.customer_id : 0,
            //     title:  title ? title : '',
            //     description : description ? description : '',
            //     remark : description ? description : '',
            //     notification_type : 'redeem_request',
            // }
            
            // var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
            // console.log('sendNotificationData',sendNotificationData);
            
        // }
        
        
        
        if(updateRedeemStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Redeem Status Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Redeem Status not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateRedeemGiftStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.redeem_request_id = decrypt(req.body.redeem_request_id);
        
        var updateRedeemGiftStatus = await Offer.updateRedeemGiftStatus(req.body);
        // console.log('updateRedeemGiftStatus',updateRedeemGiftStatus);
        
        if(req.body.gift_status == 'Shipped' && req.body.customer_id){
            
            var title = 'Hey, your gift status is Shipped by the admin';
            var description = 'Enjoy your gift';
            var sendNotificationDataBody = {
                customer_id: req.body.customer_id ? req.body.customer_id : 0,
                title:  title ? title : '',
                description : description ? description : '',
                remark : description ? description : '',
                notification_type : 'redeem_request',
            }
            
            var sendNotificationData =  await Notification.sendNotificationData(sendNotificationDataBody);
            console.log('sendNotificationData',sendNotificationData);
            
        }
        
        
        if(updateRedeemGiftStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Redeem Status Updated",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Redeem Status not Updated",
                status: "failed",
            });
        }
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


function generateRandomString(n) {
    let randomString           = '';
    let characters       = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    
    for ( let i = 0; i < n; i++ ) {
        randomString += characters.charAt(Math.floor(Math.random()*characters.length));
    }
    return randomString;
}