const Order = require("../../models/web/orderModel");
const Network = require("../../models/web/networkModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');
const Invoice = require("../../models/web/invoiceModel");


exports.getDrListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getDrList =  await Order.getDrList(req.body);
        // console.log('getDrList',getDrList);
        
        res.status(200).json({
            message: "DR List",
            status: "success",
            result : {
                data: getDrList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getDrProductDiscountListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getDrProductDiscountListData =  await Order.getDrProductDiscountListData(req.body);
        // console.log('getDrProductDiscountListData',getDrProductDiscountListData);
        
        res.status(200).json({
            message: "DR Product Discount List",
            status: "success",
            result : {
                data: getDrProductDiscountListData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getProductListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getProductList =  await Order.getProductList(req.body);
        // console.log('getProductList',getProductList);
        
        res.status(200).json({
            message: "Product List",
            status: "success",
            result : {
                data: getProductList,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getRejectReasonListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getRejectReasonListData =  await Order.getRejectReasonListData(req.body);
        // console.log('getRejectReasonListData',getRejectReasonListData);
        
        res.status(200).json({
            message: "Reject Reason List",
            status: "success",
            result : {
                data: getRejectReasonListData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getSaleOrderNo = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getSaleOrderNo =  await Order.getSaleOrderNo(req.body);
        console.log('getSaleOrderNo',getSaleOrderNo.order_no);

        if(getSaleOrderNo.order_no != undefined){
            const result = getSaleOrderNo.order_no.split('-');
            var order_no = parseInt(result[2]) + 1;
            // console.log('order_no',order_no);        
            getSaleOrderNo.order_no = result[0].concat("-", result[1]).concat("-",order_no);
            // prefix add now
            getSaleOrderNo.order_no = req.body.sfa_web_user_prefix+'-ORDER-'+order_no;    
        }else{
            getSaleOrderNo.order_no = req.body.sfa_web_user_prefix+'-ORDER-0001';    
        }
     
        res.status(200).json({
            message: "Sale order number",
            status: "success",
            result : {
                data: getSaleOrderNo,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getOrderStatusType = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
                        
        var getOrderStatusType =  await Order.getOrderStatusType(req.body);
        // console.log('getOrderStatusType',getOrderStatusType);
        
        res.status(200).json({
            message: "Order Status Type",
            status: "success",
            result : {
                data: getOrderStatusType,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getOrderList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
           
        if(req.body.filter.encrypt_user_id){
            req.body.filter = {
                user_id : decrypt(req.body.filter.encrypt_user_id),
            }
        }
        
        var getOrderCount = await Order.getOrderCount(req.body)
        
        var getOrderList = [];
        if(getOrderCount > 0){
            getOrderList =  await Order.getOrderList(req.body);
            // console.log('getOrderList',getOrderList);
        }
        
        res.status(200).json({
            message: "Order List",
            status: "success",
            result : {
                data: getOrderList,
                totalCount : getOrderCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.addOrderData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.basicData.created_by_name = req.body.sfa_web_user_name;
        req.body.basicData.created_by = req.body.sfa_web_user_id;
        req.body.basicData.created_by_type = req.body.sfa_web_user_type;
        
        var addOrderData =  await Order.addOrderData(req.body.basicData);
        // console.log('addOrderData',addOrderData);
        
        if(addOrderData.insertId){
            var i = req.body.orderItem.length - 1;
            if(req.body.orderItem.length){
                req.body.orderItem.forEach(async element => {
                    element.order_id = addOrderData.insertId;
                    var addOrderItemData =  await Order.addOrderItemData(element);
                    // console.log('addOrderItemData',addOrderItemData);
                    if(i==0){
                        
                        var logData = {
                            customer_id : req.body.basicData.customer_id,
                            type : 'Order Added',
                            description : 'Details added'
                        }
                        var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
                        console.log('distributionNetworkLogData',distributionNetworkLogData);
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Order Data added",
                            status: "success",
                        });
                    }
                    i--; 
                });
            }else{
                await db.commit();
                res.status(200).json({
                    message: "Order Data added",
                    status: "success"
                });
            }
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateOrderData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.basicData.created_by_name = req.body.sfa_web_user_name;
        req.body.basicData.created_by = req.body.sfa_web_user_id;
        
        var updateOrderData =  await Order.updateOrderData(req.body.basicData);
        // console.log('updateOrderData',updateOrderData);
        
        if(updateOrderData.affectedRows){
            var i = req.body.orderItem.length - 1;
            if(req.body.orderItem.length){

                // delete all order items
                var deleteOrderItemData =  await Order.deleteOrderItemData(req.body.basicData);
                console.log('deleteOrderItemData',deleteOrderItemData);

                req.body.orderItem.forEach(async element => {
                    
                    element.order_id = req.body.basicData.id;

                    var addOrderItemData =  await Order.addOrderItemData(element);
                    // console.log('addOrderItemData',addOrderItemData);
                    if(i==0){
                        
                        var logData = {
                            customer_id : req.body.basicData.customer_id,
                            type : 'Order Updated',
                            description : 'Details Updated'
                        }
                        var distributionNetworkLogData = await Network.distributionNetworkLogData(logData);
                        console.log('distributionNetworkLogData',distributionNetworkLogData);
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Order Data Updated",
                            status: "success",
                        });
                    }
                    i--; 
                });
            }else{
                await db.commit();
                res.status(200).json({
                    message: "Order Data Updated",
                    status: "success"
                });
            }
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateOrderStatus = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var updateOrderStatus =  await Order.updateOrderStatus(req.body);
        // console.log('updateOrderStatus',updateOrderStatus);
        
        if(updateOrderStatus.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Order Status updated",
                status: "success",
            });
        }else{
            await db.rollback()
            res.status(200).json({
                message: "Order Status not updated",
                status: "error",
            });
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};

exports.getOrderDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.order_id = decrypt(req.body.order_id);
        
        var getOrderBasicDetail =  await Order.getOrderBasicDetail(req.body);
        // console.log('getOrderBasicDetail',getOrderBasicDetail);
        var getOrderItemDetail =  await Order.getOrderItemDetail(req.body);
        
        console.log('getOrderBasicDetail',getOrderBasicDetail.id);
        if(getOrderBasicDetail.id){
            var invoiceListData = {
                filter : {
                    order_id : getOrderBasicDetail.id
                },
                limit : 100,
                start : 0
            }
            var getInvoiceList =  await Invoice.getInvoiceList(invoiceListData);
        }
        
        
        await db.commit();
        res.status(200).json({
            message: "Order Detail",
            status: "success",
            result : {
                data : getOrderBasicDetail,
                orderItem : getOrderItemDetail,
                invoiceData : getInvoiceList
            }
        });
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.orderDelete = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // req.body.order_id = decrypt(req.body.order_id);
        
        var orderDelete =  await Order.orderDelete(req.body);
        console.log('orderDelete',orderDelete);
        
        if(orderDelete.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Order Deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Order not Deleted",
                status: "failed"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



