const Purchase = require("../../models/web/purchaseModel");
const Network = require("../../models/web/networkModel");
const Vendor = require("../../models/web/vendorModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');


exports.getVendorListData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getVendorListData =  await Purchase.getVendorListData(req.body);
        // console.log('getVendorListData',getVendorListData);
        
        res.status(200).json({
            message: "Vendor List",
            status: "success",
            result : {
                data: getVendorListData,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getPurchaseOrderNo = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        var getPurchaseOrderNo =  await Purchase.getPurchaseOrderNo(req.body);
        console.log('getPurchaseOrderNo',getPurchaseOrderNo);
        if(getPurchaseOrderNo.po_no != null){
            const result = getPurchaseOrderNo.po_no.split('-');
            var po_no = parseInt(result[2]) + 1;
            // console.log('po_no',po_no);        
            getPurchaseOrderNo.po_no = result[0].concat("-", result[1]).concat("-",po_no);
        }else{
            getPurchaseOrderNo.po_no = 'PO-ORDER-001';
        }
        
        
        res.status(200).json({
            message: "Purchase Order number",
            status: "success",
            result : {
                data: getPurchaseOrderNo,
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getPurchaseOrderList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getPurchaseOrderCount = await Purchase.getPurchaseOrderCount(req.body)
        
        var getPurchaseOrderList = [];
        if(getPurchaseOrderCount > 0){
            getPurchaseOrderList =  await Purchase.getPurchaseOrderList(req.body);
            // console.log('getPurchaseOrderList',getPurchaseOrderList);
        }
        
        res.status(200).json({
            message: "Purchase Order List",
            status: "success",
            result : {
                data: getPurchaseOrderList,
                totalCount : getPurchaseOrderCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getPurchaseOrderDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        req.body.po_id = decrypt(req.body.po_id);
        
        var getOrderBasicDetail =  await Purchase.getOrderBasicDetail(req.body);
        // console.log('getOrderBasicDetail',getOrderBasicDetail);
        var getOrderItemDetail =  await Purchase.getOrderItemDetail(req.body);
        
        
        await db.commit();
        res.status(200).json({
            message: "Purchase Order Detail",
            status: "success",
            result : {
                data : getOrderBasicDetail,
                orderItem : getOrderItemDetail
            }
        });
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deletePurchaseOrder = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        req.body.po_id = decrypt(req.body.po_id);
        
        var deletePoData =  await Purchase.deletePoData(req.body);
        // console.log('deletePoData',deletePoData);
        var deletePoDataItem =  await Purchase.deletePoDataItem(req.body);
        console.log('deletePoDataItem',deletePoDataItem);

        if(deletePoData.affectedRows && deletePoDataItem.affectedRows){
            
            var getOrderBasicDetail = await Purchase.getOrderBasicDetail(req.body);
            console.log('getOrderBasicDetail',getOrderBasicDetail);

            var getOrderItemDetail =  await Purchase.getOrderItemDetail(req.body);
            console.log('getOrderItemDetail',getOrderItemDetail);
  
            var i = getOrderItemDetail.length - 1;

            if(getOrderItemDetail.length){
                getOrderItemDetail.forEach(async element => {
                    // element.po_id = addPurchaseOrderData.insertId;
                    
                    var stockData = {
                        product_id : element.product_id ? element.product_id : 0,
                        vendor_id : element.customer_id ? element.customer_id : 0,
                        stock : element.qty ? element.qty : 0,
                        po_no : getOrderBasicDetail.po_no ? getOrderBasicDetail.po_no : null,
                        po_id : element.po_id ? element.po_id : 0,
                        type : 'stockout'
                    };
                    console.log('stockData',stockData);
                    var stockLedgerMaintaindata = await Network.stockLedgerMaintaindata(stockData);                        
                    // console.log('stockLedgerMaintaindata',stockLedgerMaintaindata);
                    
                    if(i==0){
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Purchase Order Data Deleted",
                            status: "success",
                        });

                    }
                    i--; 
                });
            }else{
                await db.rollback();
                res.status(200).json({
                    message: "Purchase item not found",
                    status: "error"
                });
            }
        }else{
            await db.rollback();
            res.status(200).json({
                message : 'Purchase Data not Deleted',
                status: "error",
            });   
        }
        
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.addPurchaseOrderData = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        
        // console.log('body ==>', req.body);
        
        var addPurchaseOrderData =  await Purchase.addPurchaseOrderData(req.body);
        console.log('addPurchaseOrderData',addPurchaseOrderData);
        
        
        if(addPurchaseOrderData.insertId){
            
            var i = req.body.orderItem.length - 1;
            if(req.body.orderItem.length){
                req.body.orderItem.forEach(async element => {
                    element.po_id = addPurchaseOrderData.insertId;
                    var addPurchaseOrderItemData =  await Purchase.addPurchaseOrderItemData(element);
                    // console.log('addPurchaseOrderItemData',addPurchaseOrderItemData);
                    
                    var stockData = {
                        product_id : element.product_id ? element.product_id : 0,
                        vendor_id : req.body.vendor_id ? req.body.vendor_id : 0,
                        stock : element.order_qty ? element.order_qty : 0,
                        po_no : req.body.po_no ? req.body.po_no : 0,
                        po_id : element.po_id ? element.po_id : 0,
                        type : 'stockin'
                    };
                    
                    var stockLedgerMaintaindata = await Network.stockLedgerMaintaindata(stockData);                        
                    // console.log('stockLedgerMaintaindata',stockLedgerMaintaindata);
                    
                    if(i==0){
                        
                        var logData = {
                            vendor_id : req.body.vendor_id,
                            type : 'Purchase order Added',
                            description : 'Purchase order added'
                        }
                        var vendorLogData = await Vendor.addVendorLogData(logData);
                        // console.log('vendorLogData',vendorLogData);
                        
                        await db.commit();
                        res.status(200).json({
                            message: "Purchase Order Created",
                            status: "success",
                        });
                    }
                    i--; 
                });
            }else{
                await db.rollback();
                res.status(200).json({
                    message: "Purchase Order item empty",
                    status: "error"
                });
            }
            
        }else{
            await db.rollback()
            res.status(200).json({
                message: "Purchase Order not Created",
                status: "error"
            });
        }
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};
