const Vendor = require("../../models/web/vendorModel");
const Purchase = require("../../models/web/purchaseModel");
const { check, validationResult, matchedData, oneOf, body } = require('express-validator');
const https = require('https');
var path = require("path");
var db = require('../../config/db');
var moment = require('moment');
const { encrypt, decrypt, encrypter, my_simple_crypt } = require('../../config/common');


exports.getVendorList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  
        var getVendorCount = await Vendor.getVendorCount(req.body)
        
        var getVendorList = [];
        if(getVendorCount > 0){
            getVendorList =  await Vendor.getVendorList(req.body);
            // console.log('getVendorList',getVendorList);
        }
        
        res.status(200).json({
            message: "Vendor List",
            status: "success",
            result : {
                data: getVendorList,
                totalCount : getVendorCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.addVendor = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        var addVendorData =  await Vendor.addVendorData(req.body.basicDetail);
        console.log('addVendorData',addVendorData);
        
        if(addVendorData.insertId){
            
            if(req.body.contactDetail.length){
                req.body.vendor_id = addVendorData.insertId;
                var addVendorContactData =  await Vendor.addVendorContactData(req.body);
                console.log('addVendorContactData',addVendorContactData);    
            }
            
            var logData = {
                vendor_id : addVendorData.insertId,
                type : 'Details Added',
                description : 'Details added'
            }
            var vendorLogData = await Vendor.addVendorLogData(logData);
            // console.log('vendorLogData',vendorLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Vendor network added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Vendor network not added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.addVendorContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          

        req.body.vendor_id = decrypt(req.body.vendor_id);
        
        if(req.body.contactDetail.length){
            var vendorContactData =  await Vendor.addVendorContactData(req.body);
            console.log('vendorContactData',vendorContactData);    
        }
        
        if(vendorContactData.insertId){
            
            var logData = {
                vendor_id : req.body.vendor_id,
                type : 'Contact Details Added',
                description : 'Contact Details Added'
            }
            var vendorLogData = await Vendor.addVendorLogData(logData);
            console.log('vendorLogData',vendorLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Added",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Added",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.updateVendorBasicDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        req.body.basicDetail.vendor_id = req.body.vendor_id;
        var updateVendorData =  await Vendor.updateVendorData(req.body.basicDetail);
        console.log('updateVendorData',updateVendorData);
        
        if(updateVendorData.affectedRows){
            
            var logData = {
                vendor_id : req.body.vendor_id,
                type : 'Basic Details Updated',
                description : 'Basic Details updated'
            }
            var distributionNetworkLogData = await Vendor.addVendorLogData(logData);
            console.log('distributionNetworkLogData',distributionNetworkLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Basic Details Updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Basic Details not Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.updateVendorContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        var updateVendorContactData =  await Vendor.updateVendorContactData(req.body);
        console.log('updateVendorContactData',updateVendorContactData);
        
        if(updateVendorContactData.affectedRows){
            
            var logData = {
                vendor_id : req.body.vendor_id,
                type : 'Contact Details Updated',
                description : 'Contact Details updated'
            }
            var vendorLogData = await Vendor.addVendorLogData(logData);
            console.log('vendorLogData',vendorLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Updated",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Updated",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.deleteVendorContactDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        var deleteVendorContactDetail =  await Vendor.deleteVendorContactDetail(req.body);
        console.log('deleteVendorContactDetail',deleteVendorContactDetail);
        
        if(deleteVendorContactDetail.affectedRows){
            
            var logData = {
                vendor_id : req.body.vendor_id,
                type : 'Contact Details Deleted',
                description : 'Contact Details Deleted'
            }
            var vendorLogData = await Vendor.addVendorLogData(logData);
            console.log('vendorLogData',vendorLogData);
            
            await db.commit();
            res.status(200).json({
                message: "Contact Details Deleted",
                status: "success"
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Contact Details not Deleted",
                status: "error"
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.getVendorDetail = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        req.body.vendor_id = decrypt(req.body.vendor_id);
        
        var getBasicDetail = await Vendor.getVendorBasicDetail(req.body);
        // console.log('getBasicDetail',getBasicDetail);
        
        var getContactDetail = await Vendor.getVendorContactDetail(req.body);
        // console.log('getContactDetail',getContactDetail);
        
        var getLogDetail = await Vendor.getVendorLogDetail(req.body);
        // console.log('getLogDetail',getLogDetail);
        
        res.status(200).json({
            message: "Vendor detail",
            status: "success",
            data :{
                basicDetail : getBasicDetail,
                contactDetail : getContactDetail,
                logDetail : getLogDetail
            } 
        });
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};



exports.vendorDelete = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();          
        
        // req.body.vendor_id = decrypt(req.body.vendor_id);
        
        var vendorDelete = await Vendor.vendorDelete(req.body);
        // console.log('vendorDelete',vendorDelete);
        
        if(vendorDelete.affectedRows){
            await db.commit();
            res.status(200).json({
                message: "Vendor data deleted",
                status: "success",
            });
        }else{
            await db.rollback();
            res.status(200).json({
                message: "Vendor data not deleted",
                status: "failed",
            });
        }
        
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};


exports.getVendorPoList = async function (req, res) {
    
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
        const x = matchedData(req);
        return res.send({ message: 'Invalid Values', status: 'invalid', err: errors.mapped() });
    }
    
    try {
        await db.beginTransaction();  

        req.body.vendor_id = decrypt(req.body.vendor_id);

        req.body.filter = {
            vendor_id : req.body.vendor_id
        }

        var getPurchaseOrderCount = await Purchase.getPurchaseOrderCount(req.body)
        
        var getPurchaseOrderList = [];
        if(getPurchaseOrderCount > 0){
            getPurchaseOrderList =  await Purchase.getPurchaseOrderList(req.body);
            // console.log('getPurchaseOrderList',getPurchaseOrderList);
        }
        
        res.status(200).json({
            message: "Vendor Purchase Order List",
            status: "success",
            result : {
                data: getPurchaseOrderList,
                totalCount : getPurchaseOrderCount
            }
            
        });
        
    } catch (e) {
        console.log('e', e);
        await db.rollback()
        var err = {
            code: e.code,
            sqlMessage: e.sqlMessage
        };
        res.status(200).json({
            message: "Error Occured",
            status: "error",
            err :err
        });
        return false;
    }
};